/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "merc.h"
#include "color.h"
#include "magic.h"
#include "recycle.h"
#include "tables.h"
#include "lookup.h"
#include "vt100.h"


#define WHO_NAME_LENGTH		13

/* command procedures needed */
DECLARE_DO_FUN(	do_exits	);
DECLARE_DO_FUN( do_look		);
DECLARE_DO_FUN( do_help		);
DECLARE_DO_FUN( do_affects	);
DECLARE_DO_FUN( do_play		);



char *	const	where_name	[] =
{
    "<used as light>               ",
    "<worn on head>                ",
    "<worn in hair>                ",
    "<worn on face>                ",
    "<worn in ears>                ",
    "<worn around neck>            ",
    "<worn around neck>            ",
    "<worn on shoulder>            ",
    "<worn about body>             ",
    "<worn on chest>               ",
    "<lodged in a rib>             ",
    "<worn about waist>            ",
    "<worn on arms>                ",
    "<lodged in an arm>            ",
    "<worn around wrist>           ",
    "<worn around wrist>           ",
    "<worn on hands>               ",
    "<worn on finger>              ",
    "<worn on finger>              ",
    "<worn as shield>              ",
    "<wielded>                     ",
    "<secondary weapon>            ",
    "<held>                        ",
    "<worn on legs>                ",
    "<lodged in a leg>             ",
    "<worn on feet>                ",
    "<floating nearby>             "
};


/*
 * Static variables.
 */
static int max_on	= 0;			/* for do_count */
static bool examining	= FALSE;




/*
 * Local functions.
 */
char *	format_obj_to_char	args( ( OBJ_DATA *obj, CHAR_DATA *ch,
				    bool fShort ) );
void	show_list_to_char	args( ( OBJ_DATA *list, CHAR_DATA *ch,
				    bool fShort, bool fShowNothing ) );
void	show_char_to_char_0	args( ( CHAR_DATA *victim, CHAR_DATA *ch ) );
void	show_char_to_char_1	args( ( CHAR_DATA *victim, CHAR_DATA *ch ) );
void	show_char_to_char	args( ( CHAR_DATA *list, CHAR_DATA *ch ) );
bool	check_blind		args( ( CHAR_DATA *ch ) );
int	avg_dam			args( ( CHAR_DATA *ch ) );
char *	print_appearance	args( ( CHAR_DATA *ch, CHAR_DATA *viewer ) );
char *	print_position		args( ( CHAR_DATA *ch, CHAR_DATA *viewer,
					bool punctuate ) );



char *format_obj_to_char( OBJ_DATA *obj, CHAR_DATA *ch, bool fShort )
{
    static char buf[MAX_STRING_LENGTH];

    buf[0] = '\0';

    if ((fShort && (obj->short_descr == NULL || obj->short_descr[0] == '\0'))
    ||  (obj->description == NULL || obj->description[0] == '\0'))
	return buf;

    if ( IS_AFFECTED2(ch, AFF2_HALLUCINATION) )
        obj = get_random_obj( ch );

    if ( IS_OBJ_STAT(obj, ITEM_INVIS) )		strcat( buf, "(Invis) " );
    if ( IS_AFFECTED(ch, AFF_DETECT_EVIL)
         && IS_OBJ_STAT(obj, ITEM_EVIL) )	strcat( buf, "(Red Aura) " );
    if ( IS_AFFECTED(ch, AFF_DETECT_GOOD)
    &&   IS_OBJ_STAT(obj,ITEM_BLESS) )		strcat( buf, "(Blue Aura) " );
    if ( IS_AFFECTED(ch, AFF_DETECT_MAGIC)
    &&   IS_OBJ_STAT(obj, ITEM_MAGIC)
    &&   !IS_OBJ_STAT(obj, ITEM_DISPELLED) )	strcat( buf, "(Magical) " );
    if ( IS_OBJ_STAT(obj, ITEM_GLOW) )		strcat( buf, "(Glowing) " );
    if ( IS_OBJ_STAT(obj, ITEM_HUM) )		strcat( buf, "(Humming) " );
    if ( IS_AFFECTED(ch, AFF_DETECT_TRAPS)
    &&   is_trapped(obj) )			strcat( buf, "(Trapped) " );

    if ( fShort )
    {
	if ( obj->short_descr != NULL )
	    strcat( buf, obj->short_descr );
    }
    else
    {
	if ( obj->description != NULL)
	{
	    strcat( buf, P_COLOR( ch, C_OBJECTS ) );
	    strcat( buf, obj->description );
	    strcat( buf, P_COLOR_LITERAL( ch, COLOR_OFF ) );
	}
    }

    return buf;
}



/*
 * Show a list to a character.
 * Can coalesce duplicated items.
 */
void show_list_to_char( OBJ_DATA *list, CHAR_DATA *ch, bool fShort, bool fShowNothing )
{
    char buf[MAX_STRING_LENGTH];
    BUFFER *output;
    char **prgpstrShow;
    int *prgnShow;
    char *pstrShow;
    OBJ_DATA *obj;
    int nShow;
    int iShow;
    int count;
    bool fCombine;

    if ( ch->desc == NULL )
	return;

    /*
     * Alloc space for output lines.
     */
    output = new_buf();

    count = 0;
    for ( obj = list; obj != NULL; obj = obj->next_content )
	count++;
    prgpstrShow	= alloc_mem( count * sizeof(char *) );
    prgnShow    = alloc_mem( count * sizeof(int)    );
    nShow	= 0;

    /*
     * Format the list of objects.
     */
    for ( obj = list; obj != NULL; obj = obj->next_content )
    {
	if ( obj->wear_loc == WEAR_NONE
	&&   can_see_obj(ch, obj) )
	{
	    pstrShow = format_obj_to_char( obj, ch, fShort );

	    fCombine = FALSE;

	    if ( IS_NPC(ch) || IS_SET(ch->comm, COMM_COMBINE) )
	    {
		/*
		 * Look for duplicates, case sensitive.
		 * Matches tend to be near end so run loop backwords.
		 */
		for ( iShow = nShow - 1; iShow >= 0; iShow-- )
		{
		    if ( !strcmp( prgpstrShow[iShow], pstrShow ) )
		    {
			prgnShow[iShow]++;
			fCombine = TRUE;
			break;
		    }
		}
	    }

	    /*
	     * Couldn't combine, or didn't want to.
	     */
	    if ( !fCombine )
	    {
		prgpstrShow [nShow] = str_dup( pstrShow );
		prgnShow    [nShow] = 1;
		nShow++;
	    }
	}
    }

    /*
     * Output the formatted list.
     */
    for ( iShow = 0; iShow < nShow; iShow++ )
    {
	if (prgpstrShow[iShow][0] == '\0')
	{
	    free_string(prgpstrShow[iShow]);
	    continue;
	}

	if ( IS_NPC(ch) || IS_SET(ch->comm, COMM_COMBINE) )
	{
	    if ( prgnShow[iShow] != 1 )
	    {
		sprintf( buf, "(%2d) ", prgnShow[iShow] );
		add_buf(output,buf);
	    }
	    else
	    {
		add_buf(output,"     ");
	    }
	}
	add_buf(output,prgpstrShow[iShow]);
	add_buf(output,"\n\r");
	free_string( prgpstrShow[iShow] );
    }

    if ( fShowNothing && nShow == 0 )
    {
	if ( IS_NPC(ch) || IS_SET(ch->comm, COMM_COMBINE) )
	    send_to_char( "     ", ch );
	send_to_char( "Nothing.\n\r", ch );
    }
    page_to_char(buf_string(output),ch);

    /*
     * Clean up.
     */
    free_buf(output);
    free_mem( prgpstrShow, count * sizeof(char *) );
    free_mem( prgnShow,    count * sizeof(int)    );

    return;
}



void show_char_to_char_0( CHAR_DATA *victim, CHAR_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];

    if ( (IS_NPC(victim)
    &&    victim->polymorphed == NULL
    &&    victim->position == victim->start_pos
    &&    victim->appearance[0] == '\0'
    &&    victim->long_descr[0] == '\0')
    ||   victim->mounted_by != NULL )
	return;

    if ( IS_AFFECTED2(ch, AFF2_HALLUCINATION) )
        victim = get_random_npc( ch );

    buf[0] = '\0';
    if ( IS_NPC(victim) )
	strcat( buf, "   " );

    if ( IS_SET(victim->comm, COMM_AFK) )
	strcat( buf, "[AFK] " );
    if ( IS_AFFECTED(victim, AFF_INVISIBLE)
    ||   IS_AFFECTED2(victim, AFF2_IMPROVED_INVIS) )
	strcat( buf, "(Invis) " );
    if ( victim->invis_level >= LEVEL_HERO )
	strcat( buf, "(Wizi) " );
    if ( IS_AFFECTED(victim, AFF_HIDE) )
	strcat( buf, "(Hide) " );
    if ( IS_AFFECTED(victim, AFF_CHARM) )
	strcat( buf, "(Charmed) " );
    if ( IS_AFFECTED(victim, AFF_PASS_DOOR) )
	strcat( buf, "(Translucent) " );
    if ( IS_AFFECTED(victim, AFF_FAERIE_FIRE) )
	strcat( buf, "(Pink Aura) " );
    if ( IS_EVIL(victim)
    &&   IS_AFFECTED(ch, AFF_DETECT_EVIL)
    &&   !IS_AFFECTED2(victim, AFF2_DELUDE) )
	strcat( buf, "(Red Aura) " );
    if ( IS_GOOD(victim)
    &&   IS_AFFECTED(ch, AFF_DETECT_GOOD)
    &&   !IS_AFFECTED2(victim, AFF2_DELUDE) )
	strcat( buf, "(Golden Aura) " );
    if ( IS_AFFECTED(victim, AFF_SANCTUARY) )
	strcat( buf, "(White Aura) " );
/*
 *   if ( !IS_NPC(victim) && IS_SET(victim->act, PLR_KILLER) )
 *	strcat( buf, "(KILLER) " );
 *   if ( !IS_NPC(victim) && IS_SET(victim->act, PLR_THIEF) )
 *	strcat( buf, "(THIEF) " );
 */

    if ( victim->position == victim->start_pos
    &&   victim->riding == NULL
    &&   victim->long_descr[0] != '\0'
    &&   victim->appearance[0] == '\0'
    &&   victim->polymorphed == NULL )
    {
	if( IS_NPC( victim ) )
		strcat( buf, P_COLOR( ch, C_MOBILES ) );
	else
		strcat( buf, P_COLOR( ch, C_PLAYERS ) );
	strcat( buf, victim->long_descr );
	strcat( buf, P_COLOR_LITERAL( ch, COLOR_OFF ) );
    }
    else
    {
	if( IS_NPC( victim ) )
	    strcat( buf, P_COLOR( ch, C_MOBILES ) );
	else
	    strcat( buf, P_COLOR( ch, C_PLAYERS ) );

	strcat( buf, print_appearance(victim, ch) );
/*
 *  strcat( buf, PERS( victim, ch ) );
 *  if ( !IS_NPC(victim) && !IS_SET(ch->comm, COMM_BRIEF)
 *  &&   victim->position == POS_STANDING && ch->on == NULL )
 *	strcat( buf, victim->pcdata->title );
 *
 *  switch ( victim->position )
 *  {
 *  case POS_DEAD:     strcat( buf, " is DEAD!!" );              break;
 *  case POS_MORTAL:   strcat( buf, " is mortally wounded." );   break;
 *  case POS_INCAP:    strcat( buf, " is incapacitated." );      break;
 *  case POS_STUNNED:  strcat( buf, " is lying here stunned." ); break;
 *  case POS_SLEEPING:
 *	if (victim->on != NULL)
 *	{
 *	    if (IS_SET(victim->on->value[2],SLEEP_AT))
 *	    {
 *		sprintf(message," is sleeping at %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else if (IS_SET(victim->on->value[2],SLEEP_ON))
 *	    {
 *		sprintf(message," is sleeping on %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else
 *	    {
 *		sprintf(message, " is sleeping in %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	}
 *	else
 *	    strcat(buf," is sleeping here.");
 *	break;
 *  case POS_RESTING:
 *	if (victim->on != NULL)
 *	{
 *	    if (IS_SET(victim->on->value[2],REST_AT))
 *	    {
 *		sprintf(message," is resting at %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else if (IS_SET(victim->on->value[2],REST_ON))
 *	    {
 *		sprintf(message," is resting on %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else
 *	    {
 *		sprintf(message, " is resting in %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	}
 *	else
 *	    strcat( buf, " is resting here." );
 *	break;
 *  case POS_SITTING:
 *	if (victim->on != NULL)
 *	{
 *	    if (IS_SET(victim->on->value[2],SIT_AT))
 *	    {
 *		sprintf(message," is sitting at %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else if (IS_SET(victim->on->value[2],SIT_ON))
 *	    {
 *		sprintf(message," is sitting on %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else
 *	    {
 *		sprintf(message, " is sitting in %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	}
 *	else
 *	    strcat(buf, " is sitting here.");
 *	break;
 *  case POS_STANDING:
 *	if (victim->on != NULL)
 *	{
 *	    if (IS_SET(victim->on->value[2],STAND_AT))
 *	    {
 *		sprintf(message," is standing at %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else if (IS_SET(victim->on->value[2],STAND_ON))
 *	    {
 *		sprintf(message," is standing on %s.",
 *		   victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	    else
 *	    {
 *		sprintf(message," is standing in %s.",
 *		    victim->on->short_descr);
 *		strcat(buf,message);
 *	    }
 *	}
 *	else
 *	    strcat( buf, " is here." );
 *	break;
 *  case POS_FIGHTING:
 *	strcat( buf, " is here, fighting " );
 *	if ( victim->fighting == NULL )
 *	    strcat( buf, "thin air??" );
 *	else if ( victim->fighting == ch )
 *	    strcat( buf, "YOU!" );
 *	else if ( victim->in_room == victim->fighting->in_room )
 *	{
 *	    strcat( buf, PERS( victim->fighting, ch ) );
 *	    strcat( buf, "." );
 *	}
 *	else
 *	    strcat( buf, "someone who left??" );
 *	break;
 *  }
 */

	strcat( buf, P_COLOR_LITERAL( ch, COLOR_OFF ) );
	strcat( buf, "\n\r" );
    }
    if ( IS_AFFECTED2(victim, AFF2_MIRROR_IMAGE)
    &&   !IS_AFFECTED2(ch, AFF2_DETECT_ILLUSION) )
    {
	AFFECT_DATA *paf = affect_find( victim->affected, gsn_mirror_image );

    	if ( paf == NULL )
    	{
	    bug( "show_char_to_char_0: victim (%s) has mirror image flag but no affect.",
	        (long) PERS_TRUE(victim) );
    	    send_to_char( buf, ch );
    	}
	else
    	{
    	    int i;
    	    for ( i = 0; i < paf->modifier; i++ )
		send_to_char( buf, ch );
    	}
    }
    else
        send_to_char( buf, ch );
    return;
}



void show_char_to_char_1( CHAR_DATA *victim, CHAR_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *obj;
    int iWear;
    int percent;
    bool found;

    if ( can_see( victim, ch ) )
    {
	if (ch == victim)
	    act( "$n looks at $mself.",ch,NULL,NULL,TO_ROOM);
	else
	{
	    act( "$n looks at you.", ch, NULL, victim, TO_VICT    );
	    act( "$n looks at $N.",  ch, NULL, victim, TO_NOTVICT );
	}
    }

    if ( CHAR_DESCRIPTION(victim)[0] != '\0' )
    {
        if ( IS_AFFECTED2(ch, AFF2_HALLUCINATION) )
	    send_to_char( get_random_npc(ch)->description, ch );
        else
            send_to_char( CHAR_DESCRIPTION(victim), ch );
    }
    else
    {
	act( "You see nothing special about $M.", ch, NULL, victim, TO_CHAR );
    }

    if ( victim->max_hit > 0 )
	percent = ( 100 * victim->hit ) / victim->max_hit;
    else
	percent = -1;

    if ( IS_AFFECTED2(ch, AFF2_HALLUCINATION) )
        strcpy( buf, PERS(get_random_npc(ch), ch) );
    else
        strcpy( buf, PERS(victim, ch) );

    if (percent >= 100)
	strcat( buf, " appears to be in excellent condition.\n\r");
    else if (percent >= 90)
	strcat( buf, " has a few scratches.\n\r");
    else if (percent >= 75)
	strcat( buf," has some small wounds and bruises.\n\r");
    else if (percent >=  50)
	strcat( buf, " has quite a few wounds.\n\r");
    else if (percent >= 30)
	strcat( buf, " has some big nasty wounds and scratches.\n\r");
    else if (percent >= 15)
	strcat ( buf, " looks pretty hurt.\n\r");
    else if (percent >= 0 )
	strcat (buf, " is in awful condition.\n\r");
    else
	strcat(buf, " is bleeding to death.\n\r");

    buf[0] = UPPER(buf[0]);
    send_to_char( buf, ch );

    found = FALSE;
    for ( iWear = 0; iWear < MAX_WEAR; iWear++ )
    {
	if ( ( obj = get_eq_char( victim, iWear ) ) != NULL
	&&   can_see_obj( ch, obj ) )
	{
	    char *wear_loc;

	    if ( !found )
	    {
		send_to_char( "\n\r", ch );
		act( "$N is using:", ch, NULL, victim, TO_CHAR );
		found = TRUE;
	    }

	    if ( (wear_loc = find_use_msg(obj, USE_WEAR_LOC)) != NULL )
	    {
	    	sprintf( buf, "%-30s", wear_loc );
	    	send_to_char( buf, ch );
	    }
	    else
	    	send_to_char( where_name[iWear], ch );
	    send_to_char( format_obj_to_char( obj, ch, TRUE ), ch );
	    send_to_char( "\n\r", ch );
	}
    }

    if ( victim != ch
    &&   !IS_NPC(ch)
    &&   (number_percent( ) < get_skill(ch, gsn_peek)
    ||    victim == ch->riding) )
    {
	send_to_char( "\n\rYou peek at the inventory:\n\r", ch );
	check_improve(ch,gsn_peek,TRUE,4,TRUE);
	show_list_to_char( victim->carrying, ch, TRUE, TRUE );
    }

    return;
}



void show_char_to_char( CHAR_DATA *list, CHAR_DATA *ch )
{
    CHAR_DATA *rch;
    bool cloud;

    cloud = FALSE;

    for ( rch = list; rch != NULL; rch = rch->next_in_room )
    {
	if ( rch == ch )
	    continue;

	if ( get_trust(ch) < rch->invis_level)
	    continue;

	if ( can_see( ch, rch ) )
	{
	    show_char_to_char_0( rch, ch );
	}
	else if ( IS_AFFECTED2(rch, AFF2_DARKNESS_CLOUD) )
	{
	    if ( !cloud )
	    {
		cloud = TRUE;
		act_color(
		    "   A billowing cloud of darkness swirls menacingly about.",
		    ch, NULL, NULL, TO_CHAR, POS_RESTING, C_MOBILES );
	    }
	}
	else if ( room_is_dark( ch, ch->in_room )
	&&        IS_AFFECTED(rch, AFF_INFRARED ) )
	{
	    send_to_char( "You see glowing red eyes watching YOU!\n\r", ch );
	}
    }

    return;
}



bool check_blind( CHAR_DATA *ch )
{

    if ( has_holylight(ch) )
	return TRUE;

    if ( IS_AFFECTED(ch, AFF_BLIND) )
    {
	send_to_char( "You can't see a thing!\n\r", ch );
	return FALSE;
    }

    return TRUE;
}


int avg_dam( CHAR_DATA *ch )
{
    OBJ_DATA *wield;
    int dam;

    wield = get_eq_char( ch, WEAR_WIELD );

    if ( IS_NPC(ch) && (!ch->pIndexData->new_format || wield == NULL) )
    {
	if ( !ch->pIndexData->new_format )
	{
	    dam = ch->level;
	    if ( wield != NULL )
	    	dam += dam / 2;
	}
	else
	    dam = ch->damage[DICE_NUMBER] * (ch->damage[DICE_TYPE] + 1) / 2;
    }
    else
    {
	if ( wield != NULL )
	{
	    if ( wield->pIndexData->new_format )
		dam = wield->value[1] * (wield->value[2] + 1) / 2;
	    else
		dam = (wield->value[1] + wield->value[2]) / 2;
	}
	else
	    dam = (5 + 2*ch->level/3) / 2;
    }

    return dam + ch->damroll;
}


/* changes your scroll */
void do_scroll(CHAR_DATA *ch, char *argument)
{
    char arg[MAX_INPUT_LENGTH];
    char buf[100];
    int lines;

    one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	if (ch->lines == 0)
	    send_to_char("You do not page long messages.\n\r",ch);
	else
	{
	    sprintf(buf,"You currently display %d lines per page.\n\r",
		    ch->lines + 2);
	    send_to_char(buf,ch);
	}
	return;
    }

    if (!is_number(arg))
    {
	send_to_char("You must provide a number.\n\r",ch);
	return;
    }

    lines = atoi(arg);

    if (lines == 0)
    {
        send_to_char("Paging disabled.\n\r",ch);
        ch->lines = 0;
        return;
    }

    if (lines < 10 || lines > 100)
    {
	send_to_char("You must provide a reasonable number.\n\r",ch);
	return;
    }

    sprintf(buf,"Scroll set to %d lines.\n\r",lines);
    send_to_char(buf,ch);
    ch->lines = lines - 2;
}

/* RT does socials */
void do_socials(CHAR_DATA *ch, char *argument)
{
    char buf[MAX_STRING_LENGTH];
    int iSocial;
    int col;

    col = 0;

    for (iSocial = 0; social_table[iSocial].name[0] != '\0'; iSocial++)
    {
	sprintf(buf,"%-12s",social_table[iSocial].name);
	send_to_char(buf,ch);
	if (++col % 6 == 0)
	    send_to_char("\n\r",ch);
    }

    if ( col % 6 != 0)
	send_to_char("\n\r",ch);
    return;
}



/* RT Commands to replace news, motd, imotd, etc from ROM */

void do_motd(CHAR_DATA *ch, char *argument)
{
    do_help(ch,"motd");
}

void do_imotd(CHAR_DATA *ch, char *argument)
{
    do_help(ch,"imotd");
}

void do_rules(CHAR_DATA *ch, char *argument)
{
    do_help(ch,"rules");
}

void do_story(CHAR_DATA *ch, char *argument)
{
    do_help(ch,"story");
}

void do_wizlist(CHAR_DATA *ch, char *argument)
{
    do_help(ch,"wizlist");
}

// This replaces autolist and channels, making one handy command. -Bri, 4 Sep 99

void do_toggle(CHAR_DATA *ch, char *argument)
{
    char buf[MAX_STRING_LENGTH];

    if (IS_NPC(ch))
	return;

    send_to_char("   channel status                        action status\n\r", ch);
    send_to_char("---------------------                ---------------------\n\r", ch);
    send_to_char("gossip         ", ch);
    if (!IS_SET(ch->comm,COMM_NOGOSSIP))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("autoassist     ", ch);
    if (IS_SET(ch->act,PLR_AUTOASSIST))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("auction        ", ch);
    if (!IS_SET(ch->comm,COMM_NOAUCTION))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("autoexit       ", ch);
    if (IS_SET(ch->act,PLR_AUTOEXIT))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("music          ", ch);
    if (!IS_SET(ch->comm,COMM_NOMUSIC))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("autogold       ", ch);
    if (IS_SET(ch->act,PLR_AUTOGOLD))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("Q/A            ", ch);
    if (!IS_SET(ch->comm,COMM_NOQUESTION))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("autoloot       ", ch);
    if (IS_SET(ch->act,PLR_AUTOLOOT))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("quote          ", ch);
    if (!IS_SET(ch->comm,COMM_NOQUOTE))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("autosac        ", ch);
    if (IS_SET(ch->act,PLR_AUTOSAC))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("grats          ", ch);
    if (!IS_SET(ch->comm,COMM_NOGRATS))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("autosplit      ", ch);
    if (IS_SET(ch->act,PLR_AUTOSPLIT))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("religion       ", ch);
    if (!IS_SET(ch->comm,COMM_NORELIGION))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("compact mode   ", ch);
    if (IS_SET(ch->comm,COMM_COMPACT))
        send_to_char("ON\n\r", ch);
    else
        send_to_char("OFF\n\r", ch);

    send_to_char("orate          ",ch);
    if (!IS_SET(ch->comm,COMM_NOORATE))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("prompt         ", ch);
    if (IS_SET(ch->comm,COMM_PROMPT))
	send_to_char("ON\n\r", ch);
    else
	send_to_char("OFF\n\r", ch);

    send_to_char("shouts         ", ch);
    if (!IS_SET(ch->comm,COMM_SHOUTSOFF))
	send_to_char("ON                    ", ch);
    else
	send_to_char("OFF                   ", ch);
    send_to_char("combine items  ", ch);
    if (IS_SET(ch->comm,COMM_COMBINE))
	send_to_char("ON\n\r", ch);
    else
	send_to_char("OFF\n\r", ch);

    if (get_olc_level(ch) > 1)
    {
      send_to_char("creators       ",ch);
      if(!IS_SET(ch->comm,COMM_NOCRETALK))
        send_to_char("ON\n\r",ch);
      else
        send_to_char("OFF\n\r",ch);
    }

    if (get_trust(ch) >= SUPREME)
    {
      send_to_char("imp channel    ",ch);
      if(!IS_SET(ch->comm,COMM_NOIMP))
        send_to_char("ON\n\r",ch);
      else
        send_to_char("OFF\n\r",ch);
    }

    send_to_char("tells          ",ch);
    if (!IS_SET(ch->comm,COMM_DEAF))
        send_to_char("ON\n\r",ch);
    else
        send_to_char("OFF\n\r",ch);

    send_to_char("tell priority  ",ch);
    if (IS_SET(ch->comm,COMM_PRIORITY))        send_to_char("ON\n\r",ch);
    else
        send_to_char("OFF\n\r",ch);

    send_to_char("quiet mode     ",ch);
    if (IS_SET(ch->comm,COMM_QUIET))
      send_to_char("ON\n\r",ch);
    else
      send_to_char("OFF\n\r",ch);

    if (IS_SET(ch->comm,COMM_AFK))
        send_to_char("You are AFK.\n\r",ch);

    if (IS_SET(ch->comm,COMM_SNOOP_PROOF))
        send_to_char("You are immune to snooping.\n\r",ch);

    if (ch->lines != PAGELEN)
    {
        if (ch->lines)
        {
            sprintf(buf,"You display %d lines of scroll.\n\r",ch->lines+2);
            send_to_char(buf,ch);
        }
        else
            send_to_char("Scroll buffering is off.\n\r",ch);
    }

    if (ch->prompt != NULL)
    {
        sprintf(buf,"Your current prompt is: %s\n\r",ch->prompt);
        send_to_char(buf,ch);
    }

    if (IS_SET(ch->comm,COMM_NOSHOUT))
      send_to_char("You cannot shout.\n\r",ch);

    if (IS_SET(ch->comm,COMM_NOTELL))
      send_to_char("You cannot use tell.\n\r",ch);

    if (IS_SET(ch->comm,COMM_NOCHANNELS))
     send_to_char("You cannot use channels.\n\r",ch);

    if (IS_SET(ch->comm,COMM_NOEMOTE))
      send_to_char("You cannot show emotions.\n\r",ch);

    if (!IS_SET(ch->act,PLR_CANLOOT))
	send_to_char("Your corpse is safe from thieves.\n\r",ch);
    else
        send_to_char("Your corpse may be looted.\n\r",ch);

    if (IS_SET(ch->act,PLR_NOSUMMON))
	send_to_char("You cannot be summoned.\n\r",ch);
    else
	send_to_char("You can be summoned.\n\r",ch);

    if (IS_SET(ch->act,PLR_NOFOLLOW))
	send_to_char("You do not welcome followers.\n\r",ch);
    else
	send_to_char("You accept followers.\n\r",ch);

    if (IS_SET(ch->act,PLR_NOMERCY))
	send_to_char("You show no mercy to those who surrender to you.\n\r",ch);
    else
	send_to_char("You will accept a surrender.\n\r",ch);

    if (IS_SET(ch->act,PLR_NOCANCEL))
	send_to_char("Other peoples cancellation spells will not work on you.\n\r", ch);
    else
	send_to_char("The cancel spell will always work against you.\n\r", ch);
}

void do_autoassist(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_AUTOASSIST))
    {
      send_to_char("Autoassist removed.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_AUTOASSIST);
    }
    else
    {
      send_to_char("You will now assist when needed.\n\r",ch);
      SET_BIT(ch->act,PLR_AUTOASSIST);
    }
}

void do_autoexit(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_AUTOEXIT))
    {
      send_to_char("Exits will no longer be displayed.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_AUTOEXIT);
    }
    else
    {
      send_to_char("Exits will now be displayed.\n\r",ch);
      SET_BIT(ch->act,PLR_AUTOEXIT);
    }
}

void do_autogold(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_AUTOGOLD))
    {
      send_to_char("Autogold removed.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_AUTOGOLD);
    }
    else
    {
      send_to_char("Automatic gold looting set.\n\r",ch);
      SET_BIT(ch->act,PLR_AUTOGOLD);
    }
}

void do_autoloot(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_AUTOLOOT))
    {
      send_to_char("Autolooting removed.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_AUTOLOOT);
    }
    else
    {
      send_to_char("Automatic corpse looting set.\n\r",ch);
      SET_BIT(ch->act,PLR_AUTOLOOT);
    }
}

void do_autosac(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_AUTOSAC))
    {
      send_to_char("Autosacrificing removed.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_AUTOSAC);
    }
    else
    {
      send_to_char("Automatic corpse sacrificing set.\n\r",ch);
      SET_BIT(ch->act,PLR_AUTOSAC);
    }
}

void do_autosplit(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_AUTOSPLIT))
    {
      send_to_char("Autosplitting removed.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_AUTOSPLIT);
    }
    else
    {
      send_to_char("Automatic gold splitting set.\n\r",ch);
      SET_BIT(ch->act,PLR_AUTOSPLIT);
    }
}

void do_brief(CHAR_DATA *ch, char *argument)
{
    if (IS_SET(ch->comm,COMM_BRIEF))
    {
      send_to_char("Full descriptions activated.\n\r",ch);
      REMOVE_BIT(ch->comm,COMM_BRIEF);
    }
    else
    {
      send_to_char("Short descriptions activated.\n\r",ch);
      SET_BIT(ch->comm,COMM_BRIEF);
    }
}

void do_compact(CHAR_DATA *ch, char *argument)
{
    if (IS_SET(ch->comm,COMM_COMPACT))
    {
      send_to_char("Compact mode removed.\n\r",ch);
      REMOVE_BIT(ch->comm,COMM_COMPACT);
    }
    else
    {
      send_to_char("Compact mode set.\n\r",ch);
      SET_BIT(ch->comm,COMM_COMPACT);
    }
}

void do_show(CHAR_DATA *ch, char *argument)
{
    if (IS_SET(ch->comm,COMM_SHOW_AFFECTS))
    {
      send_to_char("Affects will no longer be shown in score.\n\r",ch);
      REMOVE_BIT(ch->comm,COMM_SHOW_AFFECTS);
    }
    else
    {
      send_to_char("Affects will now be shown in score.\n\r",ch);
      SET_BIT(ch->comm,COMM_SHOW_AFFECTS);
    }
}

void do_prompt(CHAR_DATA *ch, char *argument)
{
   char buf[MAX_STRING_LENGTH];

   if ( argument[0] == '\0' )
   {
	if (IS_SET(ch->comm,COMM_PROMPT))
   	{
      	    send_to_char("You will no longer see prompts.\n\r",ch);
      	    REMOVE_BIT(ch->comm,COMM_PROMPT);
    	}
    	else
    	{
      	    send_to_char("You will now see prompts.\n\r",ch);
      	    SET_BIT(ch->comm,COMM_PROMPT);
    	}
       return;
   }

   if( !strcmp( argument, "all" ) )
       strcpy( buf, "<%hhp %mm %vmv> ");
   else
   {
      if ( strlen(argument) > 100 )
         argument[100] = '\0';
      strcpy( buf, argument );
      smash_tilde( buf );
      if (str_suffix("%c",buf))
	  strcat(buf," ");
   }

   free_string( ch->prompt );
   ch->prompt = str_dup( buf );
   sprintf(buf,"Prompt set to %s\n\r",ch->prompt );
   send_to_char(buf,ch);
   return;
}

void do_combine(CHAR_DATA *ch, char *argument)
{
    if (IS_SET(ch->comm,COMM_COMBINE))
    {
      send_to_char("Long inventory selected.\n\r",ch);
      REMOVE_BIT(ch->comm,COMM_COMBINE);
    }
    else
    {
      send_to_char("Combined inventory selected.\n\r",ch);
      SET_BIT(ch->comm,COMM_COMBINE);
    }
}

void do_noloot(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
      return;

    if (IS_SET(ch->act,PLR_CANLOOT))
    {
      send_to_char("Your corpse is now safe from thieves.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_CANLOOT);
    }
    else
    {
      send_to_char("Your corpse may now be looted.\n\r",ch);
      SET_BIT(ch->act,PLR_CANLOOT);
    }
}

void do_nofollow(CHAR_DATA *ch, char *argument)
{
    if ( IS_NPC(ch) || IS_AFFECTED( ch, AFF_CHARM ) )
        return;

    if (IS_SET(ch->act,PLR_NOFOLLOW))
    {
      send_to_char("You now accept followers.\n\r",ch);
      REMOVE_BIT(ch->act,PLR_NOFOLLOW);
    }
    else
    {
      send_to_char("You no longer accept followers.\n\r",ch);
      SET_BIT(ch->act,PLR_NOFOLLOW);
      die_follower( ch );
    }
}

void do_nosummon(CHAR_DATA *ch, char *argument)
{
    if (IS_NPC(ch))
    {
      if (IS_SET(CHAR_IMM(ch),IMM_SUMMON))
      {
	send_to_char("You are no longer immune to summon.\n\r",ch);
	CHAR_REM_IMM(ch,IMM_SUMMON);
      }
      else
      {
	send_to_char("You are now immune to summoning.\n\r",ch);
	CHAR_SET_IMM(ch,IMM_SUMMON);
      }
    }
    else
    {
      if (IS_SET(ch->act,PLR_NOSUMMON))
      {
        send_to_char("You are no longer immune to summon.\n\r",ch);
        REMOVE_BIT(ch->act,PLR_NOSUMMON);
      }
      else
      {
        send_to_char("You are now immune to summoning.\n\r",ch);
        SET_BIT(ch->act,PLR_NOSUMMON);
      }
    }
}

void do_look( CHAR_DATA *ch, char *argument )
{
    char buf  [MAX_STRING_LENGTH];
    char arg1 [MAX_INPUT_LENGTH];
    char arg2 [MAX_INPUT_LENGTH];
    char arg3 [MAX_INPUT_LENGTH];
    int detect_traps;
    EXIT_DATA *pexit;
    CHAR_DATA *victim;
    OBJ_DATA *obj;
    char *pdesc;
    int door;
    int number,count;

    if ( ch->desc == NULL )
	return;

    if ( ch->position < POS_SLEEPING )
    {
	send_to_char( "You can't see anything but stars!\n\r", ch );
	return;
    }

    if ( ch->position == POS_SLEEPING )
    {
	send_to_char( "You can't see anything, you're sleeping!\n\r", ch );
	return;
    }

    if ( !check_blind( ch ) )
	return;

    if ( !IS_NPC(ch)
    &&   room_is_dark(ch, ch->in_room)
    &&   !has_holylight(ch) )
    {
	send_to_char( "It is pitch black ... \n\r", ch );
	show_char_to_char( ch->in_room->people, ch );
	return;
    }

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    number = number_argument(arg1,arg3);
    count = 0;

    if ( arg1[0] == '\0' || !str_cmp( arg1, "auto" ) )
    {
	/* 'look' or 'look auto' */
	send_to_char( P_COLOR( ch, C_ROOM_TITLES ), ch );
	send_to_char( get_room_name(ch), ch );
	send_to_char( P_COLOR_LITERAL( ch, COLOR_OFF ), ch );

	if ( IS_AFFECTED(ch, AFF_DETECT_TRAPS)
	&&   is_room_trapped(ch->in_room) )
	    send_to_char( " (Trapped)", ch );

	if (IS_IMMORTAL(ch) || has_holylight(ch))
	{
	    sprintf(buf," [Room %d]",ch->in_room->vnum);
	    send_to_char(buf,ch);
	}

	send_to_char( "\n\r", ch );

	if ( arg1[0] == '\0'
	|| ( !IS_NPC(ch) && !IS_SET(ch->comm, COMM_BRIEF) ) )
	{
	    send_to_char( "  ",ch);
	    send_to_char( get_room_description(ch), ch );
	}

	if ( (IS_AFFECTED(ch, AFF_DETECT_TRAPS)
	&&    is_room_trapped(ch->in_room))
        ||   (IS_AFFECTED2(ch, AFF2_HALLUCINATION)
        &&    number_bits(4) == 0) )
	    send_to_char( "There seems to be some sort of trap here.\n\r", ch );

        if ( !IS_NPC(ch) && IS_SET(ch->act, PLR_AUTOEXIT) )
	{
	    send_to_char("\n\r",ch);
            do_exits( ch, "auto" );
	}

	show_list_to_char( ch->in_room->contents, ch, FALSE, FALSE );
	show_char_to_char( ch->in_room->people,   ch );

        if ( ch->homevnum == ch->in_room->vnum )
        {
            send_to_char( "\n\r", ch );
            sprintf( buf, "There are %d/%d objects in this room.\n\r",
                count_objects(ch->in_room->contents),
                MAX_ROOM_SAVE_OBJECTS );
            send_to_char( buf, ch );
        }

	return;
    }

    if ( !str_cmp( arg1, "i" ) || !str_cmp(arg1, "in")  || !str_cmp(arg1,"on"))
    {
	/* 'look in' */
	if ( arg2[0] == '\0' )
	{
	    send_to_char( "Look in what?\n\r", ch );
	    return;
	}

	if ( ( obj = get_obj_here( ch, arg2 ) ) == NULL )
	{
	    send_to_char( "You do not see that here.\n\r", ch );
	    return;
	}

	switch ( obj->item_type )
	{
	default:
	    send_to_char( "That is not a container.\n\r", ch );
	    break;

	case ITEM_DRINK_CON:
	    if ( obj->value[1] <= 0 )
	    {
		send_to_char( "It is empty.\n\r", ch );
		break;
	    }

	    sprintf( buf, "It's %sfilled with  a %s liquid.\n\r",
		obj->value[1] <     obj->value[0] / 4
		    ? "less than half-" :
		obj->value[1] < 3 * obj->value[0] / 4
		    ? "about half-"     : "more than half-",
		liq_table[obj->value[2]].liq_color
		);

	    send_to_char( buf, ch );
	    break;

	case ITEM_CONTAINER:
	case ITEM_CORPSE_NPC:
	case ITEM_CORPSE_PC:
	    if ( IS_SET(obj->value[1], CONT_CLOSED) )
	    {
		send_to_char( "It is closed.\n\r", ch );
		break;
	    }

	    act( "$p holds:", ch, obj, NULL, TO_CHAR );
	    show_list_to_char( obj->contains, ch, TRUE, TRUE );
	    break;
	}
	return;
    }

    if ( ( victim = get_char_room( ch, arg1 ) ) != NULL )
    {
	show_char_to_char_1( victim, ch );
	return;
    }

    detect_traps = get_curr_stat( ch, STAT_PER ) / 20 +
                   get_skill( ch, gsn_detrap ) / 3 - 5;

    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( can_see_obj( ch, obj ) )
	{  /* player can see object */
	    if ( obj->item_type == ITEM_CHECKERS_BOARD )
	    {
		look_checkers(ch, obj);
		return;
	    }

	    pdesc = get_extra_descr( arg3, obj->extra_descr );
	    if ( pdesc != NULL )
		if (++count == number)
		{
		    send_to_char( pdesc, ch );

		    if ( !HAS_TRIGGER(obj, OTRIG_EXAMINE)
		    ||   !examining )
			op_simple_trigger( obj, ch, OTRIG_LOOK );
		    return;
		}
		else { continue; }

	    pdesc = get_extra_descr( arg3, obj->pIndexData->extra_descr );
	    if ( pdesc != NULL )
		if (++count == number)
		{
		    send_to_char( pdesc, ch );

		    if ( !HAS_TRIGGER(obj, OTRIG_EXAMINE)
		    ||   !examining )
			op_simple_trigger( obj, ch, OTRIG_LOOK );
		    return;
		}
		else { continue; }

	    if ( is_name( arg3, obj->name ) )
		if (++count == number)
		{

		    send_to_char( obj->description, ch );
		    send_to_char( "\n\r",ch);

		    if ( is_trapped(obj)
		    &&   number_percent() < detect_traps )
			send_to_char( "It seems to be trapped.\n\r", ch );

		    if ( !HAS_TRIGGER(obj, OTRIG_EXAMINE)
		    ||   !examining )
			op_simple_trigger( obj, ch, OTRIG_LOOK );
		    return;
		}
	  }
    }

    for ( obj = ch->in_room->contents; obj != NULL; obj = obj->next_content )
    {
	if ( can_see_obj( ch, obj ) )
	{
	    if ( obj->item_type == ITEM_CHECKERS_BOARD )
	    {
		look_checkers(ch, obj);
		return;
	    }

	    pdesc = get_extra_descr( arg3, obj->extra_descr );
	    if ( pdesc != NULL )
		if (++count == number)
		{
		    send_to_char( pdesc, ch );

		    if ( !HAS_TRIGGER(obj, OTRIG_EXAMINE)
		    ||   !examining )
			op_simple_trigger( obj, ch, OTRIG_LOOK );
		    return;
		}

	    pdesc = get_extra_descr( arg3, obj->pIndexData->extra_descr );
	    if ( pdesc != NULL )
		if (++count == number)
		{
		    send_to_char( pdesc, ch );

		    if ( !HAS_TRIGGER(obj, OTRIG_EXAMINE)
		    ||   !examining )
			op_simple_trigger( obj, ch, OTRIG_LOOK );
		    return;
	    	}

	    if ( is_name( arg3, obj->name ) )
		if (++count == number)
		{
		    send_to_char( obj->description, ch );
		    send_to_char("\n\r",ch);

		    if ( is_trapped(obj)
		    &&   number_percent() < detect_traps )
			send_to_char( "It seems to be trapped.\n\r", ch );

		    if ( !HAS_TRIGGER(obj, OTRIG_EXAMINE)
		    ||   !examining )
			op_simple_trigger( obj, ch, OTRIG_LOOK );
		    return;
		}
	}
    }

    pdesc = get_extra_descr(arg3,ch->in_room->extra_descr);
    if (pdesc != NULL)
    {
	if (++count == number)
	{
	    send_to_char(pdesc,ch);
	    return;
	}
    }

    if (count > 0 && count != number)
    {
    	if (count == 1)
    	    sprintf(buf,"You only see one %s here.\n\r",arg3);
    	else
    	    sprintf(buf,"You only see %d of those here.\n\r",count);

    	send_to_char(buf,ch);
    	return;
    }

         if ( !str_cmp( arg1, "n"   ) || !str_cmp( arg1, "north"		) ) door = 0;
    else if ( !str_cmp( arg1, "e"   ) || !str_cmp( arg1, "east"			) ) door = 1;
    else if ( !str_cmp( arg1, "s"   ) || !str_cmp( arg1, "south"		) ) door = 2;
    else if ( !str_cmp( arg1, "w"   ) || !str_cmp( arg1, "west"			) ) door = 3;
    else if ( !str_cmp( arg1, "u"   ) || !str_cmp( arg1, "up"			) ) door = 4;
    else if ( !str_cmp( arg1, "d"   ) || !str_cmp( arg1, "down"			) ) door = 5;
    else if ( !str_cmp( arg1, "ne"  ) || !str_cmp( arg1, "northeast"		) ) door = 6;
    else if ( !str_cmp( arg1, "se"  ) || !str_cmp( arg1, "southeast"		) ) door = 7;
    else if ( !str_cmp( arg1, "sw"  ) || !str_cmp( arg1, "southwest"		) ) door = 8;
    else if ( !str_cmp( arg1, "nw"  ) || !str_cmp( arg1, "northwest"		) ) door = 9;
    else if ( !str_cmp( arg1, "cw"  ) || !str_cmp( arg1, "clockwise"		) ) door = 10;
    else if ( !str_cmp( arg1, "ccw" ) || !str_cmp( arg1, "counterclockwise"	) ) door = 11;
    else
    {
	send_to_char( "You do not see that here.\n\r", ch );
	return;
    }

    /* 'look direction' */
    if ( ( pexit = ch->in_room->exit[door] ) == NULL )
    {
	send_to_char( "Nothing special there.\n\r", ch );
	return;
    }

    if ( pexit->description != NULL && pexit->description[0] != '\0' )
	send_to_char( pexit->description, ch );
    else
	send_to_char( "Nothing special there.\n\r", ch );

    if ( pexit->keyword    != NULL
    &&   pexit->keyword[0] != '\0'
    &&   pexit->keyword[0] != ' ' )
    {
	if ( IS_SET(pexit->exit_info, EX_CLOSED) )
	{
	    act( "The $d is closed.", ch, NULL, pexit->keyword, TO_CHAR );
	}
	else if ( IS_SET(pexit->exit_info, EX_ISDOOR) )
	{
	    act( "The $d is open.",   ch, NULL, pexit->keyword, TO_CHAR );
	}
    }

    return;
}

/* RT added back for the hell of it */
void do_read (CHAR_DATA *ch, char *argument )
{
    do_look(ch,argument);
}

void do_examine( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Examine what?\n\r", ch );
	return;
    }

    examining = TRUE;
    do_look( ch, arg );
    examining = FALSE;

    if ( ( obj = get_obj_here( ch, arg ) ) != NULL )
    {
	switch ( obj->item_type )
	{
	default:
	    break;

	case ITEM_JUKEBOX:
	    do_play(ch,"list");
	    break;

	case ITEM_MONEY:
	    if (obj->value[0] == 0)
	    {
	        if (obj->value[1] == 0)
		    sprintf(buf,"Odd...there's no coins in the pile.\n\r");
		else if (obj->value[1] == 1)
		    sprintf(buf,"Wow. One gold coin.\n\r");
		else
		    sprintf(buf,"There are %d gold coins in the pile.\n\r",
			obj->value[1]);
	    }
	    else if (obj->value[1] == 0)
	    {
		if (obj->value[0] == 1)
		    sprintf(buf,"Wow. One silver coin.\n\r");
		else
		    sprintf(buf,"There are %d silver coins in the pile.\n\r",
			obj->value[0]);
	    }
	    else
		sprintf(buf,
		    "There are %d gold and %d silver coins in the pile.\n\r",
		    obj->value[1],obj->value[0]);
	    send_to_char(buf,ch);
	    break;

	case ITEM_QUIVER:
	    if ( obj->value[0] <= 0 ) {
		sprintf(buf, "%s is out of arrows.\n\r", obj->short_descr);
		send_to_char(buf, ch);
		return;
	    }
	    if ( obj->value[0] == 1 ) {
		sprintf(buf, "%s has one arrow remaining.\n\r", obj->short_descr);
		send_to_char(buf, ch);
		return;
	    }
	    if ( obj->value[0] <= 0 ) {
		sprintf(buf, "%s has %d arrows in it.\n\r", obj->short_descr, obj->value[0]);
		send_to_char(buf, ch);
		return;
	    }

	case ITEM_DRINK_CON:
	case ITEM_CONTAINER:
	case ITEM_CORPSE_NPC:
	case ITEM_CORPSE_PC:
	    sprintf(buf,"in %s",argument);
	    do_look( ch, buf );
	}

	check_object_for_trap( ch, obj, TRAP_EXAMINE );
	if ( !CHAR_KILLED(ch) )
	    op_simple_trigger( obj, ch, OTRIG_EXAMINE );
    }

    return;
}



/*
 * Thanks to Zrin for auto-exit part.
 */
void do_exits( CHAR_DATA *ch, char *argument )
{
    extern char * const dir_name[];
    char buf[MAX_STRING_LENGTH];
    EXIT_DATA *pexit;
    bool found;
    bool fAuto;
    int door;

    fAuto  = !str_cmp( argument, "auto" );

    if ( !check_blind( ch ) )
	return;

    if (fAuto)
	sprintf(buf,"[Exits:");
    else if (IS_IMMORTAL(ch))
	sprintf(buf,"Obvious exits from room %d:\n\r",ch->in_room->vnum);
    else
	sprintf(buf,"Obvious exits:\n\r");

    found = FALSE;
    for ( door = 0; door <= 11; door++ )
    {
	if ( ( pexit = ch->in_room->exit[door] ) != NULL
	&&   pexit->u1.to_room != NULL
	&&   can_see_room(ch,pexit->u1.to_room)
	&&   !IS_SET(pexit->exit_info, EX_CLOSED) )
	{
	    found = TRUE;
	    if ( fAuto )
	    {
		strcat( buf, " " );
		strcat( buf, dir_name[door] );
	    }
	    else
	    {
		sprintf( buf + strlen(buf), "%-5s - %s",
		    capitalize( dir_name[door] ),
		    room_is_dark( ch, pexit->u1.to_room )
			?  "Too dark to tell"
			: pexit->u1.to_room->name
		    );
		if (IS_IMMORTAL(ch))
		    sprintf(buf + strlen(buf),
			" (room %d)\n\r",pexit->u1.to_room->vnum);
		else
		    sprintf(buf + strlen(buf), "\n\r");
	    }
	}
    }

    if ( !found )
	strcat( buf, fAuto ? " none" : "None.\n\r" );

    if ( fAuto )
	strcat( buf, "]\n\r" );

    send_to_char( buf, ch );
    return;
}

void do_worth( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if (IS_NPC(ch))
    {
	sprintf(buf,"You have %ld gold and %ld silver.\n\r",
	    ch->gold,ch->silver);
	send_to_char(buf,ch);
	return;
    }

    sprintf(buf,
    "You have %ld gold, %ld silver, and %d experience (%d exp to level).\n\r",
	ch->gold, ch->silver,ch->exp,
	(ch->level + 1) * exp_per_level(ch,ch->pcdata->points) - ch->exp);

    send_to_char(buf,ch);

    return;
}


const char *get_ac_words( int ac )
{
         if ( ac >=  101 ) return "hopeless";
    else if ( ac >=   80 ) return "defenseless";
    else if ( ac >=   60 ) return "barely protected";
    else if ( ac >=   40 ) return "slightly armored";
    else if ( ac >=   20 ) return "somewhat armored";
    else if ( ac >=    0 ) return "armored";
    else if ( ac >=  -20 ) return "well armored";
    else if ( ac >=  -40 ) return "heavily armored";
    else if ( ac >=  -80 ) return "superbly armored";
    else if ( ac >= -100 ) return "invulnerable";
    else                   return "divinely armored";
}


void do_score( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
    const char *alignment;
    const char *position;
    int btmbar = 0;

    sprintf( buf, "%s==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r",
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf2, "%s%s", ch->name, IS_NPC(ch) ? "" : ch->pcdata->title );
    sprintf( buf,
        "| %sYou are %-68s%s|\n\r",
        P_COLOR(ch, C_TITLES),
        buf2,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

         if ( ch->alignment >  900 ) alignment = "an angelic";
    else if ( ch->alignment >  700 ) alignment = "a saintly";
    else if ( ch->alignment >  350 ) alignment = "a good";
    else if ( ch->alignment >  100 ) alignment = "a kind";
    else if ( ch->alignment > -100 ) alignment = "a neutral";
    else if ( ch->alignment > -350 ) alignment = "a mean";
    else if ( ch->alignment > -700 ) alignment = "an evil";
    else if ( ch->alignment > -900 ) alignment = "a demonic";
    else                             alignment = "a satanic";
    sprintf( buf2, "%s worshiper of %s.",
        alignment,
        religion_table[GET_GOD(ch)].name );
    sprintf( buf, "| %sYou are %-68s%s|\n\r",
        P_COLOR(ch, C_INFORMATION),
        buf2,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
	"==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r");
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s    Strength: %s%-17s%s|     %sHealth: %s%7d/%7d %s|     %sAge: %s%4d %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_STR),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->hit, ch->max_hit,
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_age(ch),
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%sIntelligence: %s%-17s%s|       %sMana: %s%7d/%7d %s|   %sHours: %s%4d %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_INT),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->mana, ch->max_mana,
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ( ch->played + (int) (current_time - ch->logon) ) / 3600,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s      Wisdom: %s%-17s%s|   %sMovement: %s%7d/%7d %s|   %sWimpy: %s%4d %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_WIS),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->move, ch->max_move,
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->wimpy,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s   Dexterity: %s%-17s%s|                             |               |\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_DEX),
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%sConstitution: %s%-17s%s|      %sItems: %s%7d/%7d %s|   %sLevel: %s%4d %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_CON),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->carry_number, can_carry_n(ch),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->level,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s  Perception: %s%-17s%s|     %sPounds: %s%7ld/%7d%s%s|   %sPracs: %s%4d %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_PER),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_carry_weight(ch) / 10, can_carry_w(ch) /10,
        !IS_NPC(ch) &&
        IS_SET(class_table[ch->class].flags, CLASS_MARTIAL_ARTIST) &&
        ch->armor_weight >= MAX_MARTIAL_ARMOR + get_curr_stat(ch, STAT_STR) - 50
          ? "*" : " ",
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->practice,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s    Charisma: %s%-17s%s|                             |%s  Trains: %s%4d %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_stat_word(ch, STAT_CHA),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->train,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|                               |       %sGold: %s%15ld %s|               |\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->gold,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%sAC vs Pierce: %s%-17s%s|     %sSilver: %s%15ld %s|%sYou are:       %s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_ac_words(get_ac(ch, AC_PIERCE)),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->silver,
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s  AC vs Bash: %s%-17s%s|                             |%s a %-12s%s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_ac_words(get_ac(ch, AC_BASH)),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_INFORMATION),
        race_table[ch->race].name,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s AC vs Slash: %s%-17s%s|         %sXP: %s%15d %s|%s %-14s%s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_ac_words(get_ac(ch, AC_SLASH)),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        ch->exp,
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_INFORMATION),
        ch->sex == 0 ? "unsexed" : ch->sex == 1 ? "a male" : "a female",
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
        "|%s AC vs Magic: %s%-17s%s|%sXP to level: %s%15d %s|%s a %-12s%s|\n\r",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        get_ac_words(get_ac(ch, AC_SLASH)),
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        (!IS_NPC(ch) && ch->level < LEVEL_HERO)
          ? ((ch->level + 1) * exp_per_level(ch,ch->pcdata->points) - ch->exp)
          : 0,
        P_COLOR(ch, C_BORDERS),
        P_COLOR(ch, C_INFORMATION),
        IS_NPC(ch) ? "mobile" : class_table[ch->class].name,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    switch ( ch->position )
    {
    case POS_DEAD:      position = "DEAD";          break;
    case POS_MORTAL:    position = "dying";         break;
    case POS_INCAP:     position = "incapacitated"; break;
    case POS_STUNNED:   position = "stunned";       break;
    case POS_SLEEPING:  position = "sleeping";      break;
    case POS_RESTING:   position = "resting";       break;
    case POS_SITTING:   position = "sitting";       break;
    case POS_STANDING:  position = "standing";      break;
    case POS_FIGHTING:  position = "fighting";      break;
    default:            position = "unknown";       break;
    }

    if ( !IS_NPC(ch) ) {
	sprintf( buf2, "  %sHit Bonus: %s%15d %s",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        GET_HITROLL(ch),
        P_COLOR(ch, C_BORDERS) );
    }
    else {
	sprintf( buf2, "  %sHit Bonus: %s%15d %s",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        GET_HITROLL_MOB(ch),
        P_COLOR(ch, C_BORDERS) );
    }

    sprintf( buf,
        "|                               |%s|%s of %-10s %s|\n\r",
        ch->level >= 15 ? buf2 : "                             ",
	P_COLOR(ch, C_INFORMATION),
	hometown_table[ch->hometown].ht_name,
	P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    if ( !IS_NPC(ch) ) {
    sprintf( buf2, "  %sDam Bonus: %s%15d %s",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        GET_DAMROLL(ch),
        P_COLOR(ch, C_BORDERS) );
    }
    else {
    sprintf( buf2, "  %sDam Bonus: %s%15d %s",
        P_COLOR(ch, C_TITLES),
        P_COLOR(ch, C_INFORMATION),
        GET_DAMROLL_MOB(ch),
        P_COLOR(ch, C_BORDERS) );
    }

    sprintf( buf,
        "|                               |%s|%s %-14s%s|\n\r",
        ch->level >= 15 ? buf2 : "                             ",
        P_COLOR(ch, C_INFORMATION),
        position,
        P_COLOR(ch, C_BORDERS) );
    send_to_char( buf, ch );

    sprintf( buf,
 	"==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r%s",
        IS_SET(ch->comm, COMM_COLOR) ? COLOR_OFF : "" );

    send_to_char( buf, ch );

    if ( !IS_NPC(ch)
    &&   IS_SET(class_table[ch->class].flags, CLASS_MARTIAL_ARTIST)
    &&   ch->armor_weight >= MAX_MARTIAL_ARMOR + get_curr_stat(ch, STAT_STR) - 50 ) {
        sprintf( buf, "%s|%s You are wearing too much armor to be able to fight effectively unarmed.     %s|\n\r",
            P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION), P_COLOR(ch, C_BORDERS) );
        btmbar = 1;
        send_to_char( buf, ch );
    }

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK]   > 10 ) {
        sprintf( buf, "%s|%s You are drunk.                                                              %s|\n\r",
            P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION), P_COLOR(ch, C_BORDERS) );
        btmbar = 1;
        send_to_char( buf, ch );
        }

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_THIRST] ==  0 ) {
        sprintf( buf, "%s|%s You are thirsty.                                                            %s|\n\r",
            P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION), P_COLOR(ch, C_BORDERS) );
        btmbar = 1;
        send_to_char( buf, ch );
        }

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_HUNGER]   ==  0 ) {
        sprintf( buf, "%s|%s You are hungry.                                                             %s|\n\r",
            P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION), P_COLOR(ch, C_BORDERS) );
        btmbar = 1;
        send_to_char( buf, ch );
        }

    if ( btmbar == 1 ) {
        sprintf( buf,
	    "==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r%s",
            IS_SET(ch->comm, COMM_COLOR) ? COLOR_OFF : "" );
        btmbar = 0;
        send_to_char( buf, ch );
    }

    if (IS_SET(ch->comm,COMM_SHOW_AFFECTS)) {
        do_affects(ch,"score");
    }
}


#if 0
void do_score_old( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int i;

    sprintf( buf,
	"You are %s%s, level %d, %d years old (%d hours).\n\r",
	ch->name,
	IS_NPC(ch) ? "" : ch->pcdata->title,
	ch->level, get_age(ch),
	( ch->played + (int) (current_time - ch->logon) ) / 3600);
    send_to_char( buf, ch );

    if ( get_trust( ch ) != ch->level )
    {
	sprintf( buf, "You are trusted at level %d.\n\r",
	    get_trust( ch ) );
	send_to_char( buf, ch );
    }

    if ( get_olc_level(ch) != 0 )
    {
	sprintf( buf, "Your OLC level is %d.\n\r", get_olc_level(ch) );
	send_to_char( buf, ch );
    }

    sprintf(buf, "Race: %s  Sex: %s  Class: ",
	race_table[ch->race].name,
	ch->sex == 0 ? "sexless" : ch->sex == 1 ? "male" : "female" );
    if ( IS_NPC(ch) )
	strcat( buf, "mobile" );
    else
    {
	for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
	{
	    if ( ch->pcdata->former_professions[i] == -1 )
		break;

	    strcat( buf,
	        class_table[ch->pcdata->former_professions[i]].name );
	    strcat( buf, "-" );
	}

	strcat( buf, class_table[ch->class].name );
    }
    strcat( buf, "\n\r" );
    send_to_char( buf, ch );

    send_to_char( "You are ", ch );
         if ( ch->alignment >  900 ) send_to_char( "an angelic", ch );
    else if ( ch->alignment >  700 ) send_to_char( "a saintly",  ch );
    else if ( ch->alignment >  350 ) send_to_char( "a good",     ch );
    else if ( ch->alignment >  100 ) send_to_char( "a kind",     ch );
    else if ( ch->alignment > -100 ) send_to_char( "a neutral",  ch );
    else if ( ch->alignment > -350 ) send_to_char( "a mean",     ch );
    else if ( ch->alignment > -700 ) send_to_char( "an evil",    ch );
    else if ( ch->alignment > -900 ) send_to_char( "a demonic",  ch );
    else                             send_to_char( "a satanic",  ch );
    sprintf( buf, " worshiper of %s.\n\r", religion_table[GET_GOD(ch)].name );
    send_to_char( buf, ch );

    sprintf( buf,
	"You have %d/%d hit, %d/%d mana, %d/%d movement.\n\r",
	ch->hit,  ch->max_hit,
	ch->mana, ch->max_mana,
	ch->move, ch->max_move);
    send_to_char( buf, ch );

    sprintf( buf,
	"You have %d practices and %d training sessions.\n\r",
	ch->practice, ch->train);
    send_to_char( buf, ch );

    sprintf( buf,
	"You are carrying %d/%d items with weight %ld/%d pounds.\n\r",
	ch->carry_number, can_carry_n(ch),
	get_carry_weight(ch) / 10, can_carry_w(ch) /10 );
    send_to_char( buf, ch );

    if ( !IS_NPC(ch)
    &&   IS_SET(class_table[ch->class].flags, CLASS_MARTIAL_ARTIST)
    &&   ch->armor_weight >= MAX_MARTIAL_ARMOR + get_curr_stat(ch, STAT_STR) - 50 )
	send_to_char( "You are wearing to much armor to be able to fight effectively unarmed.\n\r", ch );

    sprintf( buf,
	"Strength:     %-15s        Intelligence: %-15s\n\r"
	"Wisdom:       %-15s        Dexterity:    %-15s\n\r"
	"Constitution: %-15s        Perception:   %-15s\n\r"
	"Charisma:     %-15s\n\r",
	get_stat_word(ch, STAT_STR),
	get_stat_word(ch, STAT_INT),
	get_stat_word(ch, STAT_WIS),
	get_stat_word(ch, STAT_DEX),
	get_stat_word(ch, STAT_CON),
	get_stat_word(ch, STAT_PER),
	get_stat_word(ch, STAT_CHA) );
    send_to_char( buf, ch );

    sprintf( buf,
	"You have scored %d exp, and have %ld gold and %ld silver coins.\n\r",
	ch->exp,  ch->gold, ch->silver );
    send_to_char( buf, ch );

    /* RT shows exp to level */
    if (!IS_NPC(ch) && ch->level < LEVEL_HERO)
    {
      sprintf (buf,
	"You need %d exp to level.\n\r",
	((ch->level + 1) * exp_per_level(ch,ch->pcdata->points) - ch->exp));
      send_to_char( buf, ch );
     }

    sprintf( buf, "Wimpy set to %d hit points.\n\r", ch->wimpy );
    send_to_char( buf, ch );

    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_DRUNK]   > 10 )
	send_to_char( "You are drunk.\n\r",   ch );
    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_THIRST] ==  0 )
	send_to_char( "You are thirsty.\n\r", ch );
    if ( !IS_NPC(ch) && ch->pcdata->condition[COND_HUNGER]   ==  0 )
	send_to_char( "You are hungry.\n\r",  ch );

    switch ( ch->position )
    {
    case POS_DEAD:
	send_to_char( "You are DEAD!!\n\r",		ch );
	break;
    case POS_MORTAL:
	send_to_char( "You are mortally wounded.\n\r",	ch );
	break;
    case POS_INCAP:
	send_to_char( "You are incapacitated.\n\r",	ch );
	break;
    case POS_STUNNED:
	send_to_char( "You are stunned.\n\r",		ch );
	break;
    case POS_SLEEPING:
	send_to_char( "You are sleeping.\n\r",		ch );
	break;
    case POS_RESTING:
	send_to_char( "You are resting.\n\r",		ch );
	break;
    case POS_SITTING:
	send_to_char( "You are sitting.\n\r",		ch );
	break;
    case POS_STANDING:
	send_to_char( "You are standing.\n\r",		ch );
	break;
    case POS_FIGHTING:
	send_to_char( "You are fighting.\n\r",		ch );
	break;
    }


    /* print AC values */
    if (ch->level >= 25)
    {
	sprintf( buf,"Armor: pierce: %d  bash: %d  slash: %d  magic: %d\n\r",
		 get_ac(ch,AC_PIERCE),
		 get_ac(ch,AC_BASH),
		 get_ac(ch,AC_SLASH),
		 get_ac(ch,AC_EXOTIC));
    	send_to_char(buf,ch);
    }

    for (i = 0; i < 4; i++)
    {
	char * temp;

	switch(i)
	{
	    case(AC_PIERCE):	temp = "piercing";	break;
	    case(AC_BASH):	temp = "bashing";	break;
	    case(AC_SLASH):	temp = "slashing";	break;
	    case(AC_EXOTIC):	temp = "magic";		break;
	    default:		temp = "error";		break;
	}

	send_to_char("You are ", ch);

	if      (get_ac(ch,i) >=  101 )
	    sprintf(buf,"hopelessly vulnerable to %s.\n\r",temp);
	else if (get_ac(ch,i) >= 80)
	    sprintf(buf,"defenseless against %s.\n\r",temp);
	else if (get_ac(ch,i) >= 60)
	    sprintf(buf,"barely protected from %s.\n\r",temp);
	else if (get_ac(ch,i) >= 40)
	    sprintf(buf,"slightly armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= 20)
	    sprintf(buf,"somewhat armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= 0)
	    sprintf(buf,"armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= -20)
	    sprintf(buf,"well-armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= -40)
	    sprintf(buf,"very well-armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= -60)
	    sprintf(buf,"heavily armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= -80)
	    sprintf(buf,"superbly armored against %s.\n\r",temp);
	else if (get_ac(ch,i) >= -100)
	    sprintf(buf,"almost invulnerable to %s.\n\r",temp);
	else
	    sprintf(buf,"divinely armored against %s.\n\r",temp);

	send_to_char(buf,ch);
    }


    /* RT wizinvis and holy light */
    if ( IS_IMMORTAL(ch))
    {
      send_to_char("Holy Light: ",ch);
      if (IS_SET(ch->act,PLR_HOLYLIGHT))
        send_to_char("on",ch);
      else
        send_to_char("off",ch);

      if (ch->invis_level)
      {
        sprintf( buf, "  Invisible: level %d",ch->invis_level);
        send_to_char(buf,ch);
      }

      if (ch->incog_level)
      {
	sprintf(buf,"  Incognito: level %d",ch->incog_level);
	send_to_char(buf,ch);
      }
      send_to_char("\n\r",ch);
    }

    if ( ch->level >= 15 )
    {
	sprintf( buf, "Hitroll: %d  Damroll: %d.\n\r",
	    GET_HITROLL(ch), GET_DAMROLL(ch) );
	send_to_char( buf, ch );
    }

/*
    if ( ch->level >= 10 )
    {
	sprintf( buf, "Alignment: %d.  ", ch->alignment );
	send_to_char( buf, ch );
    }

    send_to_char( "You are ", ch );
         if ( ch->alignment >  900 ) send_to_char( "angelic.\n\r", ch );
    else if ( ch->alignment >  700 ) send_to_char( "saintly.\n\r", ch );
    else if ( ch->alignment >  350 ) send_to_char( "good.\n\r",    ch );
    else if ( ch->alignment >  100 ) send_to_char( "kind.\n\r",    ch );
    else if ( ch->alignment > -100 ) send_to_char( "neutral.\n\r", ch );
    else if ( ch->alignment > -350 ) send_to_char( "mean.\n\r",    ch );
    else if ( ch->alignment > -700 ) send_to_char( "evil.\n\r",    ch );
    else if ( ch->alignment > -900 ) send_to_char( "demonic.\n\r", ch );
    else                             send_to_char( "satanic.\n\r", ch );
*/

    if (IS_SET(ch->comm,COMM_SHOW_AFFECTS))
	do_affects(ch,"");
}
#endif


void do_affects(CHAR_DATA *ch, char *argument )
{
    AFFECT_DATA *paf, *paf_last = NULL;
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    bool first = TRUE;

    one_argument(argument, arg);

    if ( ch->affected == NULL ) {
        send_to_char( "You are not affected by any spells.\n\r", ch );
	return;
    }

    if ( arg != "score" ) {
        sprintf( buf, "%s==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r",
            P_COLOR(ch, C_BORDERS) );
        send_to_char( buf, ch );
    }

    for ( paf = ch->affected; paf != NULL; paf = paf->next )
    {
	if ( IS_SET(paf->flags, AFF_INVIS) )
	    continue;

	if ( first )
	{
	    if ( ch->level >=20 )
		sprintf( buf, "|%s Spell                                   %s|%s Modifies        %s|%s By  %s|%s Duration  %s|\n\r",
		    P_COLOR(ch, C_TITLES), P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_TITLES), P_COLOR(ch, C_BORDERS),
		    P_COLOR(ch, C_TITLES), P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_TITLES), P_COLOR(ch, C_BORDERS) );
	    else
		sprintf( buf, "|%s You are affected by:                                                        %s|\n\r",
		    P_COLOR(ch, C_TITLES), P_COLOR(ch, C_BORDERS) );
            send_to_char( buf, ch );

            sprintf( buf, "%s==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r",
                P_COLOR(ch, C_BORDERS) );
            send_to_char( buf, ch );

	    first = FALSE;
	}

	if (paf_last != NULL && paf->type == paf_last->type)
	    if ( ch->level >= 20 ) {
		sprintf( buf, "                                         " );
                sprintf( buf, "|                                         |%s ",
		    P_COLOR(ch, C_INFORMATION) );
	    } else
		continue;
	else
            if ( ch->level >= 20 )
		sprintf( buf, "|%s %-40s%s|%s ",
		    P_COLOR(ch, C_INFORMATION),
		    paf->name[0] != '\0'
		    ? paf->name
		    : skill_table[paf->type].name,
		    P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION) );
	    else
		sprintf( buf, "|%s %-40s                                    %s|",
		    P_COLOR(ch, C_INFORMATION),
		    paf->name[0] != '\0'
		    ? paf->name
		    : skill_table[paf->type].name,
		    P_COLOR(ch, C_BORDERS) );

	send_to_char( buf, ch );

	if ( ch->level >= 20 )
	{
	    sprintf( buf,
		"%-16s%s|%s %3d %s|%s ",
		affect_loc_name( paf->location ),
		P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION),
		paf->modifier,
		P_COLOR(ch, C_BORDERS), P_COLOR(ch, C_INFORMATION) );
	    send_to_char( buf, ch );

	    if ( paf->duration == -1 )
		sprintf( buf, "permanent %s|", P_COLOR(ch, C_BORDERS) );
	    else
	    {
		if ( paf->duration > TICKS_IN_HOUR
		&&   paf->duration % TICKS_IN_HOUR != 0 )
		    sprintf( buf, "%6d:%02d %s|",
		        paf->duration / TICKS_IN_HOUR,
	  	        paf->duration % TICKS_IN_HOUR * MINUTES_IN_TICK,
			P_COLOR(ch, C_BORDERS) );
	  	else if ( paf->duration >= TICKS_IN_HOUR )
	  	    sprintf( buf, "%6d:00 %s|",
	  	        paf->duration / TICKS_IN_HOUR,
			P_COLOR(ch, C_BORDERS) );
		else
		    sprintf( buf, "     0:%02d %s|", paf->duration * MINUTES_IN_TICK,
			P_COLOR(ch, C_BORDERS) );
	    }
	    send_to_char( buf, ch );
	}

	send_to_char( "\n\r", ch );
	paf_last = paf;
    }

    sprintf( buf,
	"==<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>===<*>==\n\r%s",
        IS_SET(ch->comm, COMM_COLOR) ? COLOR_OFF : "" );
    send_to_char( buf, ch );

    if ( first )
	send_to_char("You are not affected by any spells.\n\r",ch);

    return;
}


char *	const	day_name	[] =
{
    "the Moon", "the Bull", "Deception", "Thunder", "Freedom",
    "the Great Gods", "the Sun"
};

char *	const	month_name	[] =
{
    "Winter", "the Winter Wolf", "the Frost Giant", "the Old Forces",
    "the Grand Struggle", "the Spring", "Nature", "Futility", "the Dragon",
    "the Sun", "the Heat", "the Battle", "the Dark Shades", "the Shadows",
    "the Long Shadows", "the Ancient Darkness", "the Great Evil"
};

char *	const	moon_name	[] =
{
    "new", "waxing", "waxing", "full", "waning", "waning"
};

char *	const	season_name	[] =
{
    "winter", "spring", "summer", "fall"
};

void do_time( CHAR_DATA *ch, char *argument )
{
    extern char str_boot_time[];
    extern time_t boot_time;
    extern char str_warmboot_time[];
    extern time_t warmboot_time;
    extern char str_reboot_time[];
    extern bool reboot_set;
    char buf[MAX_STRING_LENGTH];
    char *suf;
    char *tmstr;
    int day;
    int d, h, m, wd, wh, wm;
    time_t mytime, wmytime;

    day     = time_info.day + 1;

         if ( day > 4 && day <  20 ) suf = "th";
    else if ( day % 10 ==  1       ) suf = "st";
    else if ( day % 10 ==  2       ) suf = "nd";
    else if ( day % 10 ==  3       ) suf = "rd";
    else                             suf = "th";

//    tmbstr = (char *) asctime(localtime((&boot_time));
//    tmbstr = str_boot_time;
//    *(tmbstr + strlen(tmbstr) - 1) = '\0';

    tmstr = (char *) ctime(&current_time);
    *(tmstr + strlen(tmstr) - 1) = '\0';

    mytime = time(0) - boot_time;
    d = mytime / 86400;
    h = (mytime / 3600) % 24;
    m = (mytime / 60) % 60;

    wmytime = time(0) - warmboot_time;
    wd = wmytime / 86400;
    wh = (wmytime / 3600) % 24;
    wm = (wmytime / 60) % 60;

    sprintf( buf,
	"It is %d o'clock %s, Day of %s, "
	"%d%s the Month of %s, in the year %d.\n\r",
	(time_info.hour % 12 == 0) ? 12 : time_info.hour %12,
	time_info.hour >= 12 ? "pm" : "am",
	day_name[day % 7],
	day, suf,
	month_name[time_info.month],
	time_info.year);
    send_to_char(buf,ch);
    sprintf( buf,
	"The moon is %s. It is %s.\n\r",
	moon_name[LUNAR_PHASE],
	season_name[SEASON]);
    send_to_char(buf,ch);
    sprintf(buf, "Labyrinth started up at:     %-24s (%2d day%-1s, %2d:%02d ago)\n\r",
	str_boot_time, d, ((d == 1) ? "" : "s"), h, m);
    send_to_char( buf, ch );
    sprintf(buf, "The last warm reboot was at: %-24s (%2d day%-1s, %2d:%02d ago)\n\r",
	str_warmboot_time, wd, ((wd == 1) ? "" : "s"), wh, wm);
    send_to_char( buf, ch );
    sprintf(buf, "Current system time is:      %-24s\n\r", tmstr);
    send_to_char( buf, ch );
    if ( reboot_set ) {
	sprintf(buf, "The next reboot is at:       %-24s\n\r", str_reboot_time);
	send_to_char(buf, ch);
    } else
	send_to_char("There is currently no reboot scheduled.\n\r", ch);

    return;
}



void do_weather( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    static char * const sky_look[4] =
    {
	"cloudless",
	"cloudy",
	"rainy",
	"lit by flashes of lightning"
    };

    if ( !IS_OUTSIDE(ch) )
    {
	send_to_char( "You can't see the weather indoors.\n\r", ch );
	return;
    }

    if ( UNDERWATER(ch) )
    {
	send_to_char( "Not while your beneath the waves.\n\r", ch );
	return;
    }

    sprintf( buf, "The sky is %s and %s.\n\r",
	sky_look[weather_info.sky],
	weather_info.change >= 0
	? "a warm southerly breeze blows"
	: "a cold northern gust blows"
	);
    send_to_char( buf, ch );
    return;
}


void expand_help( CHAR_DATA *ch, BUFFER *output, const char *text )
{
    char buf[MAX_STRING_LENGTH];
    const char *remainder;
    const char *helpTag;

    for ( remainder = text; (helpTag = str_str("<help=", remainder)) != NULL; )
    {
        char args_buf[MAX_STRING_LENGTH], arg[MAX_STRING_LENGTH];
        const char *endHelpTag;
        const char *args;

        strncpy( buf, remainder, (int) (helpTag - remainder) );
        buf[ (int) (helpTag - remainder) ] = '\0';
        add_buf( output, buf );

        helpTag   += strlen( "<help=" );
        endHelpTag = str_str( ">", helpTag );
        strncpy( args_buf, helpTag, (int) (endHelpTag - helpTag) );
        args_buf[ (int) (endHelpTag - helpTag) ] = '\0';
        args = args_buf;

        args = one_argument( args, arg );
        if ( str_cmp(arg, "default_skills") == 0 )
            append_default_skills( ch, output, args );
        else if ( str_cmp(arg, "gainable_skills") == 0 )
            append_gainable_skills( ch, output, args );
        else if ( str_cmp(arg, "default_groups") == 0 )
            append_default_groups( ch, output, args );
        else if ( str_cmp(arg, "gainable_groups") == 0 )
            append_gainable_groups( ch, output, args );
        else if ( str_cmp(arg, "race_cost") == 0 )
            append_race_cost( ch, output, args );
        else if ( str_cmp(arg, "race_skills") == 0 )
            append_race_skills( ch, output, args );
        else if ( str_cmp(arg, "ch_cost") == 0 )
            append_ch_cost(ch, output, args );
        else
        {
            sprintf( buf, "<unknown mhtml tag specified - %s>", arg );
            add_buf( output, buf );
        }

        /* process args */
        remainder = endHelpTag + 1;
    }
}


void do_help( CHAR_DATA *ch, char *argument )
{
    char argall[MAX_INPUT_LENGTH], argone[MAX_INPUT_LENGTH];
    char buf[MAX_STRING_LENGTH];
    bool found = FALSE;
    HELP_DATA *pHelp;
    BUFFER *output;
    int level;

    output = new_buf();

    if ( argument[0] == '\0' )
	argument = "summary";

    /* this parts handles help a b so that it returns help 'a b' */
    argall[0] = '\0';
    while ( argument[0] != '\0' )
    {
	argument = one_argument( argument, argone );
	if ( argall[0] != '\0' )
	    strcat( argall, " " );
	strcat( argall, argone );
    }

    for ( pHelp = help_first; pHelp != NULL; pHelp = pHelp->next )
    {
    	level = (pHelp->level < 0) ? -1 * pHelp->level - 1 : pHelp->level;

	if ( level > get_trust(ch) )
	    continue;

	if ( is_name(argall, pHelp->keyword) )
	{
	    /* add seperator if found */
	    if ( found )
            {
                sprintf( buf, "%s\n\r============================================================\n\r\n\r",
                    P_COLOR(ch, C_BORDERS) );

		add_buf( output, buf );
            }

	    if ( pHelp->level >= 0 && str_cmp(argall, "imotd") )
	    {
                sprintf( buf, "%s%s\n\r", P_COLOR(ch, C_TITLES), pHelp->keyword );
		add_buf( output, buf );
	    }

	    /*
	     * Strip leading '.' to allow initial blanks.
	     */
            add_buf( output, P_COLOR(ch, C_INFORMATION) );
	    if ( pHelp->text[0] == '.' )
		add_buf( output, pHelp->text + 1 );
            else if ( !str_infix("<help=", pHelp->text) )
                expand_help( ch, output, pHelp->text );
	    else
		add_buf( output, pHelp->text );

	    found = TRUE;

	    /* small hack :) */
	    if ( ch->desc != NULL && ch->desc->connected != CON_PLAYING
	    &&                       ch->desc->connected != CON_GEN_GROUPS )
		break;
	}
    }

    if ( !found )
    	send_to_char( "No help on that word.\n\r", ch );
    else
    {
	if ( IS_SET(ch->comm, COMM_COLOR) )
	    add_buf( output, COLOR_OFF );
        add_buf( output, "\n\r" );
	page_to_char( buf_string(output), ch );
    }
    free_buf( output );
}


/* whois command */
void do_whois(CHAR_DATA *ch, char *argument)
{
    char arg[MAX_INPUT_LENGTH];
    BUFFER *output;
    char buf[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    bool found = FALSE;
    bool first = TRUE;

    one_argument(argument,arg);

    if (arg[0] == '\0')
    {
	    send_to_char("You must provide a name.\n\r",ch);
	    return;
    }

    output = new_buf();

    for (d = descriptor_list; d != NULL; d = d->next)
    {
        CHAR_DATA *wch;

        if (d->connected != CON_PLAYING || !can_see(ch,d->character))
            continue;

        wch = ( d->original != NULL ) ? d->original : d->character;

        if (!can_see(ch,wch))
            continue;

        if (!str_prefix(arg,wch->name))
        {
            char class_buf[MAX_STRING_LENGTH];
            char race_buf[MAX_STRING_LENGTH];
            char age_buf[MAX_STRING_LENGTH];
	    char ht_buf[MAX_STRING_LENGTH];
            const char *level_msg;
            int territory;
            found = TRUE;

            /* work out the printing */
            if ( !strcmp(wch->name, "Murgen") )
                strcpy( class_buf, "seer" );
            else if ( !strcmp(wch->name, "Thollin") ) // If them, why not me?
                strcpy( class_buf, "angel" );
            else if ( !strcmp(wch->name, "Ereptor") ) // Me too!
                strcpy( class_buf, "meddler" );
            else if ( !strcmp(wch->name, "Avvrax") )
                strcpy( class_buf, "weaver" );
            else
	    {
                int i;

                class_buf[0] = '\0';

                for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
                {
                    if ( wch->pcdata->former_professions[i] == -1 )
	                break;

                    strcat( class_buf,
	                    class_table[wch->pcdata->former_professions[i]].name );
                    strcat( class_buf, "-" );
                }

                strcat( class_buf, class_table[wch->class].name );
	    }

            if ( wch->level <= 5 )
	        level_msg = "insignificant";
            else if ( wch->level <= 10 )
	        level_msg = "inferior";
            else if ( wch->level <= 15 )
	        level_msg = "minor";
            else if ( wch->level <= 20 )
	        level_msg = "slight";
            else if ( wch->level <= 25 )
	        level_msg = "moderate";
            else if ( wch->level <= 30 )
	        level_msg = "strong";
            else if ( wch->level <= 35 )
	        level_msg = "superior";
            else if ( wch->level <= 40 )
	        level_msg = "great";
            else if ( wch->level <= 45 )
	        level_msg = "colossal";
            else if ( wch->level <= 50 )
	        level_msg = "enormous";
            else if ( wch->level <= 51 )
	        level_msg = "cosmic";
            else if ( wch->level <= 59 )
	        level_msg = "immeasurable";
            else
	        level_msg = "unsurpassed";

            if ( !strcmp(wch->name, "Murgen") )
                sprintf( age_buf, "immense" );
            else if ( !strcmp(wch->name, "Llabrynthe") )
                sprintf( age_buf, "undeterminable" );
            else if ( !strcmp(wch->name, "Thollin") )
                sprintf( age_buf, "ageless" );
            else if ( !strcmp(wch->name, "Ereptor") )
                sprintf(age_buf,  "older than dirt" );
            else if ( !strcmp(wch->name, "Gnorm") )
                sprintf(age_buf,  "you actually care?" );
            else
	        sprintf( age_buf, "%d", get_age(wch) );

	    if ( IS_SET(wch->act, PLR_HOMEHIDE) )
		sprintf( ht_buf, "unknown     " );
	    else
		sprintf( ht_buf, "%-12s", hometown_table[wch->hometown].ht_name );

            if ( !strcmp(wch->name, "Thollin") )
                sprintf( race_buf, "formless" );
            else if ( !strcmp(wch->name, "Ereptor") )
                sprintf( race_buf, "Titan" );
            else
                sprintf( race_buf, pc_race_table[wch->race].name );

            sprintf( buf, "%s%s is a %s %s of %s power.\n\r"
	                  "%s---==========================================================================---\n\r"
			  "%sAge: %-14s   Hometown: %-12s   Renown: %s\n\r"
	                  "%s---==========================================================================---\n\r"
	                  "%s%s holds the following ranks:\n\r",
                (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_TITLES) : ""),
	            wch->name,
	            race_buf,
	            class_buf,
	            level_msg,
	            (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_BORDERS) : ""),
	            (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : ""),
	            age_buf,
		    ht_buf,
	            renown(wch),
	            (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_BORDERS) : ""),
	            (IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_INFORMATION) : ""),
	            (wch->sex == 0 ? "It" : wch->sex == 1 ? "He" : "She") );

            add_buf( output, buf );
            buf[0] = '\0';

            for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
            {
	        char social_buf[MAX_STRING_LENGTH];
	        char social_class[3];

                if ( IS_NULLSTR(territories[territory].name) )
                    continue;

		if ( wch->pcdata->social_classes[territory] == 5 )
		    continue;

                sprintf( social_class, "%d", wch->pcdata->social_classes[territory] );

                sprintf( social_buf, "  %-30s%-15s %s%s%s\n\r",
                    territories[territory].name,
                    get_social_class(wch, territory),
                    IS_IMMORTAL(ch) ? "(" : "",
                    IS_IMMORTAL(ch) ? social_class : "",
                    IS_IMMORTAL(ch) ? ")" : "" );
                strcat( buf, social_buf );
            }

            add_buf( output, buf );

            buf[0] = '\0';
            for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
            {
                char terr_buf[MAX_STRING_LENGTH];

	        if ( IS_NULLSTR(territories[territory].name)
	        ||   wch->pcdata->territory_flags[territory] == 0 )
	            continue;

                if ( first )
                {
                    sprintf( buf,
                        "%s---==========================================================================---\n\r"
                        "%sPositions:\n\r%s",
                        P_COLOR(ch, C_BORDERS),
                        P_COLOR(ch, C_TITLES),
                        P_COLOR(ch, C_INFORMATION) );
                    first = FALSE;
                }

                sprintf( terr_buf,
                    "  %-30s - %s\n\r",
                    territories[territory].name,
                    territory_bit_name(wch->pcdata->territory_flags[territory]) );
                strcat( buf, terr_buf );
	    }
            add_buf( output, buf );

            buf[0] = '\0';
            first  = TRUE;
            for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
            {
	        CRIME_DATA *crime;

	        if ( IS_NULLSTR(territories[territory].name)
	        ||   wch->pcdata->crimes[territory] == NULL )
	            continue;

                for ( crime = wch->pcdata->crimes[territory]; crime != NULL; crime = crime->next )
                {
                    char crime_buf[MAX_STRING_LENGTH];

                    if ( first )
                    {
                        sprintf( crime_buf,
                            "%s---==========================================================================---\n\r"
                            "%sCrimes:\n\r%s",
                            P_COLOR(ch, C_BORDERS),
                            P_COLOR(ch, C_TITLES),
                            P_COLOR(ch, C_INFORMATION) );
                        strcat( buf, crime_buf );
                        first = FALSE;
                    }

                    sprintf( crime_buf, "  Wanted for %s %s in %s\n\r",
	                    crime->crime == CRIME_PKILLING
	                    ? "the assault of"
	                    : "stealing from",
	                    crime->victim,
	                    territories[territory].name );
                    strcat( buf, crime_buf );
                }
	    }

	    if ( IS_SET(ch->comm, COMM_COLOR) )
		strcat( buf, COLOR_OFF );
	    strcat( buf, "\n\r" );

/*
 *	    class = class_table[wch->class].who_name;
 *	    switch(wch->level)
 *	    {
 *		case MAX_LEVEL - 0 : class = "IMP"; 	break;
 *		case MAX_LEVEL - 1 : class = "CRE";	break;
 *		case MAX_LEVEL - 2 : class = "SUP";	break;
 *		case MAX_LEVEL - 3 : class = "DEI";	break;
 *		case MAX_LEVEL - 4 : class = "GOD";	break;
 *		case MAX_LEVEL - 5 : class = "IMM";	break;
 *		case MAX_LEVEL - 6 : class = "DEM";	break;
 *		case MAX_LEVEL - 7 : class = "ANG";	break;
 *		case MAX_LEVEL - 8 : class = "AVA";	break;
 *	    }
 *
 *	    sprintf(buf, "[%2d %6s %s] %s%s%s%s%s%s%s%s\n\r",
 *		wch->level,
 *		wch->race < MAX_PC_RACE ? pc_race_table[wch->race].who_name
 *					: "     ",
 *		class,
 *	     wch->incog_level >= LEVEL_HERO ? "(Incog) ": "",
 *	     wch->invis_level >= LEVEL_HERO ? "(Wizi) " : "",
 *	     clan_table[wch->clan].who_name,
 *	     IS_SET(wch->comm, COMM_AFK) ? "[AFK] " : "",
 *           IS_SET(wch->act,PLR_KILLER) ? "(KILLER) " : "",
 *           IS_SET(wch->act,PLR_THIEF) ? "(THIEF) " : "",
 *		wch->name, IS_NPC(wch) ? "" : wch->pcdata->title);
 */
            add_buf( output, buf );
	}
    }

    if ( !found )
    {
	send_to_char( "No one of that name is playing.\n\r", ch );
        free_buf( output );
	return;
    }

    page_to_char( buf_string(output), ch );
    free_buf( output );
}


void do_who( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    BUFFER *output;
    DESCRIPTOR_DATA *d;
    int iClass;
    int iRace;
    int iClan;
    int iLevelLower;
    int iLevelUpper;
    int nNumber;
    int nMatch;
    int nChars;
    bool rgfClass[MAX_CLASS];
    bool rgfRace[MAX_PC_RACE];
    bool rgfClan[MAX_CLAN];
    bool fClassRestrict = FALSE;
    bool fClanRestrict = FALSE;
    bool fClan = FALSE;
    bool fRaceRestrict = FALSE;
    bool fImmortalOnly = FALSE;

    /*
     * Set default arguments.
     */
    iLevelLower    = 0;
    iLevelUpper    = MAX_LEVEL;
    for ( iClass = 0; iClass < MAX_CLASS; iClass++ )
        rgfClass[iClass] = FALSE;
    for ( iRace = 0; iRace < MAX_PC_RACE; iRace++ )
        rgfRace[iRace] = FALSE;
    for (iClan = 0; iClan < MAX_CLAN; iClan++)
	rgfClan[iClan] = FALSE;

    /*
     * Parse arguments.
     */
    nNumber = 0;
    for ( ;; )
    {
        char arg[MAX_STRING_LENGTH];

        argument = one_argument( argument, arg );
        if ( arg[0] == '\0' )
            break;

        if ( is_number( arg ) )
        {
        /*
         *  switch ( ++nNumber )
         *  {
         *  case 1: iLevelLower = atoi( arg ); break;
         *  case 2: iLevelUpper = atoi( arg ); break;
         *  default:
         *      send_to_char( "Only two level numbers allowed.\n\r", ch );
         *      return;
         *  }
         */
        }
        else
        {

            /*
             * Look for classes to turn on.
             */
            if (!str_prefix(arg,"immortals"))
            {
                fImmortalOnly = TRUE;
            }
            else
            {
            /*  iClass = class_lookup(arg);
             *  if (iClass == -1)
             *  {
             *      iRace = race_lookup(arg);
 	     *
             *      if (iRace == 0 || iRace >= MAX_PC_RACE)
	     *	    {
	     *		if (!str_prefix(arg,"clan"))
	     *		    fClan = TRUE;
	     *		else
	     *	        {
	     *		    iClan = clan_lookup(arg);
	     *		    if (iClan)
	     *		    {
	     *			fClanRestrict = TRUE;
	     *		   	rgfClan[iClan] = TRUE;
	     *		    }
	     *		    else
	     *		    {
             *          	send_to_char(
             *              	"That's not a valid race, class, or clan.\n\r",
	     *			   ch);
             *              	return;
	     *		    }
             *          }
	     *	    }
             *      else
             *      {
             *          fRaceRestrict = TRUE;
             *          rgfRace[iRace] = TRUE;
             *      }
             *  }
             *  else
             *  {
             *      fClassRestrict = TRUE;
             *      rgfClass[iClass] = TRUE;
             *  }
             */
            }
        }
    }

    /*
     * Now show matching chars.
     */
    nChars = 0;
    nMatch = 0;
    buf[0] = '\0';
    buf2[0] = '\0';
    output = new_buf();
    if(IS_SET(ch->comm, COMM_COLOR))
    {
      strcat( buf2, P_COLOR(ch, C_BORDERS));
    }
    strcat( buf2, "[*>=======================================<*]\n\r" );
    /*strcat( buf2, "[:]                                       [:]\n\r" );*/
    strcat( buf2, "[*]   ");
    if(IS_SET(ch->comm, COMM_COLOR ) )
    {
      strcat( buf2, COLOR_OFF );
      strcat( buf2, P_COLOR(ch, C_TITLES));
    }
    strcat( buf2, "Fellow Wanderers of the Labyrinth");
    if(IS_SET(ch->comm, COMM_COLOR ) )
    {
      strcat( buf2, COLOR_OFF );
      strcat( buf2, P_COLOR(ch, C_BORDERS));
    }

    strcat( buf2, "   [*]\n\r" );
    /*strcat( buf2, "[:]                                       [:]\n\r" );*/
    strcat( buf2, "[*>=======================================<*]\n\r" );

    strcat( buf2, "\n\r" );
    if(IS_SET(ch->comm, COMM_COLOR ) )
	strcat( buf2, COLOR_OFF );
    add_buf(output,buf2);

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
        char *who_name, name_buf[] = "             ";
        CHAR_DATA *wch;
        sh_int length;

 	if ( d->connected == CON_PLAYING )
 	    nChars++;

        /*
         * Check for match against restrictions.
         * Don't use trust as that exposes trusted mortals.
         */
        if ( d->connected != CON_PLAYING || !can_see( ch, d->character ) )
            continue;

        wch   = ( d->original != NULL ) ? d->original : d->character;

	if (!can_see(ch,wch))
	    continue;

        if ( wch->level < iLevelLower
        ||   wch->level > iLevelUpper
        || ( fImmortalOnly  && wch->level < LEVEL_IMMORTAL )
        || ( fClassRestrict && !rgfClass[wch->class] )
        || ( fRaceRestrict && !rgfRace[wch->race])
 	|| ( fClan && !is_clan(wch))
	|| ( fClanRestrict && !rgfClan[wch->clan]))
            continue;

        nMatch++;

        /*
         * Figure out what to print for class.
	 */
	if ( !IS_NPC(wch)
	&&   wch->pcdata->who_name[0] != '\0'
	&&   wch->level >= LEVEL_IMMORTAL )
	    who_name = wch->pcdata->who_name;
	else if ( wch->level >= LEVEL_IMMORTAL )
	{
	    switch ( wch->level )
	    {
	    default: 		 who_name = "Unknown";	break;
	    case MAX_LEVEL - 0 : who_name = "IMP";	break;
	    case MAX_LEVEL - 1 : who_name = "CRE";	break;
	    case MAX_LEVEL - 2 : who_name = "SUP";	break;
	    case MAX_LEVEL - 3 : who_name = "DEI";	break;
	    case MAX_LEVEL - 4 : who_name = "GOD";	break;
	    case MAX_LEVEL - 5 : who_name = "IMM";	break;
	    case MAX_LEVEL - 6 : who_name = "DEM";	break;
	    case MAX_LEVEL - 7 : who_name = "ANG";	break;
	    case MAX_LEVEL - 8 : who_name = "AVA";	break;
	    }
	}
	else if ( wch->level == LEVEL_HERO )
	    who_name = "HERO";
	else if ( wch->clan == 0 || clan_table[wch->clan].clan_hidden )
	    who_name = "Citizen";
	else if ( clan_table[wch->clan].independent )
	    who_name = (wch->sex == 2) ? "Freewoman" : "Freeman";
	else if ( !IS_NPC(wch)
	&&        wch->pcdata->leader
	&&        clan_table[wch->clan].show_leaders )
	    who_name = (wch->pcdata->leader == 5) ? "Leader" : "Deputy";
	else
	    who_name = "Guilded";

	length = strlen( who_name );
	memcpy( &name_buf[(WHO_NAME_LENGTH - strlen(who_name))/2],
	    who_name, length );

	/*
	 * Format it up.
	 */
/*
 *	sprintf( buf, "[%2d %6s %s] %s%s%s%s%s%s%s%s\n\r",
 *	    wch->level,
 *	    wch->race < MAX_PC_RACE ? pc_race_table[wch->race].who_name
 *				    : "     ",
 *	    class,
 *	    wch->incog_level >= LEVEL_HERO ? "(Incog) " : "",
 *	    wch->invis_level >= LEVEL_HERO ? "(Wizi) " : "",
 *	    clan_table[wch->clan].who_name,
 *	    IS_SET(wch->comm, COMM_AFK) ? "[AFK] " : "",
 *          IS_SET(wch->act, PLR_KILLER) ? "(KILLER) " : "",
 *          IS_SET(wch->act, PLR_THIEF)  ? "(THIEF) "  : "",
 *	    wch->name,
 *	    IS_NPC(wch) ? "" : wch->pcdata->title );
 */
	sprintf( buf, "%s[%-13s]%s %s%s %s%s%s%s%s\n\r",
	    P_COLOR(ch, C_BORDERS),
	    name_buf,
	    P_COLOR(ch, C_INFORMATION),
	    wch->name,
	    IS_NPC(wch) ? "" : wch->pcdata->title,
	    wch->incog_level >= LEVEL_HERO ? "(Incog) " : "",
	    wch->invis_level >= LEVEL_HERO ? "(Wizi) " : "",
	    (clan_table[wch->clan].independent ||
              clan_table[wch->clan].clan_hidden)
	        ? "" : clan_table[wch->clan].who_name,
	    IS_SET(wch->comm, COMM_AFK) ? "[AFK] " : "",
	    /*IS_SET(wch->act, PLR_KILLER) ? "(KILLER) " : "",*/
	    /*IS_SET(wch->act, PLR_THIEF)  ? "(THIEF) "  : "",*/
	    (IS_NPC(wch) || wch->pcdata->heresy == 0) ? "" : "(HERETIC)" );
	add_buf(output,buf);
    }

    max_on = UMAX( max_on, nChars );
    sprintf( buf2, "\n\r%sWanderers of the Maze seen: [ %d ]%s\n\r",
	P_COLOR(ch, C_TITLES),
        nMatch,
        COLOR_OFF/*,
	max_on*/ );
    add_buf(output,buf2);
    page_to_char( buf_string(output), ch );
    free_buf(output);
    return;
}

void do_count ( CHAR_DATA *ch, char *argument )
{
    int count;
    DESCRIPTOR_DATA *d;
    char buf[MAX_STRING_LENGTH];

    count = 0;

    for ( d = descriptor_list; d != NULL; d = d->next )
        if ( d->connected == CON_PLAYING && can_see( ch, d->character ) )
	    count++;

    max_on = UMAX(count,max_on);

    if (max_on == count)
        sprintf(buf,"There are %d characters on, the most so far today.\n\r",
	    count);
    else
	sprintf(buf,"There are %d characters on, the most on today was %d.\n\r",
	    count,max_on);

    send_to_char(buf,ch);
}

void do_inventory( CHAR_DATA *ch, char *argument )
{
    send_to_char( "You are carrying:\n\r", ch );
    show_list_to_char( ch->carrying, ch, TRUE, TRUE );
    return;
}



void do_equipment( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    OBJ_DATA *obj;
    int iWear;
    bool found;

    send_to_char( "You are using:\n\r", ch );
    found = FALSE;
    for ( iWear = 0; iWear < MAX_WEAR; iWear++ )
    {
	char *wear_loc;

	if ( ( obj = get_eq_char( ch, iWear ) ) == NULL )
	    continue;

	if ( (wear_loc = find_use_msg(obj, USE_WEAR_LOC)) != NULL )
	{
	    sprintf( buf, "%-30s", wear_loc );
	    send_to_char( buf, ch );
	}
	else
	    send_to_char( where_name[iWear], ch );
	if ( can_see_obj( ch, obj ) )
	{
	    send_to_char( format_obj_to_char( obj, ch, TRUE ), ch );
	    send_to_char( "\n\r", ch );
	}
	else
	{
	    send_to_char( "something.\n\r", ch );
	}
	found = TRUE;
    }

    if ( !found )
	send_to_char( "Nothing.\n\r", ch );

    return;
}



void do_compare( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    OBJ_DATA *obj1;
    OBJ_DATA *obj2;
    int value1;
    int value2;
    char *msg;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );
    if ( arg1[0] == '\0' )
    {
	send_to_char( "Compare what to what?\n\r", ch );
	return;
    }

    if ( ( obj1 = get_obj_carry( ch, arg1, ch ) ) == NULL )
    {
	send_to_char( "You do not have that item.\n\r", ch );
	return;
    }

    if (arg2[0] == '\0')
    {
	for (obj2 = ch->carrying; obj2 != NULL; obj2 = obj2->next_content)
	{
	    if (obj2->wear_loc != WEAR_NONE
	    &&  can_see_obj(ch,obj2)
	    &&  obj1->item_type == obj2->item_type
	    &&  (obj1->wear_flags & obj2->wear_flags & ~ITEM_TAKE) != 0 )
		break;
	}

	if (obj2 == NULL)
	{
	    send_to_char("You aren't wearing anything comparable.\n\r",ch);
	    return;
	}
    }

    else if ( (obj2 = get_obj_carry(ch,arg2,ch) ) == NULL )
    {
	send_to_char("You do not have that item.\n\r",ch);
	return;
    }

    msg		= NULL;
    value1	= 0;
    value2	= 0;

    if ( obj1 == obj2 )
    {
	msg = "You compare $p to itself.  It looks about the same.";
    }
    else if ( obj1->item_type != obj2->item_type )
    {
	msg = "You can't compare $p and $P.";
    }
    else
    {
	switch ( obj1->item_type )
	{
	default:
	    msg = "You can't compare $p and $P.";
	    break;

	case ITEM_ARMOR:
	    value1 = obj1->value[0] + obj1->value[1] + obj1->value[2];
	    value2 = obj2->value[0] + obj2->value[1] + obj2->value[2];
	    break;

	case ITEM_WEAPON:
	    if (obj1->pIndexData->new_format)
		value1 = (1 + obj1->value[2]) * obj1->value[1];
	    else
	    	value1 = obj1->value[1] + obj1->value[2];

	    if (obj2->pIndexData->new_format)
		value2 = (1 + obj2->value[2]) * obj2->value[1];
	    else
	    	value2 = obj2->value[1] + obj2->value[2];
	    break;
	}
    }

    if ( msg == NULL )
    {
	     if ( value1 == value2 ) msg = "$p and $P look about the same.";
	else if ( value1  > value2 ) msg = "$p looks better than $P.";
	else                         msg = "$p looks worse than $P.";
    }

    act( msg, ch, obj1, obj2, TO_CHAR );
    return;
}



void do_credits( CHAR_DATA *ch, char *argument )
{
    do_help( ch, "diku" );
    return;
}



void do_where( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    DESCRIPTOR_DATA *d;
    bool found;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Players near you:\n\r", ch );
	found = FALSE;
	for ( d = descriptor_list; d; d = d->next )
	{
	    if ( d->connected == CON_PLAYING
	    && ( victim = d->character ) != NULL
	    &&   !IS_NPC(victim)
	    &&   victim->in_room != NULL
	    &&   !IS_SET(victim->in_room->room_flags,ROOM_NOWHERE)
 	    &&   (is_room_owner(ch,victim->in_room)
	    ||    !room_is_private(victim->in_room))
	    &&   victim->in_room->area == ch->in_room->area
	    &&   can_see( ch, victim )
            &&   victim->polymorphed == NULL )
	    {
		found = TRUE;
		sprintf( buf, "%-28s %s\n\r",
		    victim->name, victim->in_room->name );
		send_to_char( buf, ch );
	    }
	}
	if ( !found )
	    send_to_char( "None\n\r", ch );
    }
    else
    {
	found = FALSE;
	for ( victim = char_list; victim != NULL; victim = victim->next )
	{
	    if ( victim->in_room != NULL
	    &&   victim->in_room->area == ch->in_room->area
	    &&   !IS_AFFECTED(victim, AFF_HIDE)
	    &&   !IS_AFFECTED(victim, AFF_SNEAK)
	    &&   can_see( ch, victim )
	    &&   is_name( arg, CHAR_NAME(victim) ) )
	    {
		found = TRUE;
		sprintf( buf, "%-28s %s\n\r",
		    PERS(victim, ch), victim->in_room->name );
		send_to_char( buf, ch );
		break;
	    }
	}
	if ( !found )
	    act( "You didn't find any $T.", ch, NULL, arg, TO_CHAR );
    }

    return;
}




void do_consider( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int level_diff;
    int dam_diff;
    int ac_diff;
    int chance;
    char *msg;
    int skill;
    int roll;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Consider killing whom?\n\r", ch );
	return;
    }

    if ( ( victim = get_char_room( ch, arg ) ) == NULL )
    {
	send_to_char( "They're not here.\n\r", ch );
	return;
    }

    if (is_safe(ch,victim))
    {
	send_to_char("Don't even think about it.\n\r",ch);
	return;
    }

    /* Calculate the real differences. */
    level_diff	= victim->level - ch->level;
    if ( IS_AFFECTED2(victim, AFF2_ALKAR) )
    {
	AFFECT_DATA *paf = affect_find( victim->affected, gsn_alkar );

	if ( paf == NULL )
	    bug( "do_consider: missing AFFECT_DATA for AFF2_ALKAR.", 0 );
	else
	    level_diff += paf->modifier;
    }

    ac_diff	=
        (victim->armor[0] - ch->armor[0] + victim->armor[1] - ch->armor[1] +
         victim->armor[2] - ch->armor[2] + victim->armor[3] - ch->armor[3]) / 4;
    dam_diff 	= avg_dam( victim ) - avg_dam( ch );

    /* Get the chances. */
    skill  = get_skill( ch, gsn_consider );
    chance = URANGE( 5, get_curr_stat(ch, STAT_PER)/5 + skill, 95 );
    roll   = number_percent();

    /* If the failed randomize the result. */
    if ( roll > chance )
    {
	int error	= (roll - chance) / 10;
	int change	= number_range( 0, 2 * error );

	level_diff	+= change - error;
	dam_diff	+= change - error;
	ac_diff		+= 10 * (change - error);
    }

    WAIT_STATE( ch, skill_table[gsn_consider].beats );

         if ( level_diff <= -10 ) msg = "You can kill $N naked and weaponless.";
    else if ( level_diff <=  -5 ) msg = "$N is no match for you.";
    else if ( level_diff <=  -2 ) msg = "$N looks like an easy kill.";
    else if ( level_diff <=   1 ) msg = "The perfect match!";
    else if ( level_diff <=   4 ) msg = "$N says 'Do you feel lucky, punk?'.";
    else if ( level_diff <=   9 ) msg = "$N laughs at you mercilessly.";
    else                          msg = "Death will thank you for your gift.";
    act( msg, ch, NULL, victim, TO_CHAR );

    if ( skill <= 25 )
	return;

         if ( ac_diff <= -100 ) msg = "$E appears almost indestructable.";
    else if ( ac_diff <=  -50 ) msg = "$S defenses look fearsome!";
    else if ( ac_diff <=  -20 ) msg = "$E seems to have good defenses.";
    else if ( ac_diff <=   10 ) msg = "$E looks about as well protected as yourself.";
    else if ( ac_diff <=   40 ) msg = "Compared to your own, $Ns defenses seem rather poor.";
    else if ( ac_diff <=   90 ) msg = "Your defenses vastly exceed $Ns.";
    else                        msg = "$Ns defenses are laughable.";
    act ( msg, ch, NULL, victim, TO_CHAR );

    if ( skill <= 50 )
	return;

         if ( dam_diff <= -10 ) msg = "Your offensive capabilities look divine compared to $Ns.";
    else if ( dam_diff <=  -5 ) msg = "$N seems to have a pathetically weak offense.";
    else if ( dam_diff <=  -2 ) msg = "$N doesn't seem to have quite the offensive capabilites that you possess.";
    else if ( dam_diff <=   1 ) msg = "$N would appear about as good as you on attack.";
    else if ( dam_diff <=   4 ) msg = "$N would appear to be slightly better on attack than you.";
    else if ( dam_diff <=   9 ) msg = "$N seems to possess powerful offensive capabilities.";
    else                        msg = "$Ns combat prowess makes yours look childlike.";
    act ( msg, ch, NULL, victim, TO_CHAR );

    return;
}



void set_title( CHAR_DATA *ch, char *title )
{
    char buf[MAX_STRING_LENGTH];

    if ( IS_NPC(ch) )
    {
	bug( "Set_title: NPC.", 0 );
	return;
    }

    if ( title[0] != '.' && title[0] != ',' && title[0] != '!' && title[0] != '?' )
    {
	buf[0] = ' ';
	strcpy( buf+1, title );
    }
    else
    {
	strcpy( buf, title );
    }

    free_string( ch->pcdata->title );
    ch->pcdata->title = str_dup( buf );
    return;
}



void do_title( CHAR_DATA *ch, char *argument )
{
    if ( IS_NPC(ch) )
	return;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Change your title to what?\n\r", ch );
	return;
    }

    if ( strlen(argument) > 45 )
	argument[45] = '\0';

    smash_tilde( argument );
    set_title( ch, argument );
    send_to_char( "Ok.\n\r", ch );
}



void do_description( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];

    if ( argument[0] != '\0' )
    {
	buf[0] = '\0';
	smash_tilde( argument );

    	if (argument[0] == '-')
    	{
            int len;
            bool found = FALSE;

            if (ch->description == NULL || ch->description[0] == '\0')
            {
                send_to_char("No lines left to remove.\n\r",ch);
                return;
            }

  	    strcpy(buf,ch->description);

            for (len = strlen(buf); len > 0; len--)
            {
                if (buf[len] == '\r')
                {
                    if (!found)  /* back it up */
                    {
                        if (len > 0)
                            len--;
                        found = TRUE;
                    }
                    else /* found the second one */
                    {
                        buf[len + 1] = '\0';
			free_string(ch->description);
			ch->description = str_dup(buf);
			send_to_char( "Your description is:\n\r", ch );
			send_to_char( ch->description ? ch->description :
			    "(None).\n\r", ch );
                        return;
                    }
                }
            }
            buf[0] = '\0';
	    free_string(ch->description);
	    ch->description = str_dup(buf);
	    send_to_char("Description cleared.\n\r",ch);
	    return;
        }
	if ( argument[0] == '+' )
	{
	    if ( ch->description != NULL )
		strcat( buf, ch->description );
	    argument++;
	    while ( isspace(*argument) )
		argument++;
	}

        if ( strlen(buf) >= 1024)
	{
	    send_to_char( "Description too long.\n\r", ch );
	    return;
	}

	strcat( buf, argument );
	strcat( buf, "\n\r" );
	free_string( ch->description );
	ch->description = str_dup( buf );
    }

    send_to_char( "Your description is:\n\r", ch );
    send_to_char( ch->description ? ch->description : "(None).\n\r", ch );
    return;
}



void do_report( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_INPUT_LENGTH];

    sprintf( buf,
	"You say 'I have %d/%d hp %d/%d mana %d/%d mv %d xp.'\n\r",
	ch->hit,  ch->max_hit,
	ch->mana, ch->max_mana,
	ch->move, ch->max_move,
	ch->exp   );

    send_to_char( buf, ch );

    sprintf( buf, "$n says 'I have %d/%d hp %d/%d mana %d/%d mv %d xp.'",
	ch->hit,  ch->max_hit,
	ch->mana, ch->max_mana,
	ch->move, ch->max_move,
	ch->exp   );

    act( buf, ch, NULL, NULL, TO_ROOM );

    return;
}



void do_practice( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int sn;

    if ( IS_NPC(ch) )
	return;

    if ( argument[0] == '\0' )
    {
	int col;

	col    = 0;
	for ( sn = 0; sn < max_player_skill; sn++ )
	{
	    if ( skill_table[sn].name == NULL )
		break;
	    if ( ch->level < skill_level(ch, sn)
	    ||   ch->pcdata->learned[sn] < 1 /* skill is not known */)
		continue;

	    sprintf( buf, "%-18s %3d%%  ",
		skill_table[sn].name, ch->pcdata->learned[sn] );
	    send_to_char( buf, ch );
	    if ( ++col % 3 == 0 )
		send_to_char( "\n\r", ch );
	}

	if ( col % 3 != 0 )
	    send_to_char( "\n\r", ch );

	sprintf( buf, "You have %d practice sessions left.\n\r",
	    ch->practice );
	send_to_char( buf, ch );
    }
    else
    {
	CHAR_DATA *mob;
	int adept;

	if ( !IS_AWAKE(ch) )
	{
	    send_to_char( "In your dreams, or what?\n\r", ch );
	    return;
	}

	for ( mob = ch->in_room->people; mob != NULL; mob = mob->next_in_room )
	{
	    if ( IS_NPC(mob) && IS_SET(mob->act, ACT_PRACTICE) )
		break;
	}

	if ( mob == NULL )
	{
	    send_to_char( "You can't do that here.\n\r", ch );
	    return;
	}

	if ( ch->practice <= 0 )
	{
	    send_to_char( "You have no practice sessions left.\n\r", ch );
	    return;
	}

	if ( ( sn = find_spell(ch, argument, NULL) ) < 0
	|| ( !IS_NPC(ch)
	&&   (ch->level < skill_level(ch, sn)
 	||    ch->pcdata->learned[sn] < 1 /* skill is not known */
	||    skill_rating(ch, sn) == 0)) )
	{
	    send_to_char( "You can't practice that.\n\r", ch );
	    return;
	}

	adept = IS_NPC(ch) ? 100 : class_table[ch->class].skill_adept;

	if ( ch->pcdata->learned[sn] >= adept )
	{
	    sprintf( buf, "You are already learned at %s.\n\r",
		skill_table[sn].name );
	    send_to_char( buf, ch );
	}
	else
	{
	    bool caster, non_caster;
	    int mod;

	    mod = int_app[get_curr_stat(ch, STAT_INT)/4].learn /
	          skill_rating( ch, sn );

	    if ( !IS_NPC(ch) )
	    {
		int change, i;

		caster = non_caster = FALSE;
		if ( class_table[ch->class].pMana >= PURE_MAGIC )
		    caster	= TRUE;
		else
		    non_caster	= TRUE;

		for ( i = 0; i < MAX_PROFESSIONS - 1; i++ )
		{
		    if ( ch->pcdata->former_professions[i] == -1 )
			break;

		    if ( class_table[ch->pcdata->former_professions[i]].pMana
			 >= PURE_MAGIC )
			caster		= TRUE;
		    else
			non_caster	= TRUE;
		}

		if ( caster && non_caster )
		    change = UMAX(
			religion_table[GET_GOD(ch)].skill_modifications[
			    skill_table[sn].sclass ].caster.level_mod,
			religion_table[GET_GOD(ch)].skill_modifications[
			    skill_table[sn].sclass ].non_caster.level_mod );
		else if ( caster )
		    change = religion_table[GET_GOD(ch)].skill_modifications[
			skill_table[sn].sclass ].caster.level_mod;
		else
		    change = religion_table[GET_GOD(ch)].skill_modifications[
			skill_table[sn].sclass ].non_caster.level_mod;

		mod = (mod * (100 + change)) / 100;
	    }

	    ch->practice--;
	    ch->pcdata->learned[sn] += mod;
	    if ( ch->pcdata->learned[sn] < adept )
	    {
		act( "You practice $T.",
		    ch, NULL, skill_table[sn].name, TO_CHAR );
		act( "$n practices $T.",
		    ch, NULL, skill_table[sn].name, TO_ROOM );
	    }
	    else
	    {
		ch->pcdata->learned[sn] = adept;
		act( "You are now learned at $T.",
		    ch, NULL, skill_table[sn].name, TO_CHAR );
		act( "$n is now learned at $T.",
		    ch, NULL, skill_table[sn].name, TO_ROOM );
	    }
	}
    }
    return;
}



/*
 * 'Wimpy' originally by Dionysos.
 */
void do_wimpy( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    int wimpy;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
	wimpy = ch->max_hit / 5;
    else
	wimpy = atoi( arg );

    if ( wimpy < 0 )
    {
	send_to_char( "Your courage exceeds your wisdom.\n\r", ch );
	return;
    }

    if ( wimpy > ch->max_hit/2 )
    {
	send_to_char( "Such cowardice ill becomes you.\n\r", ch );
	return;
    }

    ch->wimpy	= wimpy;
    sprintf( buf, "Wimpy set to %d hit points.\n\r", wimpy );
    send_to_char( buf, ch );
    return;
}



void do_password( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    char *pArg;
    char *pwdnew;
    char *p;
    char cEnd;

    if ( IS_NPC(ch) )
	return;

    /*
     * Can't use one_argument here because it smashes case.
     * So we just steal all its code.  Bleagh.
     */
    pArg = arg1;
    while ( isspace(*argument) )
	argument++;

    cEnd = ' ';
    if ( *argument == '\'' || *argument == '"' )
	cEnd = *argument++;

    while ( *argument != '\0' )
    {
	if ( *argument == cEnd )
	{
	    argument++;
	    break;
	}
	*pArg++ = *argument++;
    }
    *pArg = '\0';

    pArg = arg2;
    while ( isspace(*argument) )
	argument++;

    cEnd = ' ';
    if ( *argument == '\'' || *argument == '"' )
	cEnd = *argument++;

    while ( *argument != '\0' )
    {
	if ( *argument == cEnd )
	{
	    argument++;
	    break;
	}
	*pArg++ = *argument++;
    }
    *pArg = '\0';

    if ( arg1[0] == '\0' || arg2[0] == '\0' )
    {
	send_to_char( "Syntax: password <old> <new>.\n\r", ch );
	return;
    }

    if ( strcmp( crypt( arg1, ch->pcdata->pwd ), ch->pcdata->pwd ) )
    {
	WAIT_STATE( ch, 40 );
	send_to_char( "Wrong password.  Wait 10 seconds.\n\r", ch );
	return;
    }

    if ( strlen(arg2) < 5 )
    {
	send_to_char(
	    "New password must be at least five characters long.\n\r", ch );
	return;
    }

    /*
     * No tilde allowed because of player file format.
     */
    pwdnew = crypt( arg2, ch->name );
    for ( p = pwdnew; *p != '\0'; p++ )
    {
	if ( *p == '~' )
	{
	    send_to_char(
		"New password not acceptable, try again.\n\r", ch );
	    return;
	}
    }

    free_string( ch->pcdata->pwd );
    ch->pcdata->pwd = str_dup( pwdnew );
    save_char_obj( ch, FALSE );
    send_to_char( "Ok.\n\r", ch );
    return;
}


void do_name( CHAR_DATA *ch, char *argument )
{
    if ( ch->level < LEVEL_IMMORTAL )
    {
	send_to_char( "Only immortals can set their who name.\n\r", ch );
	return;
    }

    if ( strlen(argument) > 13 )
    {
	send_to_char(
	    "Your who name can be at most 13 characters in length.\n\r", ch );
	return;
    }

    smash_tilde( argument );

    free_string( ch->pcdata->who_name );
    ch->pcdata->who_name = str_dup( argument );
    send_to_char( "Ok.\n\r", ch );
    return;
}


char *print_appearance( CHAR_DATA *ch, CHAR_DATA *viewer )
{
    static char buf[MAX_STRING_LENGTH];
    const char *str;
    const char *i;
    char *point;

    if ( ((str = ch->appearance)[0] != '\0'
    ||    (!IS_NPC(ch)
    &&     !IS_SET(viewer->comm, COMM_BRIEF)
    &&     ch->polymorphed == NULL
    &&     (str = ch->pcdata->appearance)[0] != '\0'))
    &&   ch->position != POS_FIGHTING
    &&   ch->position >  POS_STUNNED )
    {
	point = buf;

	while ( *str != '\0' )
	{
	    if( *str != '%' )
	    {
		*point++ = *str++;
		continue;
	    }

	    ++str;

	    switch ( *str )
	    {
		default:
		    i = "";
		    break;
		case 'x':
		    switch ( ch->sex )
		    {
		        default:	i = "sexless";	break;
		        case 1:		i = "male";	break;
		        case 2:		i = "female";	break;
		    }
		    break;
		case 'c':
		    if ( IS_NPC(ch) )
			i = "mobile";
		    else
			i = class_table[ch->class].name;
		    break;
		case 'a':
		    if ( IS_EVIL(ch) )
			i = "evil";
		    else if ( IS_GOOD(ch) )
			i = "good";
		    else
			i = "neutral";
		    break;
		case 'g':
		    i = religion_table[GET_GOD(ch)].head;
		    break;
		case 'p':
		    i = print_position( ch, viewer, FALSE );
		    break;
		case 'n':
		    i = PERS( ch, viewer );
		    break;
		case 'e':
		    switch ( ch->sex )
		    {
		        default:	i = "sexless";	break;
		        case 1:		i = "he";	break;
		        case 2:		i = "she";	break;
		    }
		    break;
		case 'm':
		    switch ( ch->sex )
		    {
		        default:	i = "it";	break;
		        case 1:		i = "him";	break;
		        case 2:		i = "her";	break;
		    }
		    break;
		case 'r':
		    if ( IS_NPC(ch) || ch->in_room == NULL )
		    {
			i = "";
			break;
		    }

		    i = get_social_class( ch, ch->in_room->area->territory );
		    break;
		case 's':
		    switch ( ch->sex )
		    {
		        default:	i = "its";	break;
		        case 1:		i = "his";	break;
		        case 2:		i = "her";	break;
		    }
		    break;
	    }

	    ++str;

	    while( (*point = *i) != '\0' )
	    {
		++point;
		++i;
	    }
	}

	*point = '\0';

	if ( ch->appearance[0] == '\0' )
	{
	    strcat( buf, " " );
	    strcat( buf, print_position(ch, viewer, TRUE) );
	}
    }
    else
    {
	strcpy( buf, PERS(ch, viewer) );
	if ( !IS_NPC(ch)
        &&   !IS_SET(viewer->comm, COMM_BRIEF)
	&&   ch->position == POS_STANDING
        &&   ch->on == NULL
        &&   ch->polymorphed == NULL )
	    strcat( buf, ch->pcdata->title );

	strcat( buf, " " );
	strcat( buf, print_position(ch, viewer, TRUE) );
    }

    buf[0] = UPPER( buf[0] );
    return buf;
}


char *print_position( CHAR_DATA *ch, CHAR_DATA *viewer, bool punctuate )
{
    static char buf[MAX_STRING_LENGTH];
    char victim[MAX_STRING_LENGTH];
    const char *punctuation;

    switch ( ch->position )
    {
    case POS_DEAD:
	sprintf( buf, "is DEAD%s", punctuate ? "!!" : "" );
	break;
    case POS_MORTAL:
	sprintf( buf, "is mortally wounded%s", punctuate ? "." : "" );
	break;
    case POS_INCAP:
	sprintf( buf, "is incapacitated%s", punctuate ? "." : "" );
	break;
    case POS_STUNNED:
	sprintf( buf, "is lying here stunned%s", punctuate ? "." : "" );
	break;
    case POS_SLEEPING:
	if ( ch->riding != NULL )
	{
	    sprintf( buf, "is riding %s%s",
	        PERS(ch->riding, viewer),
	        punctuate ? "." : "" );
	}
	else if ( ch->on != NULL )
	{
	    if ( IS_SET(ch->on->value[2], SLEEP_AT) )
		sprintf( buf, "is sleeping at %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else if (IS_SET(ch->on->value[2], SLEEP_ON) )
		sprintf( buf, "is sleeping on %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else
		sprintf( buf, "is sleeping in %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	}
	else if ( IS_AFFECTED2(ch, AFF2_CORPSE_FORM) )
	    sprintf( buf, "is here%s", punctuate ? "." : "" );
	else
	    sprintf( buf, "is sleeping here%s", punctuate ? "." : "" );
	break;
    case POS_RESTING:
        if ( ch->riding != NULL )
	{
	    sprintf( buf, "is riding %s%s",
	        PERS(ch->riding, viewer),
	        punctuate ? "." : "" );
	}
	else if ( ch->on != NULL )
	{
	    if ( IS_SET(ch->on->value[2], REST_AT) )
		sprintf( buf, "is resting at %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else if (IS_SET(ch->on->value[2], REST_ON) )
		sprintf( buf, "is resting on %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else
		sprintf( buf, "is resting in %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	}
	else
	    sprintf( buf, "is resting here%s", punctuate ? "." : "" );
	break;
    case POS_SITTING:
	if ( ch->riding != NULL )
	{
	    sprintf( buf, "is riding %s%s",
	        PERS(ch->riding, viewer),
	        punctuate ? "." : "" );
	}
	else if ( ch->on != NULL )
	{
	    if ( IS_SET(ch->on->value[2], SIT_AT) )
		sprintf( buf, "is sitting at %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else if (IS_SET(ch->on->value[2], SIT_ON) )
		sprintf( buf, "is sitting on %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else
		sprintf( buf, "is sitting in %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	}
	else
	    sprintf( buf, "is sitting here%s", punctuate ? "." : "" );
	break;
    case POS_STANDING:
	if ( ch->riding != NULL )
	{
	    sprintf( buf, "is riding %s%s",
	        PERS(ch->riding, viewer),
	        punctuate ? "." : "" );
	}
	else if ( ch->on != NULL )
	{
	    if ( IS_SET(ch->on->value[2], STAND_AT) )
		sprintf( buf, "is standing at %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else if (IS_SET(ch->on->value[2], STAND_ON) )
		sprintf( buf, "is standing on %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	    else
		sprintf( buf, "is standing in %s%s",
		    ch->on->short_descr,
		    punctuate ? "." : "" );
	}
	else
	    sprintf( buf, "is here%s", punctuate ? "." : "" );
	break;
    case POS_FIGHTING:
	if ( ch->fighting == NULL )
	{
	    strcpy( victim, "thin air" );
	    punctuation = "??";
	}
	else if ( ch->fighting == viewer )
	{
	    strcpy( victim, "YOU" );
	    punctuation = "!";
	}
	else if ( ch->in_room == ch->fighting->in_room )
	{
	    strcpy( victim, PERS(ch->fighting, viewer) );
	    punctuation = ".";
	}
	else
	{
	    strcpy( victim, "someone who left" );
	    punctuation = "??";
	}

	sprintf( buf, "is %s%s, fighting %s%s",
	    (ch->riding == NULL) ? "here" : "mounted on ",
	    (ch->riding == NULL) ? "" : PERS(ch->riding, viewer),
	    victim,
	    punctuate ? punctuation : "" );
	break;
    }

    return buf;
}


void do_nomercy( CHAR_DATA *ch, char *argument )
{
    if ( IS_NPC(ch) )
	return;

    if ( IS_SET(ch->act, PLR_NOMERCY) )
    {
	send_to_char("You will now accept surrenders.\n\r",ch);
	REMOVE_BIT(ch->act,PLR_NOMERCY);
    }
    else
    {
	send_to_char("You will now ignore surrenders.\n\r",ch);
	SET_BIT(ch->act,PLR_NOMERCY);
    }
}


void do_homehide( CHAR_DATA *ch, char *argument )
{
    if ( IS_NPC(ch) )
	return;

    if ( IS_SET(ch->act, PLR_HOMEHIDE) )
    {
	send_to_char("Your hometown is now visible to others.\n\r", ch);
	REMOVE_BIT(ch->act,PLR_HOMEHIDE);
    } else {
	send_to_char("Your hometown is no longer visible to others.\n\r", ch);
	SET_BIT(ch->act,PLR_HOMEHIDE);
    }
}

    /*
     * Omni Command - written by:  Prism (snazzy@ssnlink.net) and donated
     * to the Rom Mail list.
     */

void do_omni( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    BUFFER *output;
    DESCRIPTOR_DATA *d;
    int immmatch;
    int mortmatch;
    int hptemp;

    /*
     * Initalize Variables.
     */
    immmatch = 0;
    mortmatch = 0;
    buf[0] = '\0';
    output = new_buf();

    /*
     * Count and output the IMMs.
     */
    if ( ch->level == MAX_LEVEL )
    {
    sprintf( buf, "----Immortals:----\n\r" );
    add_buf( output, buf );
    sprintf( buf, "Name          Trust   Wiz   Incog   Religion\n\r" );
    add_buf( output, buf );

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
	CHAR_DATA *wch;

	if ( d->connected != CON_PLAYING || !can_see(ch, d->character) )
	    continue;

	wch = ( d->original != NULL ) ? d->original : d->character;

	if ( !can_see(ch, wch)
	||   get_trust(wch) > get_trust(ch)
	||   wch->level <= LEVEL_HERO )
	    continue;

	immmatch++;

	sprintf( buf, "%-14s %d     %-2d     %-2d     %s\n\r",
	    wch->name,
	    UMIN(MAX_LEVEL,get_trust(wch)),
	    wch->invis_level,
	    wch->incog_level,
	    religion_table[GET_GOD(wch)].name );
	add_buf( output, buf );
    }
    }

    /*
     * Count and output the Morts.
     */
    sprintf( buf, "\n\r----Mortals:----\n\r" );
    add_buf( output, buf );
    hptemp = 0;

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
	CHAR_DATA *wch;
	char *pch;

	if ( d->connected != CON_PLAYING
	||   !can_see(ch, d->character)
	||   d->character->level > LEVEL_HERO )
	    continue;

	wch = ( d->original != NULL ) ? d->original : d->character;

	if ( !can_see(ch, wch)/* || get_trust(wch) > get_trust(ch)*/ )
	    continue;

	mortmatch++;

	sprintf( buf2, "%s", race_table[wch->race].name );
	buf2[0] = UPPER( buf2[0] );
	for ( pch = buf2; *pch != '\0'; pch++ )
	    if ( *pch == '-' && *(pch+1) != '\0' )
		*(pch+1) = UPPER(*(pch+1) );

	sprintf( buf, "[%2d %-11s] %s the %s of %s.",
	    wch->level,
	    buf2,
	    wch->name,
	    class_table[wch->class].name,
	    religion_table[GET_GOD(wch)].name );
	sprintf( buf2, "%-60s %5d (%5d exp)\n\r",
	    buf,
	    wch->alignment,
	    exp_per_level(wch,wch->pcdata->points) );
	add_buf(output,buf2);
    }

    /*
     * Tally the counts and send the whole list out.
     */
    sprintf( buf2, "\n\rIMMs found: %d\n\r", immmatch );
    add_buf( output, buf2 );
    sprintf( buf2, "Morts found: %d\n\r", mortmatch );
    add_buf( output, buf2 );
    page_to_char( buf_string(output), ch );
    free_buf( output );
    return;
}


void do_nocancel( CHAR_DATA *ch, char *argument )
{
    if ( IS_NPC(ch) )
	return;

    if ( IS_SET(ch->act, PLR_NOCANCEL) )
    {
	send_to_char("The cancellation spell will now always work on you.\n\r",ch);
	REMOVE_BIT(ch->act,PLR_NOCANCEL);
    }
    else
    {
	send_to_char("Only self-cancellations will now work on you.\n\r",ch);
	SET_BIT(ch->act,PLR_NOCANCEL);
    }
}


void do_print (CHAR_DATA *ch, char *argument)
{
    bool had_color;
    int old_pagelen;
    CHAR_DATA *wch;

    if (IS_NPC(ch) || !ch->desc)
	return;

    if (!argument[0]) {
	send_to_char ("Syntax: print <command>\n\r", ch);
	return;
    }

    old_pagelen = ch->lines;
    if (IS_SET(ch->comm, COMM_COLOR))
	had_color = TRUE;
    else
	had_color = FALSE;

    ch->lines = 0;
    REMOVE_BIT (ch->comm, COMM_COLOR);

    send_to_char (VT_OPENPRINT, ch);

    send_to_char ("Printed from the Labyrinth (tamarisk.ab.ca 9000)\n\r", ch);
    send_to_char ("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-\n\r\n\r", ch);
    interpret (ch, argument);

    for (wch = char_list; wch; wch = wch->next)
	if (wch == ch)
	    break;

    if (!wch)
	return;

    if (wch->desc) {
	send_to_char (VT_FF, ch);
	send_to_char (VT_CLOSEPRINT, ch);
    }

    ch->lines = old_pagelen;
    if (had_color)
	SET_BIT (ch->comm, COMM_COLOR);

    send_to_char ("Printed.\n\r", ch);
}


// buki -- bri's universal knowledge interface -- Thollin, 30/12/99
void do_buki( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char buf3[MAX_STRING_LENGTH];
    DESCRIPTOR_DATA *d;
    int immcount = 0;
    int trustcount = 0;
    int mortcount = 0;
    int hptemp;
    char addr[MSL];

    // imms
    sprintf( buf, "%sImmortals:\n\r",
	(IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_TITLES) : ""));
    send_to_char(buf, ch);
    sprintf( buf, "%s-=-=-=-=-=-%s\n\r",
	(IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_BORDERS) : ""),
	(IS_SET(ch->comm, COMM_COLOR) ? P_COLOR_LITERAL(ch, COLOR_OFF) : ""));
    send_to_char(buf, ch);
    sprintf( buf, "Name          Trust Level Wiz Incog Religion   Site\n\r" );
    send_to_char(buf, ch);

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
	CHAR_DATA *wch;

	if ( d->connected != CON_PLAYING || !can_see(ch, d->character) )
	    continue;

	wch = ( d->original != NULL ) ? d->original : d->character;

	if ( get_trust(wch) <= LEVEL_HERO )
	    continue;

	if ( wch->level > LEVEL_HERO )
	    immcount++;
	else
	    trustcount++;

#if defined(unix)
	if ( !str_cmp( d->ident, "???" ) )
	    strcpy( addr, d->host );
	else
	    sprintf( addr, "%s@%s", d->ident, d->host );
#else
	strcpy( addr, d->host );
#endif

	sprintf( buf, "%-15s  %2d    %2d  %2d    %2d %-10s %s\n\r",
	    wch->name,
	    get_trust(wch),
	    wch->level,
	    wch->invis_level,
	    wch->incog_level,
	    religion_table[GET_GOD(wch)].name,
	    addr );

	send_to_char(buf, ch);
    }

    // morts
    sprintf( buf, "\n\r%sMortals:\n\r",
	(IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_TITLES) : ""));
    send_to_char(buf, ch);
    sprintf( buf, "%s-=-=-=-=-%s\n\r",
	(IS_SET(ch->comm, COMM_COLOR) ? P_COLOR(ch, C_BORDERS) : ""),
	(IS_SET(ch->comm, COMM_COLOR) ? P_COLOR_LITERAL(ch, COLOR_OFF) : ""));
    send_to_char(buf, ch);
    hptemp = 0;

    for ( d = descriptor_list; d != NULL; d = d->next )
    {
	CHAR_DATA *wch;
	char *pch;

	if ( d->connected != CON_PLAYING
	||   !can_see(ch, d->character)
	||   d->character->level > LEVEL_HERO )
	    continue;

	wch = ( d->original != NULL ) ? d->original : d->character;

	if ( !can_see(ch, wch)/* || get_trust(wch) > get_trust(ch)*/ )
	    continue;

	mortcount++;

	sprintf( buf2, "%s", race_table[wch->race].name );
	buf2[0] = UPPER( buf2[0] );
	for ( pch = buf2; *pch != '\0'; pch++ )
	    if ( *pch == '-' && *(pch+1) != '\0' )
		*(pch+1) = UPPER(*(pch+1) );

#if defined(unix)
	if ( !str_cmp( d->ident, "???" ) )
	    strcpy( addr, d->host );
	else
	    sprintf( addr, "%s@%s", d->ident, d->host );
#else
	strcpy( addr, d->host );
#endif

	if ( strlen( addr ) > 34 )
	    addr[34] = '\0';

	sprintf( buf3, "%s, %s %s of %s.",
	    wch->name,
	    sex_table[wch->sex].name,
	    class_table[wch->class].name,
	    religion_table[GET_GOD(wch)].name );

	sprintf( buf, "[%2d %-11s] %-49s (%5d exp)\n\r"
		      "                 On from: %-34s %5d (%5d tnl)\n\r",
	    wch->level,
	    buf2,
	    buf3,
	    exp_per_level(wch,wch->pcdata->points),
	    addr,
	    wch->alignment,
	    (!IS_NPC(wch) && wch->level < LEVEL_HERO)
		? ((wch->level + 1) * exp_per_level(wch,wch->pcdata->points) - wch->exp)
		: 0 );
	send_to_char(buf, ch);
    }

    // totals
    sprintf( buf, "\n\rImms: %d   Trusted morts: %d   Total morts: %d\n\r",
	immcount, trustcount, mortcount);
    send_to_char(buf, ch);
    return;
}


void look_checkers( CHAR_DATA *ch, OBJ_DATA *obj )
{
    unsigned int i, j, k, a, b, c, d;
    char buf[MAX_STRING_LENGTH];

    send_to_char( "The checkers board is set up as follows: \n\r", ch );

    // I used literal colors because you can't exactly configure the colors on a
    // chessboard. They're red and black. ;) -Bri

    sprintf(buf, "   %s1   2   3   4   5   6   7   8\n\r", P_COLOR_LITERAL(ch, COLOR_BLUE) );
    send_to_char(buf, ch);
    send_to_char( "  ---------------------------------\n\r", ch );

    j = 1; k = 1;
    for ( i=1;i<=4;i++ )
    {
        a = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        a += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        a += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
        b = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        b += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        b += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
        c = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        c += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        c += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
        d = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        d += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        d += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;

        sprintf( buf, "%d | %s%s%s |   | %s%s%s |   | %s%s%s |   | %s%s%s |   |\n\r",
                 k,
		 ( a >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( a >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( a >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( a >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( a >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( a >= 4 ) ? "K" :
                 ( a >= 3 ) ? "*" :
                 ( a >= 2 ) ? "K" :
                 ( a >= 1 ) ? "*" :
                 ( a >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE),

		 ( b >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( b >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( b >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( b >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( b >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( b >= 4 ) ? "K" :
                 ( b >= 3 ) ? "*" :
                 ( b >= 2 ) ? "K" :
                 ( b >= 1 ) ? "*" :
                 ( b >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE),

		 ( c >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( c >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( c >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( c >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( c >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( c >= 4 ) ? "K" :
                 ( c >= 3 ) ? "*" :
                 ( c >= 2 ) ? "K" :
                 ( c >= 1 ) ? "*" :
                 ( c >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE),

		 ( d >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( d >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( d >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( d >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( d >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( d >= 4 ) ? "K" :
                 ( d >= 3 ) ? "*" :
                 ( d >= 2 ) ? "K" :
                 ( d >= 1 ) ? "*" :
                 ( d >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE) );

        send_to_char( buf, ch );
	send_to_char( "  ---------------------------------\n\r", ch );
        k++;

        a = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        a += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        a += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
        b = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        b += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        b += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
        c = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        c += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        c += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
/*	if (i == 4) {
        d = ( IS_SET( obj->value[3], A ) ) ? 1 : 0;
        d += ( IS_SET( obj->value[3], B ) ) ? 2 : 0;
        d += ( IS_SET( obj->value[3], C ) ) ? 4 : 0;
        j *= 2;
	} else { */
        d = ( IS_SET( obj->value[0], j ) ) ? 1 : 0;
        d += ( IS_SET( obj->value[1], j ) ) ? 2 : 0;
        d += ( IS_SET( obj->value[2], j ) ) ? 4 : 0;
        j *= 2;
//	}

        sprintf( buf, "%d |   | %s%s%s |   | %s%s%s |   | %s%s%s |   | %s%s%s |\n\r",
                 k,
		 ( a >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( a >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( a >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( a >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( a >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( a >= 4 ) ? "K" :
                 ( a >= 3 ) ? "*" :
                 ( a >= 2 ) ? "K" :
                 ( a >= 1 ) ? "*" :
                 ( a >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE),

		 ( b >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( b >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( b >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( b >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( b >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( b >= 4 ) ? "K" :
                 ( b >= 3 ) ? "*" :
                 ( b >= 2 ) ? "K" :
                 ( b >= 1 ) ? "*" :
                 ( b >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE),

		 ( c >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( c >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( c >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( c >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( c >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( c >= 4 ) ? "K" :
                 ( c >= 3 ) ? "*" :
                 ( c >= 2 ) ? "K" :
                 ( c >= 1 ) ? "*" :
                 ( c >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE),

		 ( d >= 4 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( d >= 3 ) ? P_COLOR_LITERAL(ch, COLOR_LIGHTRED) :
		 ( d >= 2 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( d >= 1 ) ? P_COLOR_LITERAL(ch, COLOR_GREY) :
		 ( d >= 0 ) ? "" : P_COLOR_LITERAL(ch, COLOR_YELLOW),
                 ( d >= 4 ) ? "K" :
                 ( d >= 3 ) ? "*" :
                 ( d >= 2 ) ? "K" :
                 ( d >= 1 ) ? "*" :
                 ( d >= 0 ) ? " " : "1",
		 P_COLOR_LITERAL(ch, COLOR_BLUE) );


        send_to_char( buf, ch );
        if ( i < 4 )
            send_to_char( "  ---------------------------------\n\r", ch );
        else {
	    sprintf(buf, "  ---------------------------------%s\n\r", P_COLOR_OFF(ch) );
	    send_to_char(buf, ch);
	}
        k++;
    }
}


void disp_customizations( CHAR_DATA *ch, CHAR_DATA *victim, const char *title, int min, int max, const struct advantage_type *table )
{
    char buf1[ MAX_STRING_LENGTH ];
    char buf2[ MAX_STRING_LENGTH ];
    CUST_DATA *cust;

    buf1[0] = '\0';
    for ( cust = victim->pcdata->customizations;
          cust != NULL;
          cust = cust->next )
    {
        if ( cust->type >= min && cust->type <= max )
        {
            if ( IS_NULLSTR(buf1) )
                sprintf( buf1, "%s\n\r", title );
            sprintf( buf2, " %s\n\r", table[cust->index].name );
            strcat( buf1, buf2 );
        }
    }
    send_to_char( buf1, ch );
}


void do_customizations( CHAR_DATA *ch, char *argument )
{
    CHAR_DATA *victim;

    if ( !IS_NULLSTR(argument) )
    {
        if ( !trusted_command(ch, "stat") )
        {
            send_to_char( "Huh?\n\r", ch );
            return;
        }

        victim = get_char_world( ch, argument );
        if ( victim == NULL )
        {
            send_to_char( "You can't find them.\n\r", ch );
            return;
        }
    }
    else
        victim = ch;

    if ( IS_NPC(victim) )
    {
        send_to_char( "You can only check the customizations on a player.\n\r", ch );
        return;
    }

    disp_customizations( ch, victim, "Advantages", MIN_ADVANTAGE, MAX_ADVANTAGE, advantage_table );
    disp_customizations( ch, victim, "Disadvantages", MIN_DISADVANTAGE, MAX_DISADVANTAGE, disadvantage_table );
    disp_customizations( ch, victim, "Ancestors", MIN_ANCESTOR, MAX_ANCESTOR, ancestor_table );
    send_to_char( "\n\r", ch );
}


// EOF
