/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/
 
#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "color.h"

/*
 * This structure maps color escapes to color names.
 */
struct color_map_data color_map[] = {
/* color escape,        color name,       formatted color name    */
   { COLOR_OFF,           "Off",            "     Off    ", },
   { COLOR_BLACK,         "Black",          "    Black   ", },
   { COLOR_RED,           "Red",            "     Red    ", },
   { COLOR_GREEN,         "Green",          "    Green   ", },
   { COLOR_BROWN,         "Brown",          "    Brown   ", },
   { COLOR_BLUE,          "Blue",           "     Blue   ", },
   { COLOR_MAGENTA,       "Magenta",        "   Magenta  ", },
   { COLOR_CYAN,          "Cyan",           "     Cyan   ", },
   { COLOR_LIGHTGREY,     "LightGrey",      "  LightGrey ", },

   { COLOR_GREY,          "Grey",           "    Grey    ", },
   { COLOR_LIGHTRED,      "LightRed",       "  LightRed  ", },
   { COLOR_LIGHTGREEN,    "LightGreen",     " LightGreen ", },
   { COLOR_YELLOW,        "Yellow",         "    Yellow  ", },
   { COLOR_LIGHTBLUE,     "LightBlue",      "  LightBlue ", },
   { COLOR_LIGHTMAGENTA,  "LightMagenta",   "LightMagenta", },
   { COLOR_LIGHTCYAN,     "LightCyan",      "  LightCyan ", },
   { COLOR_WHITE,         "White",          "    White   ", }
};

/*
 * This structure defines player configurable colors.
 */
struct color_config_data color_config[MAX_COLOR] = {
/*   color name,	default color,     minimum level     */
   { "Auction",		COLOR_LIGHTMAGENTA,	0 },
   { "Bamf_InOut",	COLOR_WHITE,		0 },
   { "Borders",		COLOR_LIGHTBLUE,	0 },
   { "Clantalk",	COLOR_LIGHTMAGENTA,	0 },
   { "Question",	COLOR_LIGHTMAGENTA,	0 },
   { "Combat_Self",     COLOR_LIGHTGREY,        0 },
   { "Combat_Mobs",     COLOR_CYAN,             0 },
   //{ "Effects",         COLOR_CYAN,             0 },
   //{ "Flags",           COLOR_WHITE,            0 },
   { "Religion",	COLOR_LIGHTMAGENTA,	0 },
   { "Gossip",		COLOR_LIGHTMAGENTA,	0 },
   { "Grats",		COLOR_LIGHTMAGENTA,	0 },
   { "Grouptell",	COLOR_LIGHTMAGENTA,	0 },
   { "Information",	COLOR_LIGHTGREY,	0 },
   { "Mobiles",		COLOR_LIGHTCYAN,	0 },
   { "Music",		COLOR_LIGHTMAGENTA,	0 },
   { "Objects",		COLOR_LIGHTBLUE,	0 },
   { "Prompt",		COLOR_WHITE,		0 },
   { "Players",		COLOR_CYAN,             0 },
   { "Quote",		COLOR_LIGHTMAGENTA,	0 },
   { "Room_Titles",	COLOR_WHITE,		0 },
   { "Say",		COLOR_LIGHTGREY,	0 },
   { "Shout",		COLOR_LIGHTGREY,	2 },
   { "Tell",		COLOR_LIGHTRED,	        0 },
   { "Titles",		COLOR_WHITE,		0 },
   { "Combat_Player",   COLOR_GREY,             0 },
   //{ "Whisper",         COLOR_BLUE,             0 },
   { "Yell",		COLOR_RED,		0 },
   { "Immtalk",		COLOR_GREEN,		LEVEL_IMMORTAL	},
   { "Imptalk",		COLOR_GREEN,		SUPREME		},
   { "Wiznet",		COLOR_YELLOW,		LEVEL_IMMORTAL	},
   { "Cretalk",		COLOR_GREEN,		0 },
   { "Orate",		COLOR_GREEN,	        0 },
   { NULL }
};

/*
 * Gets a color name from an escape.
 */
char *color_lookup_escape( char *escape )
{
    int i;

    for ( i = 0; i < NUM(color_map); ++i )
    {
	if( !str_cmp( color_map[i].color_escape, escape ) )
	    return color_map[i].color_name;
    }
    return NULL;
}


/*
 * Gets an escape from a color name.
 */
char *color_lookup_color( char *name )
{
    int i;

    for( i = 0; i < NUM(color_map); ++i )
    {
	if( !str_prefix(name, color_map[i].color_name) )
	    return color_map[i].color_escape;
    }
    return NULL;
}

/*
 * Gets an configurable color index from a string.
 */
int color_lookup_config_index( char *name )
{
    int i;

    for( i = 0; i < NUM(color_config); ++i )
    {
	if( !str_prefix(name, color_config[i].config_name) )
	    return i;
    }
    return -1;
}


void do_ansi( CHAR_DATA *ch, char *argument )
{
    send_to_char( "Use the color command to modify your color settings.\n\r", ch );
    return;
}


/*
 * Handle `color' command.
 */
void do_color( CHAR_DATA *ch, char *argument )
{
    char which[MAX_STRING_LENGTH];
    char color[MAX_STRING_LENGTH];
    char arg[MAX_STRING_LENGTH];
    char buf[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char buf3[MAX_STRING_LENGTH];
    char buf4[MAX_STRING_LENGTH];
    char *escape;
    int curr, i;
    int index;

    if( argument[0] == '\0' )
    {
	if( !IS_SET( ch->comm, COMM_COLOR ) )
	{
	    send_to_char( "Colors are currently OFF.\n\r", ch );
	    return;
	}
	
	sprintf( buf,
	    "%s[*>==================================================================<*]%s\n\r",
	    P_COLOR(ch, C_BORDERS), P_COLOR_OFF(ch) );
	send_to_char( buf, ch );
	sprintf( buf,
	    "%s[:]                   %s%sCurrent Color Configuration                    %s%s[:]%s\n\r",
	    P_COLOR(ch, C_BORDERS), P_COLOR_OFF(ch), P_COLOR(ch, C_TITLES),
	    P_COLOR_OFF(ch), P_COLOR(ch, C_BORDERS), P_COLOR_OFF(ch) );
	send_to_char( buf, ch );
	sprintf( buf,
	    "%s[*>==============<*>=================================================<*]%s\n\r",
	    P_COLOR(ch, C_BORDERS), P_COLOR_OFF(ch) );
	send_to_char( buf, ch );
	
	curr = 0;
	for( i = 0; i < 16; i++ )
	{
	    while ( color_config[curr].config_min_level > get_trust( ch ) )
		curr++;
	    sprintf( buf2, "%s%-14s%s",
	        (curr < NUM(color_config) - 1) ? P_COLOR(ch, curr) : "",
	        (curr < NUM(color_config) - 1)
	         ? color_config[curr].config_name
	         : "              ",
	        P_COLOR_OFF(ch) );
	    curr++;

	    while ( color_config[curr].config_min_level > get_trust( ch ) )
		curr++;
	    sprintf( buf3, "%s%-14s%s",
	        (curr < NUM(color_config) - 1) ? P_COLOR(ch, curr) : "",
	        (curr < NUM(color_config) - 1)
	         ? color_config[curr].config_name
	         : "              ",
		P_COLOR_OFF(ch) );
	    curr++;

	    while( color_config[curr].config_min_level > get_trust( ch ) )
		curr++;
	    sprintf(buf4, "%s%-14s%s",
	        (curr < NUM(color_config) - 1) ? P_COLOR(ch, curr) : "",
	        (curr < NUM(color_config) - 1)
	         ? color_config[curr].config_name
	         : "              ",
		P_COLOR_OFF(ch) );
	    curr++;

	    sprintf( buf,
	        "%s%s%s%s %s %s%s%s%s%s   %-14s %-14s %-14s  %s%s%s%s\n\r",
	        P_COLOR(ch, C_BORDERS),
	        (i % 2) ? "[:]" : "[*]",
	        P_COLOR_OFF(ch),
	        color_map[i].color_escape,
	        color_map[i].color_formatted,
	        P_COLOR_OFF(ch),
	        P_COLOR(ch, C_BORDERS),
	        (i%2) ? "[:]" : "[*]",
	        P_COLOR_OFF(ch),
	        P_COLOR(ch, C_INFORMATION),
	        buf2,
	        buf3,
	        buf4,
	        P_COLOR_OFF(ch),
	        P_COLOR(ch, C_BORDERS),
	        (i%2) ? "[:]" : "[*]",
	        P_COLOR_OFF(ch) );
   
	    send_to_char(buf, ch);
	}

	sprintf( buf,
	    "%s[*>==============<*>=================================================<*]%s\n\r",
	    P_COLOR(ch, C_BORDERS),
	    P_COLOR_OFF(ch) );
	send_to_char( buf, ch );
	
	return;
    }

    argument = one_argument( argument, arg );

    /* Turn colors on and off */
    if( !str_prefix( arg, "on" ) )
    {
	send_to_char( "Colors are now on.\n\r", ch );
	SET_BIT( ch->comm, COMM_COLOR );
	return;
    }

    if( !str_prefix( arg, "off" ) )
    {
	send_to_char( "Colors are now off.\n\r", ch );
	REMOVE_BIT( ch->comm, COMM_COLOR );
	return;
    }
      
    if( !str_prefix(arg, "set") )
    {
	if( argument[0] == '\0' )
	{
	    send_to_char( "Set which color? To what?\n\r", ch );
	    return;
	}

	argument	= one_argument( argument, which );
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set which color? To what?\n\r", ch );
	    return;
	}
	argument = one_argument( argument, color );

	index	= color_lookup_config_index( which );
	escape	= color_lookup_color( color );

	if ( index == -1 || escape == NULL )
	{
	    send_to_char( "What? Type `color' for a list of colors.\n\r", ch );
	    return;
	}

	sprintf( buf, "Changing color \"%s\" to \"%s\".\n\r", which, color );
	send_to_char( buf, ch );

	ch->pcdata->colors[index] = escape;
	return;
    }
    
    /* A simple color test */
    if( !str_prefix(arg, "test") )
    {
	int i;

	for( i = 0; i < NUM(color_map); ++i )
	{
	    sprintf( buf, "%s%s%s\n\r", P_COLOR_LITERAL( ch, color_map[i].color_escape ),
	    color_map[i].color_name, P_COLOR_LITERAL( ch, COLOR_OFF ) );
	    send_to_char( buf, ch );
	}
	send_to_char( "Ok.\n\r", ch );
	return;
    }
    
    if( !str_cmp( arg, "dump" ) )
    {
	int i;

	for( i = 0; i < NUM(color_config) - 1; ++i )
	{
/*
 *	    sprintf( buf, "%s%s%s\n\r",
 *	        P_COLOR( ch, i ),
 *	        color_config[i].config_name,
 *	        P_COLOR_LITERAL(ch, COLOR_OFF) );
 *	    send_to_char( buf, ch );
 */			
	    act_color( color_config[i].config_name,
	        ch, NULL, NULL, TO_CHAR, POS_DEAD, i );
	}

	return;
    }

   send_to_char( "What? Type `color' for a list.\n\r", ch );
   return;
}

