/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "crash.h"
#include "interp.h"


bool	check_social	args( ( CHAR_DATA *ch, char *command,
				char *argument ) );



/*
 * Command logging types.
 */
#define LOG_NORMAL	0
#define LOG_ALWAYS	1
#define LOG_NEVER	2



/*
 * Log-all switch.
 */
bool				fLogAll		= FALSE;



/*
 * Command table.
 */
const	struct	cmd_type	cmd_table	[] =
{
    /*
     * Common movement commands.
     */
    { "north",		do_north,	POS_STANDING,    0, 0,  LOG_NEVER, 0, 1, 0 },
    { "east",		do_east,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "south",		do_south,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "west",		do_west,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "up",		do_up,		POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "down",		do_down,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "neast",		do_northeast,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "seast",		do_southeast,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "swest",		do_southwest,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "nwest",		do_northwest,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },


    /*
     * Common other commands.
     * Placed here so one and two letter abbreviations work.
     */
    { "at",             do_at,          POS_DEAD,       L8, 0,	LOG_NORMAL, 1, 0, 0 },
    { "cast",		do_cast,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "cwise",		do_clockwise,	POS_STANDING,	 0, 0,  LOG_NEVER, 0, 1, 0 },
    { "ccwise",	do_counterclockwise,	POS_STANDING,	 0, 0,	LOG_NEVER, 0, 1, 0 },
    { "auction",        do_auction,     POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "buy",		do_buy,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
//    { "channels",       do_channels,    POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "dislodge",	do_dislodge,	POS_RESTING,	 0, 0,	LOG_NORMAL, 1, 1, 1 },
    { "dismount",       do_dismount,    POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "drink",		do_drink,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "draw",		do_draw,	POS_STANDING,	 0, 0,	LOG_NORMAL, 1, 1, 1 },
    { "exits",		do_exits,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "flee",		do_flee,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "fire",		do_fire,	POS_STANDING,	 0, 0,	LOG_NORMAL, 1, 1, 1 },
    { "get",		do_get,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "goto",		do_goto,	POS_DEAD,	IM, 1,	LOG_NORMAL, 1, 0, 0 },
    { "mgoto",		do_mgoto,	POS_DEAD,	IM, 1,	LOG_NORMAL, 1, 0, 0 },
    { "group",          do_group,       POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "guild",		do_guild,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "hit",		do_kill,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "inventory",	do_inventory,   POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "kill",		do_kill,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "look",		do_look,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "clan",		do_clantalk,    POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "mount",          do_ride,   	POS_STANDING,    0, 0,  LOG_NORMAL, 1, 1, 1 }, 
    { "music",          do_music,       POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 0, 0 }, 
    { "order",		do_order,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "practice",       do_practice,    POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "rest",		do_rest,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "ride",		do_ride,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "sing",		do_sing,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "sit",		do_sit,		POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "sockets",        do_sockets,	POS_DEAD,       L4, 0,  LOG_NORMAL, 1, 0, 0 },
    { "stand",		do_stand,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "tell",		do_tell,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "toggles",	do_toggle,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "unlock",         do_unlock,      POS_RESTING,     0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "wield",		do_wear,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "wizhelp",	do_wizhelp,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },

    /*
     * Informational commands.
     */
    { "adverbs",	do_adverbs,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "affects",	do_affects,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "areas",		do_areas,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "bugs",		do_bugs,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "changes",	do_changes,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "commands",	do_commands,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "compare",	do_compare,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "consider",	do_consider,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "count",		do_count,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "credits",	do_credits,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "customizations", do_customizations,
                                        POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "equipment",	do_equipment,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "examine",	do_examine,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
/*  { "groups",		do_groups,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "help",		do_help,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "idea",		do_idea,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "info",           do_groups,      POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "motd",		do_motd,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "news",		do_news,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "read",		do_read,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 0, 1 },
    { "report",		do_report,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "rules",		do_rules,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "score",		do_score,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "scan",		do_scan,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "skills",		do_skills,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "socials",	do_socials,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "show",		do_show,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "spells",		do_spells,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "story",		do_story,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "time",		do_time,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "typo",		do_typo,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "weather",	do_weather,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "who",		do_who,		POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "whois",		do_whois,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "wizlist",	do_wizlist,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "worth",		do_worth,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },

    /*
     * Configuration commands.
     */
    { "alia",		do_alia,	POS_DEAD,	 0, 0,  LOG_NORMAL, 0, 0, 0 },
    { "alias",		do_alias,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "ansi",		do_ansi,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "appearance",	do_appearance,	POS_DEAD,	 0, 0,  LOG_ALWAYS, 1, 0, 0 },
//    { "autolist",	do_autolist,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "autoassist",	do_autoassist,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "autoexit",	do_autoexit,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "autogold",	do_autogold,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "autoloot",	do_autoloot,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "autosac",	do_autosac,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "autosplit",	do_autosplit,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "brief",		do_brief,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
/*  { "channels",	do_channels,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "color",		do_color,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "colour",		do_color,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "combine",	do_combine,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "compact",	do_compact,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "description",	do_description,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "delet",		do_delet,	POS_DEAD,	 0, 0,  LOG_ALWAYS, 0, 0, 0 },
    { "delete",		do_delete,	POS_STANDING,	 0, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "homehide",	do_homehide,	POS_DEAD,	 0, 0,	LOG_NORMAL, 1, 0, 0 },
    { "name",		do_name,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 }, 
    { "nocancel",	do_nocancel,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "nofollow",	do_nofollow,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "noloot",		do_noloot,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "nomercy",	do_nomercy,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "nosummon",	do_nosummon,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "outfit",		do_outfit,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "password",	do_password,	POS_DEAD,	 0, 0,  LOG_NEVER,  1, 0, 0 },
    { "prompt",		do_prompt,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "scroll",		do_scroll,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "sound",	        do_sound,	POS_DEAD,        0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "style",		do_style,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "title",		do_title,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "unalias",	do_unalias,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "wimpy",		do_wimpy,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },

    /*
     * Communication commands.
     */
    { "afk",		do_afk,		POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "answer",		do_answer,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
/*  { "auction",	do_auction,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "deaf",		do_deaf,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "emote",		do_emote,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "pmote",		do_pmote,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { ".",		do_gossip,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 0, 0, 0 },
    { "gossip",		do_gossip,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { ",",		do_emote,	POS_RESTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "grats",		do_grats,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "gtell",		do_gtell,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { ";",		do_gtell,	POS_DEAD,	 0, 0,  LOG_NORMAL, 0, 0, 0 },
    { "murmur",		do_murmur,	POS_RESTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
/*  { "music",		do_music,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "note",		do_note,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "orate",		do_orate,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "pose",		do_pose,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "print",		do_print,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "priority",	do_priority,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "question",	do_question,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "quote",		do_quote,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "quiet",		do_quiet,	POS_SLEEPING, 	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "religion",	do_religion,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "reply",		do_reply,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "replay",		do_replay,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "say",		do_say,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "'",		do_say,		POS_RESTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "shout",		do_shout,	POS_RESTING,	 3, 0,  LOG_NORMAL, 1, 1, 1 },
    { "unread",		do_unread,	POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "whisper",	do_whisper,	POS_RESTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "yell",		do_yell,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },

    /*
     * Object manipulation commands.
     */
    { "brandish",	do_brandish,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "close",		do_close,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "donate",		do_donate,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "detrap",		do_detrap,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "drop",		do_drop,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "eat",		do_eat,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "envenom",	do_envenom,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "fill",		do_fill,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "give",		do_give,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "heal",		do_heal,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "hold",		do_wear,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "list",		do_list,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "lock",		do_lock,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "lore",		do_lore,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "open",		do_open,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "pick",		do_pick,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "pour",		do_pour,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "put",		do_put,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "quaff",		do_quaff,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "recite",		do_recite,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "remove",		do_remove,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "second",		do_second,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "sell",		do_sell,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "take",		do_get,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "sacrifice",	do_sacrifice,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "junk",           do_sacrifice,   POS_RESTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
/*  { "tap",      	do_sacrifice,   POS_RESTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 }, */
/*  { "unlock",		do_unlock,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 }, */
    { "value",		do_value,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "wear",		do_wear,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "zap",		do_zap,		POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },

    /*
     * Combat commands.
     */
    { "backstab",	do_backstab,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "bash",		do_bash,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "bite",		do_bite,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "bs",		do_backstab,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "berserk",	do_berserk,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "circle",		do_circle,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "dirt",		do_dirt,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "disarm",		do_disarm,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "eagleclaw",	do_eagle_claw,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "escape",		do_escape,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "joust",		do_joust,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "kick",		do_kick,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "kipunch",	do_ki_punch,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "kishout",	do_ki_shout,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "murde",		do_murde,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "murder",		do_murder,	POS_FIGHTING,	 5, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "rescue",		do_rescue,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "trample",	do_trample,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "sap",		do_sap,		POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "surrender",	do_surrender,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "trip",		do_trip,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "unmount",	do_unmount,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },

    /*
     * Mob command interpreter (placed here for faster scan...)
     */
    { "mc",		do_mob,		POS_DEAD,	 0, 0,  LOG_NEVER,  0, 0, 1 },

    /*
     * Miscellaneous commands.
     */
    { "accept",		do_accept,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "defrock",	do_defrock,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "demote",		do_demote,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "deputize",	do_deputize,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "enter", 		do_enter, 	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "follow",		do_follow,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 0 },
    { "forage",		do_forage,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "gain",		do_gain,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "go",		do_enter,	POS_STANDING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
/*  { "group",		do_group,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "groups",		do_groups,	POS_SLEEPING,    0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "hide",		do_hide,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "petition",	do_petition,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "play",		do_play,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
/*  { "practice",	do_practice,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 }, */
    { "promote",	do_promote,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "qui",		do_qui,		POS_DEAD,	 0, 0,  LOG_NORMAL, 0, 0, 1 },
    { "quit",		do_quit,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 1 },
    { "recall",		do_recall,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "/",		do_recall,	POS_FIGHTING,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "rent",		do_rent,	POS_DEAD,	 0, 0,  LOG_NORMAL, 0, 1, 1 },
    { "save",		do_save,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "sleep",		do_sleep,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "sneak",		do_sneak,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "split",		do_split,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "steal",		do_steal,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "tail",		do_tail,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "track",		do_track,	POS_STANDING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "train",		do_train,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "visible",	do_visible,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "wake",		do_wake,	POS_SLEEPING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "where",		do_where,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "whereami",	do_whereami,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },

    /*
     * Banking commands.
     */
    { "balance",	do_balance,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "cbalance",	do_cbalance,	POS_RESTING,	 0, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "cdeposit",	do_cdeposit,	POS_RESTING,	 0, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "cwithdraw",	do_cwithdraw,	POS_RESTING,	 0, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "deposit",	do_deposit,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },
    { "tbalance",	do_tbalance,	POS_RESTING,	 0, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "tdeposit",	do_tdeposit,	POS_RESTING,	 0, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "twithdraw",	do_twithdraw,	POS_RESTING,	 0, 0,  LOG_ALWAYS, 1, 1, 1 },
    { "withdraw",	do_withdraw,	POS_RESTING,	 0, 0,  LOG_NORMAL, 1, 1, 1 },

    /*
     * Immortal commands.
     */
    { "advance",	do_advance,	POS_DEAD,	ML, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "dump",		do_dump,	POS_DEAD,	ML, 0,  LOG_ALWAYS, 0, 0, 0 },
    { "trust",		do_trust,	POS_DEAD,	ML, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "violate",	do_violate,	POS_DEAD,	ML, 5,  LOG_ALWAYS, 1, 0, 0 },

    { "allow",		do_allow,	POS_DEAD,	L2, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "ban",		do_ban,		POS_DEAD,	L2, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "boom",		do_boom,	POS_DEAD,	L2, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "boottime",	do_boottime,	POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "deny",		do_deny,	POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "disconnect",	do_disconnect,	POS_DEAD,	L6, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "eqstat",         do_eqstat,      POS_DEAD,	L6, 0,  LOG_NORMAL, 1, 0, 0 },
    { "flag",		do_flag,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "freeze",		do_freeze,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "intone",		do_intone,	POS_DEAD,	L6, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "message",	do_message,	POS_DEAD,	L3, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "no_delete",      do_nodelete,    POS_DEAD,       L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "permban",	do_permban,	POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "protect",	do_protect,	POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "preserve",	do_preserve,	POS_DEAD,	L2, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "reboo",		do_reboo,	POS_DEAD,	L1, 0,  LOG_NORMAL, 0, 0, 0 },
    { "reboot",		do_reboot,	POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "set",		do_set,		POS_DEAD,	L2, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "shutdow",	do_shutdow,	POS_DEAD,	L1, 0,  LOG_NORMAL, 0, 0, 0 },
    { "shutdown",	do_shutdown,	POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
/*  { "sockets",	do_sockets,	POS_DEAD,	L4, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "warmboot",	do_warmboot,	POS_DEAD,	L1, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "wizlock",	do_wizlock,	POS_DEAD,	L2, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "xp",             do_xp,          POS_DEAD,       L2, 0,  LOG_ALWAYS, 1, 0, 0 },

    { "buki",		do_buki,	POS_DEAD,	L2, 0,	LOG_NORMAL, 1, 0, 0 },
    { "force",		do_force,	POS_DEAD,	L7, 1,  LOG_ALWAYS, 1, 0, 0 },
    { "load",		do_load,	POS_DEAD,	L4, 1,  LOG_ALWAYS, 1, 0, 0 },
    { "nochannels",	do_nochannels,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "noemote",	do_noemote,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "noshout",	do_noshout,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "notell",		do_notell,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "omni",           do_omni,        POS_DEAD,       L5, 0,  LOG_NORMAL, 1, 0, 0 },
    { "pecho",		do_pecho,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "pardon",		do_pardon,	POS_DEAD,	L3, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "pload",		do_pload,	POS_DEAD,	L2, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "punload",	do_punload,	POS_DEAD,	L2, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "purge",		do_purge,	POS_DEAD,	L7, 1,  LOG_ALWAYS, 1, 0, 0 },
    { "restore",	do_restore,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "skillstat",	do_skillstat,	POS_DEAD,	L2, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "spellstat",	do_spellstat,	POS_DEAD,	L2, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "sla",		do_sla,		POS_DEAD,	L3, 0,  LOG_NORMAL, 0, 0, 0 },
    { "slay",		do_slay,	POS_DEAD,	L3, 1,  LOG_ALWAYS, 1, 0, 0 },
    { "teleport",	do_transfer,    POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "transfer",	do_transfer,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "transin",	do_transin,	POS_DEAD,	L5, 0,  LOG_NORMAL, 1, 0, 0 },
    { "transout",	do_transout,	POS_DEAD,	L5, 0,  LOG_NORMAL, 1, 0, 0 },
    { "transvict",	do_transvict,	POS_DEAD,	L5, 0,  LOG_NORMAL, 1, 0, 0 },

/*  { "at",		do_at,		POS_DEAD,	L6, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "poofin",		do_bamfin,	POS_DEAD,	L8, 0,  LOG_NORMAL, 1, 0, 0 },
    { "poofout",	do_bamfout,	POS_DEAD,	L8, 0,  LOG_NORMAL, 1, 0, 0 },
    { "gecho",		do_echo,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
/*  { "goto",		do_goto,	POS_DEAD,	L8, 0,  LOG_NORMAL, 1, 0, 0 }, */
    { "holylight",	do_holylight,	POS_DEAD,	IM, 1,  LOG_NORMAL, 1, 0, 0 },
    { "incognito",	do_incognito,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { "induct",		do_induct,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "invis",		do_invis,	POS_DEAD,	IM, 0,  LOG_NORMAL, 0, 0, 0 },
    { "log",		do_log,		POS_DEAD,	L1, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "memory",		do_memory,	POS_DEAD,	IM, 1,  LOG_NORMAL, 1, 0, 0 },
    { "mwhere",		do_mwhere,	POS_DEAD,	IM, 1,	LOG_NORMAL, 1, 0, 0 },
    { "owhere",		do_owhere,	POS_DEAD,	IM, 1,	LOG_NORMAL, 1, 0, 0 },
    { "page",		do_page,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { "peace",		do_peace,	POS_DEAD,	L5, 0,  LOG_NORMAL, 1, 0, 0 },
    { "penalty",	do_penalty,	POS_DEAD,	L7, 0,  LOG_NORMAL, 1, 0, 0 },
    { "echo",		do_recho,	POS_DEAD,	L6, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "return",         do_return,      POS_DEAD,       L7, 0,  LOG_NORMAL, 1, 0, 0 },
    { "snoop",		do_snoop,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "stat",		do_stat,	POS_DEAD,	IM, 1,	LOG_NORMAL, 1, 0, 0 },
    { "string",		do_string,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "switch",		do_switch,	POS_DEAD,	L7, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "wizinvis",	do_invis,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { "vnum",		do_vnum,	POS_DEAD,	L4, 1,  LOG_NORMAL, 1, 0, 0 },
    { "zecho",		do_zecho,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },

    { "clone",		do_clone,	POS_DEAD,	L5, 0,  LOG_ALWAYS, 1, 0, 0 },

    { "wiznet",		do_wiznet,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { "immtalk",	do_immtalk,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { "imotd",          do_imotd,       POS_DEAD,       IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { ":",		do_immtalk,	POS_DEAD,	IM, 0,  LOG_NORMAL, 0, 0, 0 },
    { "imptalk",	do_imptalk,	POS_DEAD,	L2, 0,  LOG_NORMAL, 1, 0, 0 },
    { ">",		do_imptalk,	POS_DEAD,	L2, 0,  LOG_NORMAL, 0, 0, 0 },
    { "smote",		do_smote,	POS_DEAD,	 0, 0,  LOG_NORMAL, 1, 0, 0 },
    { "prefi",		do_prefi,	POS_DEAD,	IM, 0,  LOG_NORMAL, 0, 0, 0 },
    { "prefix",		do_prefix,	POS_DEAD,	IM, 0,  LOG_NORMAL, 1, 0, 0 },
    { "mpdump",		do_mpdump,	POS_DEAD,	IM, 3,  LOG_NEVER,  1, 1, 0 },
    { "mpstat",		do_mpstat,	POS_DEAD,	IM, 3,  LOG_NEVER,  1, 0, 0 },
    { "opstat",		do_opstat,	POS_DEAD,	IM, 3,  LOG_NEVER,  1, 0, 0 },
    { "rpstat",		do_rpstat,	POS_DEAD,	IM, 3,  LOG_NEVER,  1, 0, 0 },

    { "anathametize",   do_anathametize,POS_DEAD,       L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "cure",           do_cure,        POS_DEAD,       L4, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "absolve",	do_absolve,	POS_DEAD,	L4, 0,  LOG_ALWAYS, 1, 0, 0 },

    /*
     * OLC commands.
     */
    { "account",	do_account,	POS_DEAD,	ML, 0,	LOG_ALWAYS, 1, 0, 0 },
    { "acreator",	do_area_creator,POS_DEAD,	NL, 5,  LOG_ALWAYS, 1, 0, 0 },
    { "acredits",	do_area_credits,POS_DEAD,	NL, 5,  LOG_ALWAYS, 1, 0, 0 },
    { "aflags",		do_area_flags,	POS_DEAD,	NL, 5,  LOG_ALWAYS, 1, 0, 0 },
    { "aname",		do_area_name,	POS_DEAD,	NL, 5,  LOG_ALWAYS, 1, 0, 0 },
    { "anum",		do_anum,	POS_DEAD,	NL, 1,  LOG_NORMAL, 1, 0, 0 },
    { "areset",		do_areset,	POS_DEAD,	NL, 5,	LOG_ALWAYS, 1, 0, 0 },
    { "areview",	do_areview,	POS_DEAD,	NL, 5,  LOG_NORMAL, 1, 0, 0 },
    { "asave",		do_asave,	POS_DEAD,	NL, 1,  LOG_ALWAYS, 1, 0, 0 },
    { "asound",	        do_area_sound,  POS_DEAD,	NL, 5,  LOG_ALWAYS, 1, 0, 0 },
    { "create",		do_create,	POS_DEAD,	NL, 2,  LOG_ALWAYS, 1, 0, 0 },
    { "cretalk",	do_cretalk,	POS_DEAD,	NL, 2,  LOG_NORMAL, 1, 0, 0 },
    { "]",		do_cretalk,	POS_DEAD,	NL, 2,  LOG_NORMAL, 0, 0, 0 },
    { "mob",		do_mobile,	POS_DEAD,	NL, 1,  LOG_NORMAL, 1, 0, 0 },
    { "obj",		do_object,	POS_DEAD,	NL, 1,  LOG_NORMAL, 1, 0, 0 },
    { "olchelp",	do_olchelp,	POS_DEAD,	ML, 1,  LOG_NORMAL, 1, 0, 0 },
    { "olclevel",	do_olclevel,	POS_DEAD,	L3, 0,  LOG_ALWAYS, 1, 0, 0 },
    { "program",	do_program,	POS_DEAD,	NL, 4,  LOG_NORMAL, 1, 0, 0 },
    { "room",		do_room,	POS_DEAD,	NL, 1,  LOG_NORMAL, 1, 0, 0 },
    { "territory",	do_territory,	POS_DEAD,	NL, 5,	LOG_ALWAYS, 1, 0, 0 },
    { "trap",		do_trap,	POS_DEAD,	NL, 3,  LOG_NORMAL, 1, 0, 0 },
    { "unsave",		do_unsave,	POS_DEAD,	NL, 1,  LOG_NORMAL, 1, 0, 0 },

    /*
     * End of list.
     */
    { "",		0,		POS_DEAD,	 0, 0,  LOG_NORMAL, 0, 0, 0 }
};


/*
 * The main entry point for executing commands.
 * Can be recursively called from 'at', 'order', 'force'.
 */
void interpret( CHAR_DATA *ch, char *argument )
{
    char command[MAX_INPUT_LENGTH];
    char logline[MAX_INPUT_LENGTH];
    int cmd;
    int trust;
    bool found;

    REMOVE_BIT( ch->act, PLR_KILLED );

    /*
     * Strip leading spaces.
     */
    while ( isspace(*argument) )
	argument++;
    if ( argument[0] == '\0' )
	return;

    /*
     * No hiding.
     */
    REMOVE_BIT( ch->affected_by, AFF_HIDE );

    /*
     * Implement freeze command.
     */
    if ( !IS_NPC(ch) && IS_SET(ch->act, PLR_FREEZE) )
    {
	send_to_char( "You're totally frozen!\n\r", ch );
	return;
    }

    /*
     * Update the crash save info.
     */
    crash_set_last_char( ch->name );
    crash_set_last_cmd( argument );

    /*
     * Grab the command word.
     * Special parsing so ' can be a command,
     *   also no spaces needed after punctuation.
     */
    strcpy( logline, argument );
    if ( !isalpha(argument[0]) && !isdigit(argument[0]) )
    {
	command[0] = argument[0];
	command[1] = '\0';
	argument++;
	while ( isspace(*argument) )
	    argument++;
    }
    else
    {
	argument = one_argument( argument, command );
    }

    /*
     * Look for command in command table.
     */
    found = FALSE;
    trust = get_trust( ch );
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
	if ( command[0] == cmd_table[cmd].name[0]
	&&   !str_prefix( command, cmd_table[cmd].name )
	&&   (cmd_table[cmd].level <= trust
	||    (cmd_table[cmd].olc_level != 0
	&&     cmd_table[cmd].olc_level <= get_olc_level(ch))) )
	{
	    if ( IS_AFFECTED(ch, AFF_PARALYSIS) 
	    &&   cmd_table[cmd].no_paralysis )
	    {
		send_to_char( "You can't seem to move!\n\r", ch );

		crash_set_last_char( "" );
		crash_set_last_cmd( "" );

		return;
	    }
	    
	    found = TRUE;
	    break;
	}
    }

    /*
     * Log and snoop.
     */
    if ( cmd_table[cmd].log == LOG_NEVER )
	strcpy( logline, "" );

    if ( ( !IS_NPC(ch) && IS_SET(ch->act, PLR_LOG) )
    ||   fLogAll
    ||   cmd_table[cmd].log == LOG_ALWAYS )
    {
	sprintf( log_buf, "Log %s: %s", ch->name, logline );
	wiznet(log_buf,ch,NULL,WIZ_SECURE,0,get_trust(ch));
	log_string( log_buf );
    }

    if ( ch->desc != NULL && ch->desc->snoop_by != NULL )
    {
	write_to_buffer( ch->desc->snoop_by, "% ",    2 );
	write_to_buffer( ch->desc->snoop_by, logline, 0 );
	write_to_buffer( ch->desc->snoop_by, "\n\r",  2 );
    }

    if ( !found )
    {
	/*
	 * Look for command in socials table.
	 */
	if ( !check_social( ch, command, argument ) )
	    send_to_char( "Huh?\n\r", ch );

	crash_set_last_char( "" );
	crash_set_last_cmd( "" );
	return;
    }

    /*
     * Character not in position for command?
     */
    if ( ch->position < cmd_table[cmd].position )
    {
	switch( ch->position )
	{
	case POS_DEAD:
	    send_to_char( "Lie still; you are DEAD.\n\r", ch );
	    break;

	case POS_MORTAL:
	case POS_INCAP:
	    send_to_char( "You are hurt far too bad for that.\n\r", ch );
	    break;

	case POS_STUNNED:
	    send_to_char( "You are too stunned to do that.\n\r", ch );
	    break;

	case POS_SLEEPING:
	    send_to_char( "In your dreams, or what?\n\r", ch );
	    break;

	case POS_RESTING:
	    send_to_char( "Nah... You feel too relaxed...\n\r", ch);
	    break;

	case POS_SITTING:
	    send_to_char( "Better stand up first.\n\r",ch);
	    break;

	case POS_FIGHTING:
	    send_to_char( "No way!  You are still fighting!\n\r", ch);
	    break;

	}

	crash_set_last_char( "" );
	crash_set_last_cmd( "" );
	return;
    }

    /*
     * Toast any tildes (for pfile and note spool security).
     */
    smash_tilde( argument );

    /*
     * Stop singing if appropriate.
     */
    if ( !IS_NPC(ch)
    &&   IS_SET(ch->act, PLR_SINGING)
    &&   cmd_table[cmd].breaks_bard_song )
	remove_bard_song( ch, FALSE );

    /*
     * Dispatch the command.
     */
    (*cmd_table[cmd].do_fun) ( ch, argument );

    crash_set_last_char( "" );
    crash_set_last_cmd( "" );
    
    tail_chain( );
    return;
}



bool disp_social( struct social_data *social, CHAR_DATA *ch, CHAR_DATA *victim, const char *adverb, int to )
{
    int triggered = 0;
    
    if ( social->type == SOCIAL_NULL )
    {
	return FALSE;
    }

    if ( adverb[0] != '\0' )
    {
	bool found = FALSE;
	int i;

	for ( i=0; i<MAX_ADVERBS && adverb_table[i] != NULL; i++ )
	{
	    if (LOWER(adverb[0]) == LOWER(adverb_table[i][0])
	    &&	!str_prefix(adverb,adverb_table[i]))
	    {
		adverb = adverb_table[i];
		found = TRUE;
		break;
	    }
	}

	if ( !found )
	{
	    if ( victim == NULL )
	    {
		if ( to == TO_CHAR )
		    send_to_char( "They aren't here.\n\r", ch );
		return FALSE;
	    }
	    else
		adverb = "";
	}
    }

    switch ( social->type )
    {
    case SOCIAL_NO_ADVERB:
	triggered = act( social->message, ch, NULL, victim, to );
	break;
    case SOCIAL_ADVERB:
	if ( adverb[0] == '\0' )
	    triggered = act( social->message, ch, social->adverb, victim, to );
	else
	    triggered = act( social->message, ch, adverb, victim, to );
	break;
    case SOCIAL_OPT_ADVERB:
	if ( adverb[0] == '\0' )
	    triggered = act( social->message, ch, "", victim, to );
	else
	{
	    char buf[MAX_STRING_LENGTH];

	    sprintf( buf, " %s", adverb );
	    triggered = act( social->message, ch, buf, victim, to );
	}
	break;
    }
    
    return IS_SET( triggered, TRIGGERED_VICT );
}


bool check_social( CHAR_DATA *ch, char *command, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    char *adverb;
    int cmd;
    bool found;

    found  = FALSE;
    for ( cmd = 0; social_table[cmd].name[0] != '\0'; cmd++ )
    {
	if ( command[0] == social_table[cmd].name[0]
	&&   !str_prefix( command, social_table[cmd].name ) )
	{
	    found = TRUE;
	    break;
	}
    }

    if ( !found )
	return FALSE;

    if ( !IS_NPC(ch) && IS_SET(ch->comm, COMM_NOEMOTE) )
    {
	send_to_char( "You are anti-social!\n\r", ch );
	return TRUE;
    }
    
    if ( IS_AFFECTED(ch, AFF_PARALYSIS) )
    {
	send_to_char( "You can't seem to move!\n\r", ch );
	return TRUE;
    }

    switch ( ch->position )
    {
    case POS_DEAD:
	send_to_char( "Lie still; you are DEAD.\n\r", ch );
	return TRUE;

    case POS_INCAP:
    case POS_MORTAL:
	send_to_char( "You are hurt far too bad for that.\n\r", ch );
	return TRUE;

    case POS_STUNNED:
	send_to_char( "You are too stunned to do that.\n\r", ch );
	return TRUE;

    case POS_SLEEPING:
	/*
	 * I just know this is the path to a 12" 'if' statement.  :(
	 * But two players asked for it already!  -- Furey
	 */
	if ( !str_cmp( social_table[cmd].name, "snore" ) )
	    break;
	send_to_char( "In your dreams, or what?\n\r", ch );
	return TRUE;

    }

    /*
     * Stop singing if appropriate.
     */
    if ( !IS_NPC(ch)
    &&   IS_SET(ch->act, PLR_SINGING)
    &&   cmd_table[cmd].breaks_bard_song )
	remove_bard_song( ch, FALSE );

    adverb = one_argument( argument, arg );
    victim = NULL;
    if ( arg[0] == '\0' )
    {
	disp_social( &social_table[cmd].others_no_arg, 
	    ch, NULL, "", TO_ROOM );
	disp_social( &social_table[cmd].char_no_arg,   
	    ch, NULL, "", TO_CHAR );
    }
    else if ( ( victim = get_char_room( ch, arg ) ) == NULL )
    {
	disp_social( &social_table[cmd].others_no_arg, 
	    ch, NULL, arg, TO_ROOM );
	disp_social( &social_table[cmd].char_no_arg,   
	    ch, NULL, arg, TO_CHAR );
    }
    else if ( victim == ch )
    {
	disp_social( &social_table[cmd].others_auto, 
	    ch, victim, adverb, TO_ROOM );
	disp_social( &social_table[cmd].char_auto,   
	    ch, victim, adverb, TO_CHAR );
    }
    else
    {
	bool triggered;

	disp_social( &social_table[cmd].others_found,
	    ch, victim, adverb, TO_NOTVICT );
	disp_social( &social_table[cmd].char_found,
	    ch, victim, adverb, TO_CHAR );
	triggered = disp_social( &social_table[cmd].vict_found,
	    ch, victim, adverb, TO_VICT );

	if ( !triggered
	&&   !IS_NPC(ch) && IS_NPC(victim)
	&&   !IS_AFFECTED(victim, AFF_CHARM)
	&&   IS_AWAKE(victim) 
	&&   victim->desc == NULL)
	{
	    switch ( number_bits( 4 ) )
	    {
	    case 0:
	    case 1: case 2: case 3: case 4:
	    case 5: case 6: case 7: case 8:
		disp_social( &social_table[cmd].others_found, 
		    victim, ch, "", TO_NOTVICT );
		disp_social( &social_table[cmd].char_found,   
		    victim, ch, "", TO_CHAR );
		disp_social( &social_table[cmd].vict_found,   
		    victim, ch, "", TO_VICT );
		break;

	    case 9: case 10: case 11: case 12:
		act( "$n slaps $N.",  victim, NULL, ch, TO_NOTVICT );
		act( "You slap $N.",  victim, NULL, ch, TO_CHAR	   );
		act( "$n slaps you.", victim, NULL, ch, TO_VICT	   );
		break;
	    }
	}
    }

    return TRUE;
}



/*
 * Return true if an argument is completely numeric.
 */
bool is_number ( char *arg )
{
 
    if ( *arg == '\0' )
        return FALSE;
 
    if ( *arg == '+' || *arg == '-' )
        arg++;
 
    for ( ; *arg != '\0'; arg++ )
    {
        if ( !isdigit( *arg ) )
            return FALSE;
    }
 
    return TRUE;
}



/*
 * Given a string like 14.foo, return 14 and 'foo'
 */
int number_argument( char *argument, char *arg )
{
    char *pdot;
    int number;
    
    for ( pdot = argument; *pdot != '\0'; pdot++ )
    {
	if ( *pdot == '.' )
	{
	    *pdot = '\0';
	    number = atoi( argument );
	    *pdot = '.';
	    strcpy( arg, pdot+1 );
	    return number;
	}
    }

    strcpy( arg, argument );
    return 1;
}

/* 
 * Given a string like 14*foo, return 14 and 'foo'
*/
int mult_argument(char *argument, char *arg)
{
    char *pdot;
    int number;

    for ( pdot = argument; *pdot != '\0'; pdot++ )
    {
        if ( *pdot == '*' )
        {
            *pdot = '\0';
            number = atoi( argument );
            *pdot = '*';
            strcpy( arg, pdot+1 );
            return number;
        }
    }
 
    strcpy( arg, argument );
    return 1;
}



/*
 * Pick off one argument from a string and return the rest.
 * Understands quotes.
 */
char *one_argument( char *argument, char *arg_first )
{
    char cEnd;

    while ( isspace(*argument) )
	argument++;

    cEnd = ' ';
    if ( *argument == '\'' || *argument == '"' )
	cEnd = *argument++;

    while ( *argument != '\0' )
    {
	if ( *argument == cEnd )
	{
	    argument++;
	    break;
	}
	*arg_first = LOWER(*argument);
	arg_first++;
	argument++;
    }
    *arg_first = '\0';

    while ( isspace(*argument) )
	argument++;

    return argument;
}

/*
 * Pick off one argument from a string and return the rest.
 * Understands quotes. Doesn't convert case.
 */
char *cased_one_argument( char *argument, char *arg_first )
{
    char cEnd;

    while ( isspace(*argument) )
	argument++;

    cEnd = ' ';
    if ( *argument == '\'' || *argument == '"' )
	cEnd = *argument++;

    while ( *argument != '\0' )
    {
	if ( *argument == cEnd )
	{
	    argument++;
	    break;
	}
	*arg_first = *argument;
	arg_first++;
	argument++;
    }
    *arg_first = '\0';

    while ( isspace(*argument) )
	argument++;

    return argument;
}

/*
 * Contributed by Alander.
 */
void do_commands( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int cmd;
    int col;
 
    col = 0;
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
        if ( cmd_table[cmd].level <  LEVEL_HERO
        &&   cmd_table[cmd].level <= get_trust( ch ) 
	&&   cmd_table[cmd].show)
	{
	    sprintf( buf, "%-12s", cmd_table[cmd].name );
	    send_to_char( buf, ch );
	    if ( ++col % 6 == 0 )
		send_to_char( "\n\r", ch );
	}
    }
 
    if ( col % 6 != 0 )
	send_to_char( "\n\r", ch );
    return;
}

void do_wizhelp( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int cmd;
    int col;
 
    col = 0;
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
        if ( cmd_table[cmd].level >= LEVEL_HERO
        &&   cmd_table[cmd].level <= get_trust( ch ) 
        &&   cmd_table[cmd].show)
	{
	    sprintf( buf, "%-14s", cmd_table[cmd].name );
	    send_to_char( buf, ch );
	    if ( ++col % 6 == 0 )
		send_to_char( "\n\r", ch );
	}
    }
 
    if ( col % 6 != 0 )
	send_to_char( "\n\r", ch );
    return;
}



void do_olchelp( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int cmd;
    int col;
 
    col = 0;
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
        if ( cmd_table[cmd].olc_level != 0
        &&   cmd_table[cmd].olc_level <= get_olc_level(ch)
        &&   cmd_table[cmd].show
        &&   (!IS_IMMORTAL(ch)
        ||    cmd_table[cmd].level >= get_trust(ch)) )
	{
	    sprintf( buf, "%-14s", cmd_table[cmd].name );
	    send_to_char( buf, ch );
	    if ( ++col % 6 == 0 )
		send_to_char( "\n\r", ch );
	}
    }
 
    if ( col % 6 != 0 )
	send_to_char( "\n\r", ch );
    return;
}


void do_adverbs( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int adverb;
    int col;
 
    col = 0;
    for (adverb=0; adverb_table[adverb]!=NULL && adverb<MAX_ADVERBS; adverb++)
    {
	sprintf( buf, "%-19s", adverb_table[adverb] );
	send_to_char( buf, ch );
	if ( ++col % 4 == 0 )
	    send_to_char( "\n\r", ch );
    }
 
    if ( col % 4 != 0 )
	send_to_char( "\n\r", ch );
    return;
}


bool trusted_command( CHAR_DATA *ch, char *command )
{
    int cmd, trust;
    
    trust = get_trust( ch );
    
    for ( cmd = 0; cmd_table[cmd].name[0] != '\0'; cmd++ )
    {
	if ( !str_cmp(cmd_table[cmd].name, command) )
	    return cmd_table[cmd].level <= get_trust( ch );
    }
    
    return FALSE;
}



