/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "merc.h"
#include "recycle.h"

/* stuff for recyling notes */
NOTE_DATA *note_free;

NOTE_DATA *new_note()
{
    NOTE_DATA *note;

    if (note_free == NULL)
	note = alloc_perm(sizeof(*note));
    else
    { 
	note = note_free;
	note_free = note_free->next;
    }
    VALIDATE(note);
    return note;
}

void free_note(NOTE_DATA *note)
{
    if (!IS_VALID(note))
	return;

    free_string( note->text    );
    free_string( note->subject );
    free_string( note->to_list );
    free_string( note->date    );
    free_string( note->sender  );
    INVALIDATE(note);

    note->next = note_free;
    note_free   = note;
}

    
/* stuff for recycling ban structures */
BAN_DATA *ban_free;

BAN_DATA *new_ban(void)
{
    static BAN_DATA ban_zero;
    BAN_DATA *ban;

    if (ban_free == NULL)
	ban = alloc_perm(sizeof(*ban));
    else
    {
	ban = ban_free;
	ban_free = ban_free->next;
    }

    *ban = ban_zero;
    VALIDATE(ban);
    ban->name = &str_empty[0];
    return ban;
}

void free_ban(BAN_DATA *ban)
{
    if (!IS_VALID(ban))
	return;

    free_string(ban->name);
    INVALIDATE(ban);

    ban->next = ban_free;
    ban_free = ban;
}

/* stuff for recycling descriptors */
DESCRIPTOR_DATA *descriptor_free;

DESCRIPTOR_DATA *new_descriptor(void)
{
    static DESCRIPTOR_DATA d_zero;
    DESCRIPTOR_DATA *d;

    if (descriptor_free == NULL)
	d = alloc_perm(sizeof(*d));
    else
    {
	d = descriptor_free;
	descriptor_free = descriptor_free->next;
    }
	
    *d = d_zero;
    VALIDATE(d);

    d->connected     = CON_GET_NAME;
    d->showstr_head  = NULL;
    d->showstr_point = NULL;
    d->outsize       = 2000;
    d->outbuf        = alloc_mem( d->outsize );
#if defined(unix)
    d->ident         = str_dup( "???" );
    d->ifd           = -1;
    d->ipid          = -1;
#endif

    return d;
}

void free_descriptor(DESCRIPTOR_DATA *d)
{
    if (!IS_VALID(d))
	return;

    free_string( d->host );
    free_mem( d->outbuf, d->outsize );
    INVALIDATE(d);
    d->next = descriptor_free;
    descriptor_free = d;
}

/* stuff for recycling gen_data */
GEN_DATA *gen_data_free;

GEN_DATA *new_gen_data(void)
{
    static GEN_DATA gen_zero;
    GEN_DATA *gen;

    if (gen_data_free == NULL)
	gen = alloc_perm(sizeof(*gen));
    else
    {
	gen = gen_data_free;
	gen_data_free = gen_data_free->next;
    }
    *gen = gen_zero;
    VALIDATE(gen);
    return gen;
}

void free_gen_data(GEN_DATA *gen)
{
    if (!IS_VALID(gen))
	return;

    INVALIDATE(gen);

    gen->next = gen_data_free;
    gen_data_free = gen;
} 

/* stuff for recycling extended descs */
EXTRA_DESCR_DATA *extra_descr_free;

EXTRA_DESCR_DATA *new_extra_descr(void)
{
    EXTRA_DESCR_DATA *ed;

    if (extra_descr_free == NULL)
	ed = alloc_perm(sizeof(*ed));
    else
    {
	ed = extra_descr_free;
	extra_descr_free = extra_descr_free->next;
    }

    ed->keyword = &str_empty[0];
    ed->description = &str_empty[0];
    VALIDATE(ed);
    return ed;
}

void free_extra_descr(EXTRA_DESCR_DATA *ed)
{
    if (!IS_VALID(ed))
	return;

    free_string(ed->keyword);
    free_string(ed->description);
    INVALIDATE(ed);
    
    ed->next = extra_descr_free;
    extra_descr_free = ed;
}


/* stuff for recycling affects */
AFFECT_DATA *affect_free;

AFFECT_DATA *new_affect(void)
{
    static AFFECT_DATA af_zero;
    AFFECT_DATA *af;

    if (affect_free == NULL)
	af = alloc_perm(sizeof(*af));
    else
    {
	af = affect_free;
	affect_free = affect_free->next;
    }

    *af = af_zero;
    af->name = str_dup( "" );
    af->appearance = str_dup( "" );

    VALIDATE(af);
    return af;
}

void free_affect(AFFECT_DATA *af)
{
    if (!IS_VALID(af))
	return;

    free_string( af->name );
    free_string( af->appearance );

    INVALIDATE(af);
    af->next = affect_free;
    affect_free = af;
}

/* stuff for recycling objects */
OBJ_DATA *obj_free;

OBJ_DATA *new_obj(void)
{
    static OBJ_DATA obj_zero;
    OBJ_DATA *obj;

    if (obj_free == NULL)
	obj = alloc_perm(sizeof(*obj));
    else
    {
	obj = obj_free;
	obj_free = obj_free->next;
    }
    *obj = obj_zero;
    VALIDATE(obj);

    return obj;
}

void free_obj(OBJ_DATA *obj)
{
    AFFECT_DATA *paf, *paf_next;
    EXTRA_DESCR_DATA *ed, *ed_next;

/*  - removed as we invalidate before freeing now.
 *  if (!IS_VALID(ch))
 *	return;
 */

    for (paf = obj->affected; paf != NULL; paf = paf_next)
    {
	paf_next = paf->next;
	free_affect(paf);
    }
    obj->affected = NULL;

    for (ed = obj->extra_descr; ed != NULL; ed = ed_next )
    {
	ed_next = ed->next;
	free_extra_descr(ed);
     }
     obj->extra_descr = NULL;
   
    free_string( obj->name        );
    free_string( obj->description );
    free_string( obj->short_descr );
    free_string( obj->owner     );
    INVALIDATE(obj);

    obj->next   = obj_free;
    obj_free    = obj; 
}


/* stuff for recyling characters */
CHAR_DATA *char_free;

CHAR_DATA *new_char (void)
{
    static CHAR_DATA ch_zero;
    CHAR_DATA *ch;
    int i;

    if (char_free == NULL)
	ch = alloc_perm(sizeof(*ch));
    else
    {
	ch = char_free;
	char_free = char_free->next;
    }

    *ch				= ch_zero;
    VALIDATE(ch);
    ch->name			= &str_empty[0];
    ch->short_descr		= &str_empty[0];
    ch->long_descr		= &str_empty[0];
    ch->description		= &str_empty[0];
    ch->appearance		= &str_empty[0];
    ch->prompt			= &str_empty[0];
    ch->prefix			= &str_empty[0];
    ch->logon			= current_time;
    ch->lines			= PAGELEN;
    for (i = 0; i < 4; i++)
        ch->armor[i]		= 100;
    ch->position		= POS_STANDING;
    ch->hit			= 20;
    ch->max_hit			= 20;
    ch->mana			= 100;
    ch->max_mana		= 100;
    ch->move			= 100;
    ch->max_move		= 100;
    for (i = 0; i < MAX_STATS; i ++)
    {
        ch->perm_stat[i] = 52;
        ch->mod_stat[i] = 0;
    }

    return ch;
}


void free_char (CHAR_DATA *ch)
{
    OBJ_DATA *obj;
    OBJ_DATA *obj_next;
    AFFECT_DATA *paf;
    AFFECT_DATA *paf_next;

/*  - removed as we invalidate before freeing now.
 *  if (!IS_VALID(ch))
 *	return;
 */

    if (IS_NPC(ch))
	mobile_count--;

    for (obj = ch->carrying; obj != NULL; obj = obj_next)
    {
	obj_next = obj->next_content;
	extract_obj(obj);
    }

    for (paf = ch->affected; paf != NULL; paf = paf_next)
    {
	paf_next = paf->next;
	affect_remove(ch,paf);
    }

    free_string(ch->name);
    free_string(ch->short_descr);
    free_string(ch->long_descr);
    free_string(ch->description);
    free_string(ch->appearance);
    free_string(ch->prompt);
    free_string(ch->prefix);
    free_note(ch->pnote);

    if (ch->pcdata != NULL)
    	free_pcdata(ch->pcdata);

    ch->next = char_free;
    char_free  = ch;

    INVALIDATE(ch);
    return;
}

PC_DATA *pcdata_free;

PC_DATA *new_pcdata(void)
{
    int alias;

    static PC_DATA pcdata_zero;
    PC_DATA *pcdata;

    if (pcdata_free == NULL)
	pcdata = alloc_perm(sizeof(*pcdata));
    else
    {
	pcdata = pcdata_free;
	pcdata_free = pcdata_free->next;
    }

    *pcdata = pcdata_zero;

    for (alias = 0; alias < MAX_ALIAS; alias++)
    {
	pcdata->alias[alias]		= NULL;
	pcdata->alias_sub[alias]	= NULL;
    }

    pcdata->buffer = new_buf();
    
    VALIDATE(pcdata);
    return pcdata;
}
	

void free_pcdata(PC_DATA *pcdata)
{
    int alias;

    if (!IS_VALID(pcdata))
	return;

    free_string(pcdata->pwd);
    free_string(pcdata->bamfin);
    free_string(pcdata->bamfout);
    free_string(pcdata->title);
    free_buf(pcdata->buffer);
    
    for (alias = 0; alias < MAX_ALIAS; alias++)
    {
	free_string(pcdata->alias[alias]);
	free_string(pcdata->alias_sub[alias]);
    }

    free_string(pcdata->transin);
    free_string(pcdata->transout);
    free_string(pcdata->transvict);
    free_string(pcdata->who_name);
    
    INVALIDATE(pcdata);
    pcdata->next = pcdata_free;
    pcdata_free = pcdata;

    return;
}

	


/* stuff for setting ids */
long	last_pc_id;
long	last_mob_id;

long get_pc_id(void)
{
    int val;

    val = (current_time <= last_pc_id) ? last_pc_id + 1 : current_time;
    last_pc_id = val;
    return val;
}

long get_mob_id(void)
{
    last_mob_id++;
    return last_mob_id;
}

MEM_DATA *mem_data_free;

/* procedures and constants needed for buffering */

BUFFER *buf_free;

MEM_DATA *new_mem_data(void)
{
    MEM_DATA *memory;
  
    if (mem_data_free == NULL)
	memory = alloc_mem(sizeof(*memory));
    else
    {
	memory = mem_data_free;
	mem_data_free = mem_data_free->next;
    }

    memory->next = NULL;
    memory->id = 0;
    memory->reaction = 0;
    memory->duration = 0;
    VALIDATE(memory);

    return memory;
}

void free_mem_data(MEM_DATA *memory)
{
    if (!IS_VALID(memory))
	return;

    memory->next = mem_data_free;
    mem_data_free = memory;
    INVALIDATE(memory);
}


/* Reset data. */
RESET_DATA *reset_data_free;

RESET_DATA *new_reset_data(void)
{
    RESET_DATA *reset;
  
    if (reset_data_free == NULL)
	reset = alloc_mem(sizeof(*reset));
    else
    {
	reset = reset_data_free;
	reset_data_free = reset_data_free->next;
    }
    
    reset->next = NULL;

    VALIDATE(reset);

    return reset;
}


void free_reset_data(RESET_DATA *reset)
{
    if (!IS_VALID(reset))
	return;

    reset->next = reset_data_free;
    reset_data_free = reset;
    INVALIDATE(reset);
}


/* Shop data. */
SHOP_DATA *shop_free;

SHOP_DATA *new_shop(void)
{
    SHOP_DATA *shop;
  
    if (shop_free == NULL)
	shop = alloc_mem(sizeof(*shop));
    else
    {
	shop = shop_free;
	shop_free = shop_free->next;
    }
    
    shop->next = NULL;

    VALIDATE(shop);

    return shop;
}


void free_shop(SHOP_DATA *shop)
{
    if (!IS_VALID(shop))
	return;

    shop->next = shop_free;
    shop_free = shop;
    INVALIDATE(shop);
}


/* Breadth first search queue data. */
BFS_QUEUE_DATA *bfs_data_free;

BFS_QUEUE_DATA *new_bfs_data(void)
{
    BFS_QUEUE_DATA *bfs;
  
    if (bfs_data_free == NULL)
	bfs = alloc_mem(sizeof(*bfs));
    else
    {
	bfs = bfs_data_free;
	bfs_data_free = bfs_data_free->next;
    }
    
    bfs->next = NULL;

    VALIDATE(bfs);

    return bfs;
}


void free_bfs_data(BFS_QUEUE_DATA *bfs)
{
    if (!IS_VALID(bfs))
	return;

    bfs->next = bfs_data_free;
    bfs_data_free = bfs;
    INVALIDATE(bfs);
}


/* Mob program data. */
MPROG_LIST *mob_program_free;

MPROG_LIST *new_mob_program(void)
{
    MPROG_LIST *mp;
  
    if (mob_program_free == NULL)
	mp = alloc_mem(sizeof(*mp));
    else
    {
	mp = mob_program_free;
	mob_program_free = mob_program_free->next;
    }
    
    mp->next = NULL;

    mp->trig_phrase = str_dup( "" );
    VALIDATE(mp);

    return mp;
}


void free_mob_program(MPROG_LIST *mp)
{
    if (!IS_VALID(mp))
	return;

    free_string( mp->trig_phrase );

    mp->next = mob_program_free;
    mob_program_free = mp;
    INVALIDATE(mp);
}


/* Periodic check data. */
PERIODIC_CHECK_DATA *periodic_check_free;

PERIODIC_CHECK_DATA *new_periodic_check(void)
{
    PERIODIC_CHECK_DATA *pc;
  
    if (periodic_check_free == NULL)
	pc = alloc_mem(sizeof(*pc));
    else
    {
	pc = periodic_check_free;
	periodic_check_free = periodic_check_free->next;
    }
    
    pc->next = NULL;
    pc->data = NULL;
    VALIDATE(pc);

    return pc;
}


void free_periodic_check(PERIODIC_CHECK_DATA *pc)
{
    if (!IS_VALID(pc))
	return;

    pc->next = periodic_check_free;
    periodic_check_free = pc;
    INVALIDATE(pc);
}


/* Use message data. */
USE_MSG_DATA *use_msg_free;

USE_MSG_DATA *new_use_msg(void)
{
    USE_MSG_DATA *um;
  
    if (use_msg_free == NULL)
	um = alloc_mem(sizeof(*um));
    else
    {
	um = use_msg_free;
	use_msg_free = use_msg_free->next;
    }
    
    um->next = NULL;
    um->msg  = str_dup( "" );
    VALIDATE(um);

    return um;
}


void free_use_msg(USE_MSG_DATA *um)
{
    if (!IS_VALID(um))
	return;

    free_string( um->msg );

    um->next = use_msg_free;
    use_msg_free = um;
    INVALIDATE(um);
}


/* Use message data. */
MUD_EVENT *mud_event_free;

MUD_EVENT *new_mud_event( void )
{
    MUD_EVENT *me;
  
    if ( mud_event_free == NULL )
	me = alloc_mem( sizeof(*me) );
    else
    {
	me = mud_event_free;
	mud_event_free = mud_event_free->next;
    }
    
    me->next	= NULL;
    me->message	= str_dup( "" );
    VALIDATE(me);

    return me;
}


void free_mud_event(MUD_EVENT *me)
{
    if ( !IS_VALID(me) )
	return;

    free_string( me->message );

    me->next = mud_event_free;
    mud_event_free = me;
    INVALIDATE( me );
}


/* Account data. */
BANK_ACCOUNT *account_free;

BANK_ACCOUNT *new_account( void )
{
    BANK_ACCOUNT *ac;
  
    if ( account_free == NULL )
	ac = alloc_mem( sizeof(*ac) );
    else
    {
	ac = account_free;
	account_free = account_free->next;
    }
    
    ac->next	= NULL;
    ac->balance	= 0;
    ac->name	= str_dup( "" );
    
    VALIDATE( ac );

    return ac;
}


void free_account( BANK_ACCOUNT *ac )
{
    if ( !IS_VALID(ac) )
	return;

    free_string( ac->name );

    ac->next = account_free;
    account_free = ac;
    
    INVALIDATE( ac );
}


/* Crime data. */
CRIME_DATA *crime_free;

CRIME_DATA *new_crime( void )
{
    CRIME_DATA *cd;
  
    if ( crime_free == NULL )
	cd = alloc_mem( sizeof(*cd) );
    else
    {
	cd = crime_free;
	crime_free = crime_free->next;
    }
    
    cd->next	= NULL;
    cd->victim	= str_dup( "" );
    
    VALIDATE( cd );

    return cd;
}


void free_crime( CRIME_DATA *cd )
{
    if ( !IS_VALID(cd) )
	return;

    free_string( cd->victim );

    cd->next = crime_free;
    crime_free = cd;
    
    INVALIDATE( cd );
}


/* Advantage, etc data. */
CUST_DATA *cust_free;

CUST_DATA *new_cust_data( void )
{
    static CUST_DATA cust_zero;
    CUST_DATA *cd;
  
    if ( cust_free == NULL )
	cd = alloc_mem( sizeof(*cd) );
    else
    {
	cd = cust_free;
	cust_free = cust_free->next;
    }
    
    *cd = cust_zero;
    cd->affect.where = -1;
   
    return cd;
}


void free_cust_data( CUST_DATA *cd )
{
    cd->next = cust_free;
    cust_free = cd;
}


/* buffer sizes */
const int buf_size[MAX_BUF_LIST] =
{
    16,32,64,128,256,1024,2048,4096,8192,16384
};

/* local procedure for finding the next acceptable size */
/* -1 indicates out-of-boundary error */
int get_size (int val)
{
    int i;

    for (i = 0; i < MAX_BUF_LIST; i++)
	if (buf_size[i] >= val)
	{
	    return buf_size[i];
	}
    
    return -1;
}

BUFFER *new_buf()
{
    BUFFER *buffer;

    if (buf_free == NULL) 
	buffer = alloc_perm(sizeof(*buffer));
    else
    {
	buffer = buf_free;
	buf_free = buf_free->next;
    }

    buffer->next	= NULL;
    buffer->state	= BUFFER_SAFE;
    buffer->size	= get_size(BASE_BUF);

    buffer->string	= alloc_mem(buffer->size);
    buffer->string[0]	= '\0';
    VALIDATE(buffer);

    return buffer;
}

BUFFER *new_buf_size(int size)
{
    BUFFER *buffer;
 
    if (buf_free == NULL)
        buffer = alloc_perm(sizeof(*buffer));
    else
    {
        buffer = buf_free;
        buf_free = buf_free->next;
    }
 
    buffer->next        = NULL;
    buffer->state       = BUFFER_SAFE;
    buffer->size        = get_size(size);
    if (buffer->size == -1)
    {
        bug("new_buf: buffer size %d too large.",size);
        exit(1);
    }
    buffer->string      = alloc_mem(buffer->size);
    buffer->string[0]   = '\0';
    VALIDATE(buffer);
 
    return buffer;
}


void free_buf(BUFFER *buffer)
{
    if (!IS_VALID(buffer))
	return;

    free_mem(buffer->string,buffer->size);
    buffer->string = NULL;
    buffer->size   = 0;
    buffer->state  = BUFFER_FREED;
    INVALIDATE(buffer);

    buffer->next  = buf_free;
    buf_free      = buffer;
}


bool add_buf(BUFFER *buffer, char *string)
{
    int len;
    char *oldstr;
    int oldsize;

    oldstr = buffer->string;
    oldsize = buffer->size;

    if (buffer->state == BUFFER_OVERFLOW) /* don't waste time on bad strings! */
	return FALSE;

    len = strlen(buffer->string) + strlen(string) + 1;

    while (len >= buffer->size) /* increase the buffer size */
    {
	buffer->size 	= get_size(buffer->size + 1);
	{
	    if (buffer->size == -1) /* overflow */
	    {
		buffer->size = oldsize;
		buffer->state = BUFFER_OVERFLOW;
		bug("buffer overflow past size %d",buffer->size);
		return FALSE;
	    }
  	}
    }

    if (buffer->size != oldsize)
    {
	buffer->string	= alloc_mem(buffer->size);

	strcpy(buffer->string,oldstr);
	free_mem(oldstr,oldsize);
    }

    strcat(buffer->string,string);
    return TRUE;
}


void clear_buf(BUFFER *buffer)
{
    buffer->string[0] = '\0';
    buffer->state     = BUFFER_SAFE;
}


char *buf_string(BUFFER *buffer)
{
    return buffer->string;
}

    

	







