/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "merc.h"
#include "db.h"
#include "olc.h"
#include "color.h"
#include "tables.h"
#include "recycle.h"


int flag_lookup args( ( const char *name, const struct flag_type *flag_table) );


static bool terrListDirty;


void load_territories()
{
    int territory;
    int version;
    FILE *fp;

    if ( (fp = fopen(TERRITORIES_FILE, "r")) == NULL )
    {
	bug( "Couldn't open territories file.",0);
	exit( 0 );
    }

    version = fread_number( fp );
    
    for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
    {
    	char command;
    	
    	command = fread_letter( fp );
    	if ( command == '$' )
    	    break;
    	
	territories[territory].name	= fread_string( fp );
	territories[territory].crimes	= fread_flag( fp );
	territories[territory].jail	= 0;
	
	territories[territory].social_classes[0].male	= str_dup( "slave" );
	territories[territory].social_classes[0].female	= str_dup( "slave" );
	territories[territory].social_classes[MAX_SOCIAL_CLASS / 4].male
							= str_dup( "commoner" );
	territories[territory].social_classes[MAX_SOCIAL_CLASS / 4].female
							= str_dup( "commoner" );
	territories[territory].social_classes[MAX_SOCIAL_CLASS - 1].male
							= str_dup( "king" );
	territories[territory].social_classes[MAX_SOCIAL_CLASS - 1].female
							= str_dup( "queen" );
	
	do {
	    AREA_DATA *pArea;
	    char *str;
	    int num;
	
	    command = fread_letter( fp );
	    
	    switch ( command )
	    {
	    case 'A':
	    	str = fread_string( fp );
	    	
	    	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    	{
		    if ( !str_cmp(pArea->name, str) )
		    {
			pArea->territory = territory;
			break;
		    }
	    	}
	    	
	    	if ( pArea == NULL )
	    	{
		    bug( "load_territories: area not found %s.", (long) str );
		    break;
	    	}
	    	
	    	break;
	    	
	    case 'S':
		num  = fread_number( fp );
		
		if ( num < 0 || num > MAX_SOCIAL_CLASS )
		{
		    bug( "load_territories: invalid social class level.", 0 );
		    break;
		}
		
		territories[territory].social_classes[num].male   = fread_string( fp );
		territories[territory].social_classes[num].female = fread_string( fp );
		
		break;
	
	    case 'J':
	    	num = fread_number( fp );
	    	
	    	if ( get_room_index(num) == NULL )
	    	{
		    bug( "load_territories: invalid jail vnum (%d).", num );
		    break;
		}
		
		territories[territory].jail = num;
		
		break;

/*
 *	    case 'L':
 *		num = fread_number( fp );
 *		
 *		if ( num < 0 || num >= MAX_CRIME )
 *		{
 *		    bug( "load_territories: invalid crime number.", 0 );
 *		    break;
 *		}
 *		
 *		territories[territory].punishments[num] = fread_number( fp );
 *		
 *		break;
 */
 		
	    default:
		break;
	    }
	
	} while ( command != '#' );
    }

    fclose( fp );
}


void save_territories()
{
    int territory;
    FILE *fp;
 
    if ( !terrListDirty )
	return;
    
    fclose( fpReserve );
    if ( (fp = fopen(TERRITORY_TEMP, "w")) == NULL )
    {
	bug( "save_territories: fopen.", 0 );

	fpReserve = fopen( NULL_FILE, "r" );
	return;
    }

    fprintf( fp, "%d\n\n", OLC_TERRITORY_VERSION );
    
    for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
    {
	AREA_DATA *pArea;
	int i;
	
	if ( IS_NULLSTR(territories[territory].name) )
	    continue;
	
	fprintf( fp, "T %s~\n%s\n",
	    territories[territory].name,
	    print_flags(territories[territory].crimes) );
	
	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	{
	    if ( pArea->territory == territory )
		fprintf( fp, "A %s~\n", pArea->name );
	}
	
	for ( i = 0; i < MAX_SOCIAL_CLASS; i++ )
	{
	    if ( IS_NULLSTR(territories[territory].social_classes[i].male) )
		continue;

	    fprintf( fp, "S %d\n%s~\n%s~\n", i, 
		territories[territory].social_classes[i].male,
		territories[territory].social_classes[i].female );
	}
	
	if ( territories[territory].jail != 0 )
	    fprintf( fp, "J %d\n", territories[territory].jail );

/*	
 *	for ( i = 0; i < MAX_CRIME; i++ )
 *	{
 *	    if ( territories[territory].punishments[i] != 0 )
 *		fprintf( fp, "L %d %s\n", i, 
 *		    print_flags(territories[territory].punishments[i]) );
 *	}
 */	
	fprintf( fp, "#\n\n\n" );
    }
    
    fprintf( fp, "$\n\n" );
    fclose( fp );

    #if defined( WIN32 )
    remove( TERRITORIES_FILE );
    #endif
    rename( TERRITORY_TEMP, TERRITORIES_FILE );
    fpReserve = fopen( NULL_FILE, "r" );

    terrListDirty = FALSE;
    return;
}


void territory_unsaved( CHAR_DATA *ch )
{
    if ( terrListDirty )
	send_to_char( "Territories are unsaved.\n\r", ch );
}


void do_territory( CHAR_DATA *ch, char *argument )
{
    char buf1[MAX_STRING_LENGTH];
    char buf2[MAX_STRING_LENGTH];
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    int territory;
    
    argument = cased_one_argument( argument, arg1 );
    
    if ( arg1[0] == '\0' )
    {
	    send_to_char( "Syntax: territory list\n\r"
	                  "        territory <name> create\n\r"
	                  "        territory <name> display\n\r"
	                  "        territory <name> add area <area>\n\r"
	                  "        territory <name> remove area <area>\n\r"
	                  "        territory <name> add social <number> <male> <female>\n\r"
	                  "        territory <name> remove social <number>\n\r"
	                  "        territory <name> crimes [+-] <crime flags>\n\r"
	                  "        territory <name> jail <vnum>\n\r"
	                  "        territory <name> delete\n\r"
	                  "        territory save\n\r", ch );
	    return;
    }
    
    if ( !str_prefix(arg1, "list") )
    {
	    bool found = FALSE;
	    int col = 0;
	    
	    for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
	    {
	        if ( IS_NULLSTR(territories[territory].name) )
	    	    continue;
		    
	        found = TRUE;
	        
	        sprintf( buf1, "%-20s", territories[territory].name );
	        send_to_char( buf1, ch );
	        if ( ++col % 3 == 0 )
    		    send_to_char( "\n\r", ch );
	    }

	    if ( !found )
	    {
	        send_to_char( "There are no territories defined.\n\r", 0 );
	        return;
	    }

	    if ( col % 3 != 0 )
	        send_to_char( "\n\r", ch );
	    return;
    } 
    else if ( !str_prefix(arg1, "save") )
    {
	    save_territories();
	    send_to_char( "OK.\n\r", ch );
	    return;
    }

    argument = one_argument( argument, arg2 );
    
    if ( arg2[0] == '\0' )
    {
	    do_territory( ch, "" );
	    return;
    }

    territory = 0;
    while ( 1 )
    {
	    if ( territory == MAX_TERRITORIES )
	    {
	        if ( !str_prefix(arg2, "create") )
    		    send_to_char( "Maximum territory limit reached.\n\r", ch );
	        else
	    	    send_to_char( "Territory not found.\n\r", ch );
	        return;
	    }

	    if ( !str_prefix(arg2, "create")
	    &&   IS_NULLSTR(territories[territory].name) )
	        break;

	    if ( !IS_NULLSTR(territories[territory].name)
	    &&   !str_prefix(arg1, territories[territory].name) )
	        break;
	    	    
	    territory++;
    }
	        
    if ( !str_cmp(arg2, "create") )
    {
	    memset( &territories[territory], 0, sizeof(struct territory_data) );

	    territories[territory].name = str_dup( arg1 );
	    
	    territories[territory].social_classes[0].male	= str_dup( "slave" );
	    territories[territory].social_classes[0].female	= str_dup( "slave" );
	    territories[territory].social_classes[MAX_SOCIAL_CLASS / 4].male
							    = str_dup( "commoner" );
	    territories[territory].social_classes[MAX_SOCIAL_CLASS / 4].female
							    = str_dup( "commoner" );
	    territories[territory].social_classes[MAX_SOCIAL_CLASS - 1].male
							    = str_dup( "king" );
	    territories[territory].social_classes[MAX_SOCIAL_CLASS - 1].female
							    = str_dup( "queen" );
	    
	    send_to_char( "OK.\n\r", ch );
	    terrListDirty = TRUE;
	    return;
    }
    else if ( !str_prefix(arg2, "display") )
    {
	    AREA_DATA *pArea;
	    BUFFER *output;
	    bool found;
	    int col, i;
	    
	    output = new_buf();
	    
	    sprintf( buf1, "%s:\n\r"
	                   "  Crimes: %s\n\r"
                           "  Jail:   %d\n\r"
	                   "  Areas:\n\r",
	             territories[territory].name,
	             crime_bit_name(territories[territory].crimes),
                     territories[territory].jail );

	    add_buf( output, buf1 );

	    col = 0;
	    found = FALSE;
	    strcpy( buf1, "    " );
	    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    {
	        if ( pArea->territory != territory )
    		    continue;
	        
	        found = TRUE;
	        
	        sprintf( buf2, "%-25s", pArea->name );
	        strcat( buf1, buf2 );

	        if ( ++col % 3 == 0 )
	        {
		        strcat( buf1, "\n\r" );
		        add_buf( output, buf1 );
		        
		        strcpy( buf1, "    " );
	        }
	    }

	    if ( col % 3 != 0 && found )
	    {
	        strcat( buf1, "\n\r" );
	        add_buf( output, buf1 );
	    }
	
	    add_buf( output, "Social classes:\n\r" );	

	    for ( i = 0; i < MAX_SOCIAL_CLASS; i++ )
	    {
	        if ( !IS_NULLSTR(territories[territory].social_classes[i].male) )
	        {
		        sprintf( buf1, "    %2d %-20s  %-20s\n\r", i,
		            territories[territory].social_classes[i].male,
		            territories[territory].social_classes[i].female );
		        add_buf( output, buf1 );
	        }
	    }
	
	    add_buf( output, "\n\r" );
	    
	    page_to_char( buf_string(output), ch );
	    free_buf(output);
	    return;
    }
    else if ( !str_prefix(arg2, "add") )
    {
	    argument = one_argument( argument, arg2 );

	    if ( arg2[0] == '\0' || argument[0] == '\0' )
	    {
	        do_territory( ch, "" );
	        return;
	    }
	
	    if ( !str_prefix(arg2, "area") )
	    {
	        AREA_DATA *pArea;
	        
	        for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	        {
		        if ( !str_prefix(argument, pArea->name) )
		        {
                            pArea->territory = territory;
		            send_to_char( "OK.\n\r", ch );
		            terrListDirty = TRUE;
		            return;
		        }
	        }
	        
	        send_to_char( "Area not found.\n\r", ch );
	        return;
	    }
	    else if ( !str_prefix(arg2, "social") )
	    {
	        int level;
	        
	        argument = one_argument( argument, arg2 );
	        
	        if ( arg2[0] == '\0'
	        ||   !is_number(arg2) )
	        {
		        do_territory( ch, "" );
		        return;
	        }
	        
	        level = atoi( arg2 );
	        
	        if ( level < 0 || level >= MAX_SOCIAL_CLASS )
	        {
		        sprintf( buf1, "Level must lie from 0 to %d.\n\r", MAX_SOCIAL_CLASS - 1 );
		        send_to_char( buf1, ch );
		        return;
	        }
	        
	        argument = cased_one_argument( argument, arg1 );
	        argument = cased_one_argument( argument, arg2 );
	    
	        if ( arg1[0] == '\0' || arg2[0] == '\0' )
	        {
		        do_territory( ch, "" );
		        return;
	        }
	    
	        free_string( territories[territory].social_classes[level].male );
	        free_string( territories[territory].social_classes[level].female );
	        
	        territories[territory].social_classes[level].male = str_dup( arg1 );
	        territories[territory].social_classes[level].female = str_dup( arg2 );
	        
	        send_to_char( "OK.\n\r", ch );
	        terrListDirty = TRUE;
	        return;
	    }
	    else
	    {
	        do_territory( ch, "" );
	        return;
	    }
    }
    else if ( !str_prefix(arg2, "remove") )
    {
	    argument = one_argument( argument, arg2 );
	    
	    if ( arg2[0] == '\0' || argument[0] == '\0' )
	    {
	        do_territory( ch, "" );
	        return;
	    }
	    
	    if ( !str_prefix(arg2, "area") )
	    {
	        AREA_DATA *pArea;
	        
	        for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	        {
		        if ( !str_prefix(argument, pArea->name) )
		        {
		            if ( pArea->territory == 0 )
		            {
			            send_to_char( "You can't remove an area from the default territory.\n\r", ch );
			            return;
		            }
		            
		            pArea->territory = 0;
		            send_to_char( "OK.\n\r", ch );
		            terrListDirty = TRUE;
		            return;
		        }
	        }
	    
	        send_to_char( "Area not found.\n\r", ch );
	        return;
	    }
	    else if ( !str_prefix(arg2, "social") )
	    {
	        int level;
	        
	        argument = one_argument( argument, arg2 );
	        
	        if ( arg2[0] == '\0'
	        ||   !is_number(arg2) )
	        {
		        do_territory( ch, "" );
		        return;
	        }
	        
	        level = atoi( arg2 );
	        
	        if ( level < 0 || level >= MAX_SOCIAL_CLASS )
	        {
		        sprintf( buf1, "Level must lie from 0 to %d.\n\r", MAX_SOCIAL_CLASS - 1 );
		        send_to_char( buf1, ch );
		        return;
	        }
	    	    
	        free_string( territories[territory].social_classes[level].male );
	        free_string( territories[territory].social_classes[level].female );
	        territories[territory].social_classes[level].male	= str_dup( "" );
	        territories[territory].social_classes[level].female	= str_dup( "" );
	        
	        send_to_char( "OK.\n\r", ch );
	        terrListDirty = TRUE;
	        return;
	    }
	    else
	    {
	        do_territory( ch, "" );
	        return;
	    }
    }
    else if ( !str_prefix(arg2, "delete") )
    {
	    AREA_DATA *pArea;
	    int i;

	    if ( territory == 0 )
	    {
	        send_to_char( "You can't remove the default area.\n\r", ch );
	        return;
	    }
	    
	    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    {
	        if ( pArea->territory == territory )
		    pArea->territory = 0;
	    }
	    
	    for ( i = 0; i < MAX_SOCIAL_CLASS; i++ )
	    {
	        free_string( territories[territory].social_classes[i].male );
	        free_string( territories[territory].social_classes[i].female );
	    }
	
	    free_string( territories[territory].name );
	    territories[territory].name = str_dup( "" );
	    
	    send_to_char( "OK.\n\r", ch );
	    terrListDirty = TRUE;
	    return;
    }
    else if ( !str_prefix(arg2, "crimes") )
    {
	    long flags;
	    char type;
	    int pos;

	    type = argument[0];
	    if ( type == '=' || type == '-' || type == '+' )
	        argument++;


	    flags = 0;
	    for ( ; ; )
	    {
	        argument = one_argument( argument, arg1 );
	        if ( arg1[0] == '\0' )
		    break;
	    
	        pos = flag_lookup( arg1, crime_table );
	        if ( pos == 0 )
	        {
		        sprintf( buf1, "Bad flag %s.\n\r", arg1 );
		        send_to_char( buf1, ch );
		        return;
	        }
	    
	        SET_BIT( flags, pos );
	    }

	    for ( pos = 0; crime_table[pos].name != NULL; pos++ )
	    {
            if ( !crime_table[pos].settable && IS_SET(flags, crime_table[pos].bit) )
	        {
		        sprintf( buf1, "Flag cannot be modified: %s.\n\r", 
		            crime_table[pos].name );
		        send_to_char( buf1, ch );
		        return;
	        }
	    }
	
	    switch ( type )
	    {
	    case '+':
	        SET_BIT( territories[territory].crimes, flags );
	        break;
	    case '-':
	        REMOVE_BIT( territories[territory].crimes, flags );
	        break;
	    case '=':
	        territories[territory].crimes = flags;
	        break;
	    default:
	        territories[territory].crimes = territories[territory].crimes ^ flags;
	    }
	    
            send_to_char( "OK.\n\r", ch );
	    terrListDirty = TRUE;
	    return;
    }
    else if ( !str_prefix(arg2, "jail") )
    {
	    ROOM_INDEX_DATA *jail;
	    int num;
	    
	    if ( !is_number(argument) )
	    {
	        send_to_char( "The jail vnum must be numeric.\n\r", ch );
	        return;
	    }
	    
	    num = atoi( argument );
	    if ( (jail = get_room_index(num)) == NULL )
	    {
	        send_to_char( "No such room.\n\r", ch );
	        return;
	    }
	
	    if ( jail->area->territory != territory )
	    {
	        send_to_char( "That room doesn't lie within that territory.\n\r", ch );
	        return;
	    }
	    
	    territories[territory].jail = num;
	    
	    send_to_char( "OK.\n\r", ch );
	    terrListDirty = TRUE;
	    return;
    } 
      
    do_territory( ch, "" );
    return;
}


char *get_social_class( CHAR_DATA *ch, int territory )
{
    int i;
    
    if ( IS_NPC(ch) )
	return "none";

    i = ch->pcdata->social_classes[ territory ];
    
    for ( ; i >= 0; i-- )
    {
	if ( IS_NULLSTR(territories[territory].social_classes[i].male) )
	    continue;

	if ( ch->sex == 2 )
	    return territories[territory].social_classes[i].female;
	else
	    return territories[territory].social_classes[i].male;
    }
    
    return "NONE";
}


int territory_lookup( const char *name )
{
    int territory;

    for ( territory = 0; territory < MAX_TERRITORIES; territory++ )
    {
	if ( !IS_NULLSTR(territories[territory].name)
	&&   LOWER(name[0]) == LOWER(territories[territory].name[0])
	&&   !str_prefix(name, territories[territory].name) )
	    return territory;
    }

    return -1;
}


bool is_killer( CHAR_DATA *ch )
{
    int territory;
    CRIME_DATA *crime;
    
    if ( IS_NPC(ch)
    ||   ch->in_room == NULL
    ||   (territory = ch->in_room->area->territory) == 0
    ||   (crime = ch->pcdata->crimes[territory]) == NULL )
	return FALSE;

    for ( ; crime != NULL; crime = crime->next )
    {
	if ( crime->crime == CRIME_PKILLING )
	    return TRUE;
    }

    return FALSE;
}


bool is_thief( CHAR_DATA *ch )
{
    int territory;
    CRIME_DATA *crime;
    
    if ( IS_NPC(ch)
    ||   ch->in_room == NULL
    ||   (territory = ch->in_room->area->territory) == 0
    ||   (crime = ch->pcdata->crimes[territory]) == NULL )
	return FALSE;

    for ( ; crime != NULL; crime = crime->next )
    {
	if ( crime->crime == CRIME_PSTEALING )
	    return TRUE;
    }

    return FALSE;
}


void make_criminal( CHAR_DATA *ch, long crime, CHAR_DATA *victim )
{
    char buf[MAX_STRING_LENGTH];
    CRIME_DATA *pCrime;
    NOTE_DATA *pnote;
    int territory;
    char *strtime;
    
    if ( IS_NPC(ch)
    ||   (territory = ch->in_room->area->territory) == 0 )
	    return;

    pCrime                          = new_crime();
    pCrime->crime                   = crime;
    pCrime->victim                  = str_dup( PERS_TRUE(victim) );
    
    pCrime->next                    = ch->pcdata->crimes[territory];
    ch->pcdata->crimes[territory]   = pCrime;

    strtime                         = ctime( &current_time );
    strtime[strlen(strtime)-1]      = '\0';

    sprintf( buf, 
        "%s is now wanted in %s for %s %s.\n\r",
        ch->name,
        territories[territory].name,
        crime == CRIME_PKILLING ? "the murder of" : "stealing from",
        PERS_TRUE(victim) );

    pnote               = new_note();
    pnote->next         = NULL;
    pnote->type         = NOTE_NOTE;
    pnote->sender       = str_dup( "Fionavar" );
    pnote->to_list      = str_dup( "justicar" );
    pnote->date         = strtime;
    pnote->date_stamp   = current_time;
    pnote->subject      = str_dup( "Crime Notification." );
    pnote->text         = str_dup( buf );
    
    append_note( pnote );
}


bool is_criminal( CHAR_DATA *victim, bool anywhere )
{
    int territory;
    
    if ( IS_NPC(victim) )
	    return FALSE;
    
    if ( anywhere )
    {
	    for ( territory = 1; territory < MAX_TERRITORIES; territory++ )
	    {
	        if ( IS_NULLSTR(territories[territory].name) )
		        continue;
	    
	        if ( victim->pcdata->crimes[territory] != NULL )
		        return TRUE;
	    }
	    
	    return FALSE;
    }
    
    if ( victim->in_room == NULL 
    ||   (territory = victim->in_room->area->territory) == 0 )
	    return FALSE;

    return victim->pcdata->crimes[territory] ? TRUE : FALSE;
}


char *vis_crime_names( int flags )
{
    static char buf[512];

    buf[0] = '\0';

    if (flags & CRIME_PKILLING)
	strcat(buf, "no player assault");
    if (flags & CRIME_PSTEALING)
    {
	if ( buf[0] != '\0' )
	    strcat( buf, ", " );
        strcat(buf, "no player theft");
    }
    if (flags & TERRITORY_JUSTICAR_ALIGNED)
    {
	if ( buf[0] != '\0' )
	    strcat( buf, ", " );
        strcat(buf, "justicar aligned");
    }

    return ( buf[0] != '\0' ) ? buf : "none";
}


void do_whereami( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    BUFFER *output;
    int territory;
    int i;

    territory = ch->in_room->area->territory;
	
    output = new_buf();

    sprintf( buf, "%sYou are in %s\n\r"
	          "%s---==========================================================================---\n\r"
	          "%sLaws: %s%s\n\r"
	          "%s---==========================================================================---\n\r"
		  "%sRanks / Social classes:%s\n\r",
	P_COLOR(ch, C_TITLES),
	territories[territory].name,
	P_COLOR(ch, C_BORDERS),
	P_COLOR(ch, C_TITLES),
	P_COLOR(ch, C_INFORMATION),
	vis_crime_names( territories[territory].crimes ),
	P_COLOR(ch, C_BORDERS),
	P_COLOR(ch, C_TITLES),
	P_COLOR(ch, C_INFORMATION) );
    add_buf( output, buf );

    for ( i = 0; i < MAX_SOCIAL_CLASS; i++ )
    {
	char *male = NULL, *female = NULL;

	if ( !IS_NULLSTR(territories[territory].social_classes[i].male)
	&&   str_cmp(territories[territory].social_classes[i].male, "NONE") )
	    male = territories[territory].social_classes[i].male;

	if ( !IS_NULLSTR(territories[territory].social_classes[i].female)
	&&   str_cmp(territories[territory].social_classes[i].female, "NONE") )
	    female = territories[territory].social_classes[i].female;

	if ( male || female )
	{
	    sprintf( buf, "  %-20s  %-20s\n\r",
		male != NULL ? male : "", 
		female != NULL ? female : "" );
	    add_buf( output, buf );
	}
    } 

    sprintf( buf, "%s\n\r", IS_SET(ch->comm, COMM_COLOR) ? COLOR_OFF : "" );
    add_buf( output, buf );
    page_to_char( buf_string(output), ch );
    free_buf( output );
}


void do_promote( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int territory;
    int new_rank;

    argument = one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
        send_to_char( "Promote whom?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They aren't here.\n\r", ch );
        return;
    }

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can use this command.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	send_to_char( "You can only promote other players.\n\r", ch );
	return;
    }

    if ( ch == victim )
    {
	send_to_char( "You can't do that!\n\r", ch );
	return;
    }

    territory = ch->in_room->area->territory;

    if ( ch->pcdata->social_classes[territory] < MAX_SOCIAL_CLASS - 1 )
    {
	send_to_char( "You are not the ruler of this territory.\n\r", ch );
	return;
    }
    
    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	for ( new_rank = victim->pcdata->social_classes[territory] + 1;
	      new_rank < MAX_SOCIAL_CLASS - 1; new_rank++ )
	{
	    const char *name;
	    if ( victim->sex == 2 )
		name = territories[territory].social_classes[new_rank].female;
	    else
		name = territories[territory].social_classes[new_rank].male;
	    if ( !IS_NULLSTR(name) && str_cmp(name, "NONE") )
		break;
	}

	if ( new_rank == MAX_SOCIAL_CLASS - 1 )
	{
	    send_to_char( "They can't be promoted.\n\r", ch );
	    return;
	}
    }
    else
    {
	bool found = FALSE;

	if ( !str_cmp(arg, "NONE") )
	{
	    send_to_char( "\"NONE\" is not a valid social class name.\n\r", ch );
	    return;
	}

	for ( new_rank = victim->pcdata->social_classes[territory] + 1;
	      new_rank < MAX_SOCIAL_CLASS; new_rank++ )
	{
	    const char *name;
	    if ( victim->sex == 2 )
		name = territories[territory].social_classes[new_rank].female;
	    else
		name = territories[territory].social_classes[new_rank].male;
	    if ( !IS_NULLSTR(name) && !str_cmp(name, arg) )
	    {
		found = TRUE;
		break;
	    }
	}

	if ( !found )
	{
	    send_to_char( "Social class not found.\n\r", ch );
	    return;
	}
    }

    if ( new_rank == MAX_SOCIAL_CLASS - 1 )
    {
	send_to_char( "You can't promote them that high.\n\r", ch );
	return;
    }

    sprintf( buf, "%s has promoted %s to %s:%s.", 
        ch->name, victim->name, territories[territory].name, 
	(victim->sex == 2)
	  ? territories[territory].social_classes[new_rank].female
	  : territories[territory].social_classes[new_rank].male );
    log_string( buf );
    sprintf( buf, "$N has promoted %s to %s:%s.",
        victim->name, territories[territory].name, 
	(victim->sex == 2)
	  ? territories[territory].social_classes[new_rank].female
	  : territories[territory].social_classes[new_rank].male );
    wiznet( buf, ch, NULL, WIZ_LEVELS, 0, 0 );

    victim->pcdata->social_classes[territory] = new_rank;
    sprintf( buf, "$n has promoted you to the rank of %s.\n\r",
	(victim->sex == 2)
	  ? territories[territory].social_classes[new_rank].female
	  : territories[territory].social_classes[new_rank].male );
    act( buf, ch, NULL, victim, TO_VICT );
    send_to_char( "OK.\n\r", ch );
}


void do_demote( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;
    int territory;
    int new_rank;

    argument = one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
        send_to_char( "Demote whom?\n\r", ch );
        return;
    }
    if ( (victim = get_char_room(ch, arg)) == NULL )
    {
        send_to_char( "They aren't here.\n\r", ch );
        return;
    }

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players can use this command.\n\r", ch );
	return;
    }

    if ( IS_NPC(victim) )
    {
	send_to_char( "You can only demote other players.\n\r", ch );
	return;
    }

    if ( ch == victim )
    {
	send_to_char( "You can't do that!\n\r", ch );
	return;
    }

    territory = ch->in_room->area->territory;

    if ( ch->pcdata->social_classes[territory] < MAX_SOCIAL_CLASS - 1 )
    {
	send_to_char( "You are not the ruler of this territory.\n\r", ch );
	return;
    }
    
    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	for ( new_rank = victim->pcdata->social_classes[territory] - 1;
	      new_rank >= 0; new_rank-- )
	{
	    const char *name;
	    if ( victim->sex == 2 )
		name = territories[territory].social_classes[new_rank].female;
	    else
		name = territories[territory].social_classes[new_rank].male;
	    if ( !IS_NULLSTR(name) && str_cmp(name, "NONE") )
		break;
	}

	if ( new_rank < 0 )
	{
	    send_to_char( "They can't be demoted.\n\r", ch );
	    return;
	}
    }
    else
    {
	bool found = FALSE;

	if ( !str_cmp(arg, "NONE") )
	{
	    send_to_char( "\"NONE\" is not a valid social class name.\n\r", ch );
	    return;
	}

	for ( new_rank = victim->pcdata->social_classes[territory] - 1;
	      new_rank >= 0; new_rank-- )
	{
	    const char *name;
	    if ( victim->sex == 2 )
		name = territories[territory].social_classes[new_rank].female;
	    else
		name = territories[territory].social_classes[new_rank].male;
	    if ( !IS_NULLSTR(name) && !str_cmp(name, arg) )
	    {
		found = TRUE;
		break;
	    }
	}

	if ( !found )
	{
	    send_to_char( "Social class not found.\n\r", ch );
	    return;
	}
    }

    sprintf( buf, "%s has demoted %s to %s:%s.", 
        ch->name, victim->name, territories[territory].name, 
	(victim->sex == 2)
	  ? territories[territory].social_classes[new_rank].female
	  : territories[territory].social_classes[new_rank].male );
    log_string( buf );
    sprintf( buf, "$N has demoted %s to %s:%s.",
        victim->name, territories[territory].name, 
	(victim->sex == 2)
	  ? territories[territory].social_classes[new_rank].female
	  : territories[territory].social_classes[new_rank].male );
    wiznet( buf, ch, NULL, WIZ_LEVELS, 0, 0 );

    victim->pcdata->social_classes[territory] = new_rank;
    sprintf( buf, "$n has demoted you to the rank of %s.\n\r",
	(victim->sex == 2)
	  ? territories[territory].social_classes[new_rank].female
	  : territories[territory].social_classes[new_rank].male );
    act( buf, ch, NULL, victim, TO_VICT );
    send_to_char( "OK.\n\r", ch );
}


bool has_territory_flag( CHAR_DATA *ch, int territory, long flag )
{
    if ( IS_NPC(ch) )
        return FALSE;
    if ( territory < 0
    ||   territory >= MAX_TERRITORIES
    ||   IS_NULLSTR(territories[territory].name) )
    {
        bug( "has_territory_flag: invalid territory specified %d.", territory );
        return FALSE;
    }
    if ( IS_SET(ch->pcdata->territory_flags[territory], flag) )
        return TRUE;
    return FALSE;
}
