/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/


#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>
#include "merc.h"
#include "recycle.h"


/* command procedures needed */
DECLARE_DO_FUN( do_emote );
DECLARE_DO_FUN( do_look );
DECLARE_DO_FUN( do_yell );
DECLARE_DO_FUN( do_sit );


extern ROOM_INDEX_DATA *room_index_hash[];
extern char * const 	dir_name[];


static BFS_QUEUE_DATA *	queue_head;
static BFS_QUEUE_DATA * queue_tail;
static int 		bfs_tag;


/*
 * Local function prototypes.
 */
CHAR_DATA *find_victim	args( ( CHAR_DATA *ch, ROOM_INDEX_DATA *room,
				void *target, int target_type ) );


void bfs_enqueue( ROOM_INDEX_DATA *room, int dir, int depth )
{
    BFS_QUEUE_DATA *curr;

    curr = new_bfs_data();
    curr->room  = room;
    curr->dir   = dir;
    curr->depth = depth;

    if ( queue_tail ) 
    {
        queue_tail->next = curr;
        queue_tail       = curr;
    } 
    else
        queue_head = queue_tail = curr;
}


void bfs_dequeue(void)
{
    BFS_QUEUE_DATA *curr;

    curr = queue_head;

    if ( !(queue_head = queue_head->next) )
        queue_tail = 0;
        
    free_bfs_data( curr );
}


void bfs_clear_queue(void)
{
    while (queue_head) bfs_dequeue();
}


int find_first_step( CHAR_DATA *ch, void *target, int target_type, int max_depth, 
                     bool cross_air, bool cross_water, bool cross_deep_water )
{
    EXIT_DATA *pexit;
    sh_int door;
    
    if ( ch->in_room == NULL )
	return BFS_ERROR;

    if ( find_victim(ch, ch->in_room, target, target_type) )
	return BFS_FOUND;

    /* clear marks first */
    if ( bfs_tag == INT_MAX )
    {
	ROOM_INDEX_DATA *current;
	int i;
	
	for ( i=0; i<MAX_KEY_HASH; i++ )
	    for (current = room_index_hash[i]; current; current=current->next)
		current->bfs_tag = 0;

	bfs_tag = 0;
    }
    
    bfs_tag++;

    ch->in_room->bfs_tag = bfs_tag;

    /* first, enqueue the first steps, saving which direction we're going. */
    for ( door = 0; door <= 11; door++ )
    {
	if ((pexit = ch->in_room->exit[door]) != NULL
	&&  pexit->u1.to_room != NULL
	&&  can_see_room(ch, pexit->u1.to_room) 
	&&  (pexit->u1.to_room->sector_type != SECT_AIR
	||   cross_air)
	&&  (pexit->u1.to_room->sector_type != SECT_WATER_SWIM
	||   cross_water)
	&&  (pexit->u1.to_room->sector_type != SECT_WATER_NOSWIM
	||   cross_deep_water)
	&&  !IS_SET(pexit->exit_info, EX_CLOSED) )
	{
	    pexit->u1.to_room->bfs_tag = bfs_tag;
	    bfs_enqueue( pexit->u1.to_room, door, 1 );
	}
    }
    
    /* now, do the classic BFS. */
    while ( queue_head ) 
    {
	CHAR_DATA *victim;

	if ( (victim = find_victim(ch, queue_head->room, target, target_type)) != NULL
	&&   (IS_NPC(ch)
	||    !IS_AFFECTED(victim, AFF_FLYING))
        &&   (!IS_AFFECTED2(victim, AFF2_TRACELESS_TRAVEL)
        ||    IS_AFFECTED(ch, AFF_DETECT_MAGIC)) )
	{
	    door = queue_head->dir;
	    bfs_clear_queue();
	    return door;
	}
	
	if ( queue_head->depth == max_depth )
	    bfs_dequeue();
	else
	{
	    for ( door = 0; door <= 11; door++ )
	    {
		if ((pexit = queue_head->room->exit[door]) != NULL
		&&  pexit->u1.to_room != NULL
		&&  pexit->u1.to_room->bfs_tag != bfs_tag
		&&  can_see_room(ch, pexit->u1.to_room) 
		&&  (pexit->u1.to_room->sector_type != SECT_AIR
		||   cross_air)
		&&  (pexit->u1.to_room->sector_type != SECT_WATER_SWIM
		||   cross_water)
		&&  (pexit->u1.to_room->sector_type != SECT_WATER_NOSWIM
		||   cross_deep_water)
		&&  !IS_SET(pexit->exit_info, EX_CLOSED) ) 
		{
		    pexit->u1.to_room->bfs_tag = bfs_tag;
		    bfs_enqueue( pexit->u1.to_room, queue_head->dir, 
			queue_head->depth + 1 );
		}
	    }
	    bfs_dequeue();
	}
    }

    return BFS_NO_PATH;
}


void do_track( CHAR_DATA *ch, char *argument )
{
    EXIT_DATA *pexit;
    int chance;
    int door;

    if ( (chance = get_skill(ch,gsn_track)) == 0
    ||   (!IS_NPC(ch) 
    &&    ch->level < skill_level(ch, gsn_track)) )
    {
	send_to_char("You don't know how.\n\r",ch);
	return;
    }

    chance += get_curr_stat( ch, STAT_PER ) * 4/10 - 20;

    if ( argument[0] == '\0' ) 
    {
	send_to_char("Whom are you trying to track?\r\n", ch);
	return;
    }
          
    door = find_first_step( 
        ch, argument, TRACK_NAME, 
        effective_level(ch, gsn_track) / 5 + 5, 
        FALSE, FALSE, FALSE );

    switch (door) 
    {
    case BFS_ERROR:
	send_to_char( "Hmm.. something seems to be wrong.\r\n", ch );
	break;
    case BFS_FOUND:
	send_to_char( "You're already in the same room!!\r\n", ch );
	break;
    case BFS_NO_PATH:
	act( "You can't seem to sense a trail from here.",
	    ch, NULL, NULL, TO_CHAR );
	break;
    default:
	if ( chance < number_percent() )
	{
	    int tries = 0;

	    /* Randomize on a failure. */
	    do {
		door = number_range( 0, 5 );
	        
		if ( tries++ == 20 )
		{
		    send_to_char( "You seem to have completely lost the trail.\n\r", ch );
		    check_improve( ch, gsn_track, FALSE, 2, TRUE );
		    WAIT_STATE( ch, skill_table[gsn_track].beats * 2 );\
		    return;
		}
	    } while ((pexit = ch->in_room->exit[door]) != NULL
	    &&  pexit->u1.to_room != NULL
	    &&  can_see_room(ch, pexit->u1.to_room) 
	    &&  !IS_SET(pexit->exit_info, EX_CLOSED) );
	}
        
	act( "You sense a trail $t from here!", ch, dir_name[door], NULL, TO_CHAR );
	check_improve( ch, gsn_track, TRUE, 2, TRUE );
	WAIT_STATE( ch, skill_table[gsn_track].beats * 2 ); 
	break;
    }
    
    return;
}


void hunt( CHAR_DATA *ch )
{
    MEM_DATA *memory;
    
    if ( !IS_AWAKE(ch) || ch->fighting != NULL )
	return;

    for ( memory = ch->memory; memory != NULL; memory = memory->next )
    {
	if ( memory->reaction == MEM_HOSTILE )
	{
	    CHAR_DATA *victim;
	    EXIT_DATA *pexit;
	    int door;

	    if ( (victim = find_victim(ch, ch->in_room, (void *) memory->id, TRACK_PC_ID)) == NULL
	    &&   !IS_SET(ch->act, ACT_SENTINEL) )
	    {
                int distance;
                if ( IS_NPC(ch) && IS_SET(ch->off_flags, OFF_NASTY_HUNTER) )
                    distance = 1000;
                else
                    distance = effective_level( ch, gsn_track ) / 5 + 5;
		door = find_first_step( ch, (void *) memory->id, TRACK_PC_ID,
		    distance,
		    IS_AFFECTED(ch, AFF_FLYING) ? TRUE :  FALSE,
		    TRUE,
		    IS_AFFECTED(ch, AFF_SWIM) ? TRUE :  FALSE );
	
		switch ( door )
		{
		case BFS_ERROR:
		case BFS_NO_PATH:
                    if ( IS_NPC(ch)
                    &&   IS_SET(ch->off_flags, OFF_NASTY_HUNTER)
                    &&   number_bits(2) == 0 )
                    {
                        ROOM_INDEX_DATA *pRoomIndex = get_random_room( ch );
                        if ( pRoomIndex != NULL )
                        {
                            char_from_room( ch, TRUE );
                            char_to_room( ch, pRoomIndex );
                            do_look( ch, "auto" );
                        }
                    }
		    break;
	
		case BFS_FOUND:
		    victim = find_victim( ch, ch->in_room, (void *) memory->id, TRACK_PC_ID );
		    break;
	
		default:
		    pexit = ch->in_room->exit[ door ];
		
		    if ( pexit->u1.to_room == NULL
		    ||   !can_see_room(ch, pexit->u1.to_room)
		    ||   IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB) )
			break;
		
		    if ( IS_SET(ch->act, ACT_STAY_AREA)
		    &&   ch->in_room->area != pexit->u1.to_room->area )
			break;
		
		    if ( IS_SET(ch->act, ACT_OUTDOORS)
		    &&   IS_SET(pexit->u1.to_room->room_flags, ROOM_INDOORS) )
			break;
		
		    if ( IS_SET(ch->act, ACT_INDOORS)
		    &&   !IS_SET(pexit->u1.to_room->room_flags, ROOM_INDOORS) )
			break;
		
		    move_char( ch, door, TRUE );
		    victim = find_victim( ch, ch->in_room, (void *) memory->id, TRACK_PC_ID );
		    break;
		}
	    }
	    
	    if ( victim != NULL )
	    {
		if ( IS_SET(CHAR_FORM(ch), FORM_SENTIENT) )
		{
		    char buf[MAX_STRING_LENGTH];
		    bool no_shout = FALSE;

		    sprintf( buf, "I've got you now %s", PERS(victim, ch) );
		    if ( IS_SET(ch->comm, COMM_NOSHOUT) )
			no_shout = TRUE;
		    REMOVE_BIT( ch->comm, COMM_NOSHOUT );
		    do_yell( ch, buf );
		    if ( no_shout )
			SET_BIT( ch->comm, COMM_NOSHOUT );
		}
		else
		    do_emote( ch, "screams and attacks!" );

		multi_hit( ch, victim, TYPE_UNDEFINED );
		remove_memory( ch, memory );
		break;
	    }
	}
    }
}


void answer_call( CHAR_DATA *ch )
{
    MEM_DATA *memory;
    
    if ( !IS_AWAKE(ch) || ch->fighting != NULL )
	return;
    
    for ( memory = ch->memory; memory != NULL; memory = memory->next )
    {
	if ( memory->reaction == MEM_SUMMONED_PC
	||   memory->reaction == MEM_SUMMONED_NPC )
	{
	    CHAR_DATA *victim;
	    EXIT_DATA *pexit;
	    int victim_type;
	    int door;

	    victim_type = (memory->reaction == MEM_SUMMONED_PC)
	                ? TRACK_PC_ID_BLIND : TRACK_NPC_ID_BLIND;

	    if ( (victim = find_victim(ch, ch->in_room, (void *) memory->id, 
	                               victim_type)) == NULL
	    &&   !IS_SET(ch->act, ACT_SENTINEL) )
	    {
		door = find_first_step( 
		    ch, (void *) memory->id, victim_type, 20,
		    IS_AFFECTED(ch, AFF_FLYING) ? TRUE : FALSE, 
		    TRUE, 
		    IS_AFFECTED(ch, AFF_SWIM) ? TRUE : FALSE );

		switch ( door )
		{
		case BFS_ERROR:
		case BFS_NO_PATH:
		    break;

		case BFS_FOUND:
		    victim = find_victim( ch, ch->in_room, (void *) memory->id,
		                          victim_type );
		    break;

		default:
		    pexit = ch->in_room->exit[ door ];

		    if ( pexit->u1.to_room == NULL
		    ||   !can_see_room(ch, pexit->u1.to_room)
		    ||   IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB)
		    ||   (IS_SET(pexit->u1.to_room->room_flags, ROOM_LAW)
		    &&    IS_SET(ch->act, ACT_AGGRESSIVE)) )
			break;

		    if ( IS_SET(ch->act, ACT_STAY_AREA)
		    &&   ch->in_room->area != pexit->u1.to_room->area )
			break;

		    if ( number_bits(0) == 0 )
		    {
			if ( door <= 3 )
			    act( "$n raises $s head, looking briefly to the %t.",
			         ch, dir_name[door], NULL, TO_ROOM );
			else
			    act( "$n raises $s head, looking briefly $twards.",
			         ch, dir_name[door], NULL, TO_ROOM );
		    }
		    move_char( ch, door, TRUE );
		    victim = find_victim( ch, ch->in_room, (void *) memory->id,
		                          victim_type );
		    break;
		}
	    }
	    
	    if ( victim != NULL )
	    {
		do_sit( ch, "" );
		if ( IS_SET(CHAR_PARTS(ch), PART_HEAD) )
		{
		    act( "$n raises $s head and looks at you.",
		         ch, NULL, victim, TO_VICT );
		    act( "$n raises $s head and looks at $N.",
		         ch, NULL, victim, TO_NOTVICT );
		}
		remove_memory( ch, memory );
		break;
	    }
	}
    }
}


void fire_track( CHAR_DATA *ch )
{
    CHAR_DATA *victim;
    AFFECT_DATA *paf;

    if ( !IS_AWAKE(ch) || ch->fighting != NULL )
	return;
    
    paf = affect_find( ch->affected, gsn_fire_track );
    if ( paf == NULL )
    {
        bug( "fire_track: unable to find fire_track paf data on %s.", (long) ch->short_descr );
        return;
    }
    if ( IS_NULLSTR(paf->name) )
    {
        bug( "fire_track: paf->name is NULL on %s.", (long) ch->short_descr );
        return;
    }

    if ( (victim = find_victim(ch, ch->in_room, (void *) paf->name, 
	                       TRACK_NAME)) == NULL )
    {
	EXIT_DATA *pexit;
        int door;

	door = find_first_step(
	    ch, paf->name, TRACK_NAME, paf->level,
	    TRUE, TRUE, FALSE );

	switch ( door )
	{
	case BFS_ERROR:
	case BFS_NO_PATH:
	    break;

	case BFS_FOUND:
	    victim = find_victim( ch, ch->in_room, paf->name,
		                  TRACK_NAME );
	    break;

	default:
	    pexit = ch->in_room->exit[ door ];

	    if ( pexit->u1.to_room == NULL
	    ||   !can_see_room(ch, pexit->u1.to_room)
	    ||   IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB)
	    ||   (IS_SET(pexit->u1.to_room->room_flags, ROOM_LAW)
	    &&    IS_SET(ch->act, ACT_AGGRESSIVE)) )
		break;

	    move_char( ch, door, TRUE );
	    victim = find_victim( ch, ch->in_room, paf->name,
		                  TRACK_NAME );
	    break;
	}
    }
	    
    if ( victim != NULL )
    {
        act( "$n circles $N in a slow bouncing dance.", ch, NULL, victim, TO_ROOM );
        act( "$n circles you in a slow bouncing dance.", ch, NULL, victim, TO_VICT );
    }
}


CHAR_DATA *find_victim( CHAR_DATA *ch, ROOM_INDEX_DATA *room, void *target, int target_type )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *rch;
    int number;
    int count;

    switch ( target_type )
    {
	default:
	    bug( "find_victim: invalid target type (%d)", target_type );
	    return NULL;
	
	case TRACK_NAME:
	    number = number_argument( (char *) target, arg );
	    count  = 0;

            for ( rch = room->people;
                  rch != NULL;
                  rch = rch->next_in_room )
            {
		if ( !can_see(ch, rch) || !is_name(arg, CHAR_NAME(rch)) )
		    continue;
		if ( ++count == number )
		    return rch;
	    }
	    
	case TRACK_PC_ID:
	case TRACK_NPC_ID:
	case TRACK_PC_ID_BLIND:
	case TRACK_NPC_ID_BLIND:
	    for ( rch = room->people; 
	          rch != NULL; 
	          rch = rch->next_in_room )
	    {
		if ( target_type == TRACK_PC_ID
		&&   !IS_NPC(rch)
		&&   rch->id == (long) target
		&&   can_see(ch, rch) )
		    return rch;

		if ( target_type == TRACK_PC_ID_BLIND
		&&   !IS_NPC(rch)
		&&   rch->id == (long) target )
		    return rch;

		if ( target_type == TRACK_NPC_ID
		&&   IS_NPC(rch)
		&&   rch->id == (long) target
		&&   can_see(ch, rch) )
		    return rch;

		if ( target_type == TRACK_NPC_ID_BLIND
		&&   IS_NPC(rch)
		&&   rch->id == (long) target )
		    return rch;
	    }
	    return NULL;
    }
}



